@@ -57,7 +57,6 @@ import javax.swing.border.Border;
 import javax.swing.border.CompoundBorder;
 import javax.swing.event.TreeExpansionEvent;
 import javax.swing.event.TreeExpansionListener;
-import javax.swing.plaf.basic.BasicButtonListener;
 import javax.swing.table.JTableHeader;
 import javax.swing.tree.TreeModel;
 import javax.swing.tree.TreePath;
@@ -72,6 +71,7 @@ public final class UIUtils {
     //~ Static fields/initializers -----------------------------------------------------------------------------------------------
 
     // Used to mark explicit expand/collapse on JTree which shouldn't be handled by automatic expander
+    public static final String PROP_AUTO_EXPANDING = "auto_expanding"; // NOI18N
     public static final String PROP_EXPANSION_TRANSACTION = "expansion_transaction"; // NOI18N
     public static Dimension DIMENSION_SMALLEST = new Dimension(0, 0);
 
@@ -94,14 +94,12 @@ public final class UIUtils {
     }
     
     public static JPanel createFillerPanel() {
-        JPanel fillerPanel = new JPanel(new FlowLayout(0, 0, FlowLayout.LEADING)) {
+        JPanel fillerPanel = new JPanel(null) {
             public Dimension getPreferredSize() {
                 return DIMENSION_SMALLEST;
             }
         };
-
         fillerPanel.setOpaque(false);
-
         return fillerPanel;
     }
 
@@ -116,7 +114,7 @@ public final class UIUtils {
     }
     
     public static JSeparator createHorizontalLine(Color background) {
-        final boolean customPaint = isNimbus();
+        final boolean customPaint = isNimbus() || isAquaLookAndFeel();
         JSeparator separator = new JSeparator() {
             public Dimension getMaximumSize() {
                 return new Dimension(super.getMaximumSize().width, 1);
@@ -170,10 +168,12 @@ public final class UIUtils {
     }
     
     public static Color getDisabledForeground(Color c) {
-        if (isNimbusLookAndFeel()) return UIManager.getColor("nimbusDisabledText").darker(); //NOI18N
+        Color b = c.brighter();
+        if (c.getRGB() == b.getRGB()) return b; // Selection foreground
+        else if (isNimbusLookAndFeel()) return UIManager.getColor("nimbusDisabledText").darker(); //NOI18N
         else if (isMetalLookAndFeel()) return UIManager.getColor("Label.disabledForeground"); //NOI18N
-        else if (Color.BLACK.equals(c)) return Color.GRAY;
-        else return c.brighter();
+        else if (Color.BLACK.getRGB() == c.getRGB()) return Color.GRAY;
+        else return b;
     }
 
     public static int getDefaultRowHeight() {
@@ -359,9 +359,23 @@ public final class UIUtils {
         
         return profilerResultsBackground;
     }
+    
+    private static Boolean darkResultsBackground;
+    
+    public static boolean isDarkResultsBackground() {
+        if (darkResultsBackground == null) {
+            Color c = getProfilerResultsBackground();
+            int b = (int)(0.3 * c.getRed() + 0.59 * c.getGreen() + 0.11 * c.getBlue());
+            darkResultsBackground = b < 85;
+        }
+        
+        return darkResultsBackground;
+    }
 
     /** Determines if current L&F is Windows Classic LookAndFeel */
     public static boolean isWindowsClassicLookAndFeel() {
+        if ("Windows Classic".equals(UIManager.getLookAndFeel().getName())) return true; //NOI18N
+        
         if (!isWindowsLookAndFeel()) {
             return false;
         }
@@ -389,6 +403,17 @@ public final class UIUtils {
 
         return ((xpThemeActiveOS) && (!xpThemeDisabled));
     }
+    
+    public static boolean isWindowsModernLookAndFeel() {
+        if (!isWindowsXPLookAndFeel()) return false;
+        String osName = System.getProperty("os.name"); // NOI18N
+        return osName != null && (osName.contains("Windows 8") || osName.contains("Windows 10")); // NOI18N
+    }
+    
+    public static boolean isOracleLookAndFeel() {
+        // is current L&F some kind of WindowsLookAndFeel?
+        return UIManager.getLookAndFeel().getID().contains("Oracle"); //NOI18N
+    }
 
     /** Checks give TreePath for the last node, and if it ends with a node with just one child,
      * it keeps expanding further.
@@ -499,16 +524,6 @@ public final class UIUtils {
         }
     }
 
-    public static BufferedImage createScreenshot(Component component) {
-        if (component instanceof JScrollPane) {
-            JScrollPane scrollPane = (JScrollPane) component;
-
-            return createComponentScreenshot(scrollPane.getViewport());
-        } else {
-            return createComponentScreenshot(component);
-        }
-    }
-
     public static void ensureMinimumSize(Component comp) {
         comp = getParentWindow(comp);
 
@@ -596,8 +611,13 @@ public final class UIUtils {
                     TreePath path = event.getPath();
                     JTree tree = (JTree) event.getSource();
                     internalChange = true;
-                    autoExpand(tree, path, MAX_TREE_AUTOEXPAND_LINES, maxChildToExpand, dontExpandToLeafs);
-                    internalChange = false;
+                    tree.putClientProperty(PROP_AUTO_EXPANDING, Boolean.TRUE);
+                    try {
+                        autoExpand(tree, path, MAX_TREE_AUTOEXPAND_LINES, maxChildToExpand, dontExpandToLeafs);
+                    } finally {
+                        tree.putClientProperty(PROP_AUTO_EXPANDING, null);
+                        internalChange = false;
+                    }
                 }
             });
     }
@@ -632,36 +652,34 @@ public final class UIUtils {
     
     public static Color getDisabledLineColor() {
         if (disabledLineColor == null) {
-            disabledLineColor = UIManager.getColor("Label.disabledForeground"); // NOI18N
+            disabledLineColor = UIManager.getColor(isAquaLookAndFeel() ? "controlShadow" : // NOI18N
+                                                   "Label.disabledForeground"); // NOI18N
             if (disabledLineColor == null)
                 disabledLineColor = UIManager.getColor("Label.disabledText"); // NOI18N
-            if (disabledLineColor == null) disabledLineColor = Color.GRAY;
+            if (disabledLineColor == null || disabledLineColor.equals(getProfilerResultsBackground()))
+                disabledLineColor = Color.GRAY;
         }
         return disabledLineColor;
-    } 
-
-    private static BufferedImage createComponentScreenshot(final Component component) {
-        final BufferedImage[] result = new BufferedImage[1];
-
-        final Runnable screenshotPerformer = new Runnable() {
-            public void run() {
-                if (component instanceof JTable
-                        || (component instanceof JViewport && ((JViewport) component).getView() instanceof JTable)) {
-                    result[0] = createTableScreenshot(component);
-                } else {
-                    result[0] = createGeneralComponentScreenshot(component);
-                }
-            }
-        };
+    }    
+    
+    public static BufferedImage createScreenshot(Component component) {
+        assert SwingUtilities.isEventDispatchThread();
         
-        try {
-            if (SwingUtilities.isEventDispatchThread()) screenshotPerformer.run();
-            else SwingUtilities.invokeAndWait(screenshotPerformer);
-        } catch (Exception e) {
-            return null;
+        if (component instanceof JScrollPane) {
+            JScrollPane scrollPane = (JScrollPane) component;
+            return createComponentScreenshot(scrollPane.getViewport());
+        } else {
+            return createComponentScreenshot(component);
+        }
+    }
+
+    private static BufferedImage createComponentScreenshot(Component component) {
+        if (component instanceof JTable || (component instanceof JViewport &&
+                                           ((JViewport) component).getView() instanceof JTable)) {
+            return createTableScreenshot(component);
+        } else {
+            return createGeneralComponentScreenshot(component);
         }
-        
-        return result[0];
     }
 
     private static BufferedImage createGeneralComponentScreenshot(Component component) {
@@ -729,40 +747,11 @@ public final class UIUtils {
 
         BufferedImage tableScreenshot = new BufferedImage(sourceSize.width, tableHeaderSize.height + sourceSize.height,
                                                           BufferedImage.TYPE_INT_RGB);
-        final Graphics tableScreenshotGraphics = tableScreenshot.getGraphics();
-
-        // Component.printAll has to run in AWT Thread to print component contents correctly
-        if (SwingUtilities.isEventDispatchThread()) {
-            tableHeader.printAll(tableScreenshotGraphics);
-        } else {
-            try {
-                SwingUtilities.invokeAndWait(new Runnable() {
-                        public void run() {
-                            tableHeader.printAll(tableScreenshotGraphics);
-                        }
-                    });
-            } catch (Exception e) {
-            }
-        }
-
+        Graphics tableScreenshotGraphics = tableScreenshot.getGraphics();
+        tableHeader.printAll(tableScreenshotGraphics);
         tableScreenshotGraphics.translate(0, tableHeaderSize.height);
-
-        final Component printSrc = source;
-
-        // Component.printAll has to run in AWT Thread to print component contents correctly
-        if (SwingUtilities.isEventDispatchThread()) {
-            printSrc.printAll(tableScreenshotGraphics);
-        } else {
-            try {
-                SwingUtilities.invokeAndWait(new Runnable() {
-                        public void run() {
-                            printSrc.printAll(tableScreenshotGraphics);
-                        }
-                    });
-            } catch (Exception e) {
-            }
-        }
-
+        source.printAll(tableScreenshotGraphics);
         return tableScreenshot;
     }
+    
 }
