@@ -49,19 +49,17 @@ import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.StringSelection;
 import java.awt.datatransfer.Transferable;
 import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
 import java.awt.event.HierarchyEvent;
 import java.awt.event.HierarchyListener;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyAdapter;
 import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
-import java.awt.event.MouseListener;
 import java.io.IOException;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.net.URL;
-import java.util.Hashtable;
+import java.util.HashMap;
 import java.util.Map;
 import java.util.ResourceBundle;
 import javax.swing.*;
@@ -69,6 +67,7 @@ import javax.swing.event.HyperlinkEvent;
 import javax.swing.event.HyperlinkListener;
 import javax.swing.text.AttributeSet;
 import javax.swing.text.BadLocationException;
+import javax.swing.text.EditorKit;
 import javax.swing.text.Element;
 import javax.swing.text.StyleConstants;
 import javax.swing.text.html.HTML;
@@ -82,7 +81,7 @@ import org.netbeans.lib.profiler.ui.UIUtils;
  * @author Ian Formanek
  * @author Jiri Sedlacek
  */
-public class HTMLTextArea extends JEditorPane implements HyperlinkListener, MouseListener {
+public class HTMLTextArea extends JEditorPane implements HyperlinkListener {
     //~ Inner Classes ------------------------------------------------------------------------------------------------------------
 
     /** Private Writer that extracts correctly formatted string from HTMLDocument */
@@ -235,7 +234,7 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
 
         private static synchronized Map getEntities() {
             if (entities == null) {
-                entities = new Hashtable();
+                entities = new HashMap();
                 //Quotation mark
                 entities.put("quot", "\""); //NOI18N
                                             //Ampersand
@@ -555,15 +554,8 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
 
     //~ Instance fields ----------------------------------------------------------------------------------------------------------
 
-    private ActionListener popupListener;
-    private JMenuItem itemCopy;
-    private JMenuItem itemCut;
-    private JMenuItem itemDelete;
-    private JMenuItem itemPaste;
-    private JMenuItem itemSelectAll;
+    private URL activeLink;
 
-    // --- Popup menu support ----------------------------------------------------
-    private JPopupMenu popupMenu;
     private boolean showPopup = true;
 
     // --- Lazy setting text ---------------------------------------------------
@@ -574,7 +566,7 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
     //~ Constructors -------------------------------------------------------------------------------------------------------------
 
     public HTMLTextArea() {
-        setEditorKit(new HTMLEditorKit());
+        setContentType("text/html"); // NOI18N
         setEditable(false);
         setOpaque(true);
         setAutoscrolls(true);
@@ -582,7 +574,6 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
         setTransferHandler(new HTMLTextAreaTransferHandler());
         setFont(UIManager.getFont("Label.font")); //NOI18N
         setBackground(UIUtils.getProfilerResultsBackground());
-        addMouseListener(this);
         
         addKeyListener(new KeyAdapter() {
             public void keyPressed(KeyEvent e) {
@@ -608,6 +599,11 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
 
     //~ Methods ------------------------------------------------------------------------------------------------------------------
     
+    public EditorKit getEditorKitForContentType(String type) {
+        // Always assumes "text/html" as this is a HTML displayer
+        return new HTMLEditorKit();
+    }
+    
     public void setOpaque(boolean o) {
         super.setOpaque(o);
         if (UIUtils.isNimbusLookAndFeel() && !o)
@@ -711,136 +707,145 @@ public class HTMLTextArea extends JEditorPane implements HyperlinkListener, Mous
         }
 
         if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
-            showURL(e.getURL());
+            activeLink = e.getURL();
+            showURL(activeLink, e.getInputEvent());
         } else if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
+            activeLink = e.getURL();
             setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
         } else if (e.getEventType() == HyperlinkEvent.EventType.EXITED) {
+            activeLink = null;
             setCursor(Cursor.getDefaultCursor());
         }
     }
+    
+    public URL getActiveLink() {
+        return activeLink;
+    }
+    
+    protected void processMouseEvent(MouseEvent e) {
+        if (e.isPopupTrigger()) showPopupMenu(e);
+        super.processMouseEvent(e);
+    }
+    
+    protected void processKeyEvent(KeyEvent e) {
+        int code = e.getKeyCode();
+        if (code == KeyEvent.VK_CONTEXT_MENU ||
+           (code == KeyEvent.VK_F10 && e.getModifiers() == InputEvent.SHIFT_MASK)) {
+            e.consume();
+            showPopupMenu(null);
+        }
+        
+        super.processKeyEvent(e);
+    }
+    
+    private void showPopupMenu(MouseEvent e) {
+        if (isEnabled() && isFocusable() && showPopup) {
+            JPopupMenu popup = new JPopupMenu();
+            populatePopup(popup);
 
-    public void mouseClicked(MouseEvent e) {
-        if (e.getModifiers() == InputEvent.BUTTON3_MASK) {
-            if (isEnabled() && isFocusable() && showPopup) {
-                JPopupMenu popup = getPopupMenu();
+            if (popup.getComponentCount() > 0) {
 
-                if (popup != null) {
-                    updatePopupMenu();
-
-                    if (!hasFocus()) {
-                        requestFocus(); // required for Select All functionality
-                    }
-
-                    popup.show(this, e.getX(), e.getY());
+                if (!hasFocus()) requestFocus(); // required for Select All functionality
+                
+                int x, y;
+                if (e != null) {
+                    x = e.getX();
+                    y = e.getY();
+                } else {
+                    Rectangle vis = getVisibleRect();
+                    x = vis.x + vis.width / 2;
+                    y = vis.y + vis.height / 2;
+                    
+                    try {
+                        Rectangle pos = modelToView(getCaretPosition());
+                        if (pos != null) {
+                            pos.width = Math.max(pos.width, 1); // must have nonzero width for the intersects() to work
+                            if (vis.intersects(pos)) {
+                                x = pos.x + pos.width;
+                                y = pos.y + pos.height;
+                            }
+                        }
+                    } catch (BadLocationException ex) {}
                 }
+
+                popup.show(this, x, y);
             }
         }
     }
 
-    public void mouseEntered(MouseEvent e) {
+    protected void populatePopup(JPopupMenu popup) {
+        popup.add(createCutMenuItem());
+        popup.add(createCopyMenuItem());
+        popup.add(createPasteMenuItem());
+        popup.add(createDeleteMenuItem());
+        popup.addSeparator();
+        popup.add(createSelectAllMenuItem());
     }
-
-    public void mouseExited(MouseEvent e) {
+    
+    protected JMenuItem createCutMenuItem() {
+        return new JMenuItem(CUT_STRING) {
+            { setEnabled(isEditable() && getSelectedText() != null); }
+            protected void fireActionPerformed(ActionEvent e) { cut(); }
+        };
     }
-
-    public void mousePressed(MouseEvent e) {
+    
+    protected JMenuItem createCopyMenuItem() {
+        return new JMenuItem(COPY_STRING) {
+            { setEnabled(getSelectedText() != null); }
+            protected void fireActionPerformed(ActionEvent e) { copy(); }
+        };
     }
-
-    public void mouseReleased(MouseEvent e) {
+    
+    protected JMenuItem createPasteMenuItem() {
+        return new JMenuItem(PASTE_STRING) {
+            {
+                if (isEditable()) {
+                    try {
+                        Transferable clipboardContent = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this);
+                        setEnabled(clipboardContent != null && clipboardContent.isDataFlavorSupported(DataFlavor.stringFlavor));
+                    } catch (Exception e) {
+                        setEnabled(false);
+                    }
+                } else {
+                    setEnabled(false);
+                }
+            }
+            protected void fireActionPerformed(ActionEvent e) { paste(); }
+        };
     }
-
+    
+    protected JMenuItem createDeleteMenuItem() {
+        return new JMenuItem(DELETE_STRING) {
+            {
+                if (isEditable()) {
+                    setEnabled(getSelectedText() != null);
+                } else {
+                    setVisible(false);
+                }
+            }
+            protected void fireActionPerformed(ActionEvent e) { deleteSelection(); }
+        };
+    }
+    
+    protected JMenuItem createSelectAllMenuItem() {
+        return new JMenuItem(SELECT_ALL_STRING) {
+            protected void fireActionPerformed(ActionEvent e) { selectAll(); }
+        };
+    }
+    
     public void paste() {
         try {
             replaceSelection(Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this)
                                     .getTransferData(DataFlavor.stringFlavor).toString());
-        } catch (Exception ex) {
-        }
-
-        ;
+        } catch (Exception ex) {}
     }
-
-    protected JPopupMenu getPopupMenu() {
-        if (popupMenu == null) {
-            popupMenu = createPopupMenu();
-        }
-
-        return popupMenu;
-    }
-
-    protected JPopupMenu createPopupMenu() {
-        JPopupMenu popup = new JPopupMenu();
-
-        popupListener = createPopupListener();
-
-        itemCut = new JMenuItem(CUT_STRING);
-        itemCopy = new JMenuItem(COPY_STRING);
-        itemPaste = new JMenuItem(PASTE_STRING);
-        itemDelete = new JMenuItem(DELETE_STRING);
-        itemSelectAll = new JMenuItem(SELECT_ALL_STRING);
-
-        itemCut.addActionListener(popupListener);
-        itemCopy.addActionListener(popupListener);
-        itemPaste.addActionListener(popupListener);
-        itemDelete.addActionListener(popupListener);
-        itemSelectAll.addActionListener(popupListener);
-
-        popup.add(itemCut);
-        popup.add(itemCopy);
-        popup.add(itemPaste);
-        popup.add(itemDelete);
-        popup.addSeparator();
-        popup.add(itemSelectAll);
-
-        return popup;
+    
+    protected void showURL(URL url, InputEvent e) {
+        showURL(url);
     }
 
     protected void showURL(URL url) {
         // override to react to URL clicks
     }
-
-    protected void updatePopupMenu() {
-        // Cut
-        itemCut.setEnabled(isEditable() && (getSelectedText() != null));
-
-        // Copy
-        itemCopy.setEnabled(getSelectedText() != null);
-
-        // Paste
-        try {
-            Transferable clipboardContent = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this);
-            itemPaste.setEnabled(isEditable() && (clipboardContent != null)
-                                 && clipboardContent.isDataFlavorSupported(DataFlavor.stringFlavor));
-        } catch (Exception e) {
-            itemPaste.setEnabled(false);
-        }
-
-        // Delete
-        if (isEditable()) {
-            itemDelete.setVisible(true);
-            itemDelete.setEnabled(getSelectedText() != null);
-        } else {
-            itemDelete.setVisible(false);
-        }
-
-        // Select All
-        // always visible and enabled...
-    }
-
-    private ActionListener createPopupListener() {
-        return new ActionListener() {
-                public void actionPerformed(ActionEvent e) {
-                    if (e.getSource() == itemCut) {
-                        cut();
-                    } else if (e.getSource() == itemCopy) {
-                        copy();
-                    } else if (e.getSource() == itemPaste) {
-                        paste();
-                    } else if (e.getSource() == itemDelete) {
-                        deleteSelection();
-                    } else if (e.getSource() == itemSelectAll) {
-                        selectAll();
-                    }
-                }
-            };
-    }
+    
 }
