@@ -43,6 +43,14 @@
 
 package org.netbeans.lib.profiler;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
+import java.net.ConnectException;
+import java.net.Socket;
+import java.text.MessageFormat;
+import java.util.*;
+import java.util.concurrent.atomic.AtomicBoolean;
 import org.netbeans.lib.profiler.classfile.ClassRepository;
 import org.netbeans.lib.profiler.client.AppStatusHandler;
 import org.netbeans.lib.profiler.client.ClientUtils;
@@ -63,18 +71,13 @@ import org.netbeans.lib.profiler.results.coderegion.CodeRegionResultsSnapshot;
 import org.netbeans.lib.profiler.results.cpu.CPUCCTProvider;
 import org.netbeans.lib.profiler.results.cpu.CPUResultsSnapshot;
 import org.netbeans.lib.profiler.results.cpu.FlatProfileProvider;
+import org.netbeans.lib.profiler.results.jdbc.JdbcCCTProvider;
+import org.netbeans.lib.profiler.results.jdbc.JdbcResultsSnapshot;
 import org.netbeans.lib.profiler.results.memory.*;
+import org.netbeans.lib.profiler.results.threads.ThreadDump;
 import org.netbeans.lib.profiler.utils.MiscUtils;
 import org.netbeans.lib.profiler.utils.StringUtils;
 import org.netbeans.lib.profiler.wireprotocol.*;
-import java.io.IOException;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-import java.net.ConnectException;
-import java.net.Socket;
-import java.text.MessageFormat;
-import java.util.*;
-import java.util.concurrent.atomic.AtomicBoolean;
 
 
 /**
@@ -250,8 +253,6 @@ public class ProfilerClient implements CommonConstants {
                     (new Thread() {
                             public void run() {
                                 try {
-                                    int instrType = getCurrentInstrType();
-
                                     if (currentInstrTypeIsRecursiveCPUProfiling() || currentInstrTypeIsMemoryProfiling()) {
                                         forceObtainedResultsDump(false, 15);
                                     }
@@ -383,6 +384,7 @@ public class ProfilerClient implements CommonConstants {
     private InitiateProfilingCommand commandOnStartup = null;
     private Instrumentor instrumentor;
     private MemoryCCTProvider memCctProvider;
+    private JdbcCCTProvider jdbcCctProvider;
     private final Object execInSeparateThreadLock = new Object();
     final private Object forceObtainedResultsDumpLock = new Object(); // To make dump processing and other commands mutually
                                                                 // exclusive
@@ -416,7 +418,6 @@ public class ProfilerClient implements CommonConstants {
      * The getAllocatedObjectsCountResults() method below provides only one aspect of the memory profiling data.
      */
     private int[] savedAllocatedObjectsCountResults;
-    private volatile boolean connectionWithServerOpen; // Used just to prevent double entry into closeConnection()
     private volatile boolean forceObtainedResultsDumpCalled;
     private volatile boolean handlingEventBufferDump;
     private volatile boolean instrMethodsLimitReported;
@@ -436,7 +437,7 @@ public class ProfilerClient implements CommonConstants {
         appStatusHandler = ash;
         serverCommandHandler = sch;
         instrumentor = new Instrumentor(status, settings);
-        histogramManager = new HeapHistogramManager();
+        histogramManager = new HeapHistogramManager(settings);
         EventBufferProcessor.initialize(this);
         EventBufferResultsProvider.getDefault().addDispatcher(ProfilingResultsDispatcher.getDefault());
     }
@@ -500,8 +501,8 @@ public class ProfilerClient implements CommonConstants {
             }
         }
         synchronized (this) {
-            int len = 0;
-            boolean twoTimeStamps = false;
+            int len;
+            boolean twoTimeStamps;
             String[] instrClassNames, instrMethodNames, instrMethodSigs;
             try {
                 status.beginTrans(false);
@@ -660,6 +661,40 @@ public class ProfilerClient implements CommonConstants {
         }
     }
 
+   /**
+     * Returns the snapshot of current JDBC(Selects) profiling results
+     *
+     * @return JDBC Results snapshot
+     * @throws ClientUtils.TargetAppOrVMTerminated
+     *          In case the profiled application has already terminated
+     */
+    public JdbcResultsSnapshot getJdbcProfilingResultsSnapshot()
+        throws ClientUtils.TargetAppOrVMTerminated {
+        return getJdbcProfilingResultsSnapshot(true);
+    }
+
+    /**
+     * Returns the snapshot of current Memory profiling results
+     *
+     * @param dump true to fetch latest events from server, false otherwise (use only available data)
+     * @return Memory Results snapshot
+     * @throws ClientUtils.TargetAppOrVMTerminated
+     *          In case the profiled application has already terminated
+     */
+    public JdbcResultsSnapshot getJdbcProfilingResultsSnapshot(boolean dump)
+        throws ClientUtils.TargetAppOrVMTerminated {
+        checkForTargetVMAlive();
+        if (dump) {
+            if (!forceObtainedResultsDump(false, 5)) {
+                return null;
+            }
+        }
+
+        synchronized (this) {
+            return new JdbcResultsSnapshot(resultsStart, System.currentTimeMillis(), jdbcCctProvider, this);
+        }
+    }
+    
     public Marker getMethodMarker() {
         return settings.getMethodMarker();
     }
@@ -672,11 +707,12 @@ public class ProfilerClient implements CommonConstants {
      * Assumption is that jMethodId is never reused inside the JVM.
      *
      * @param methodIds array of jMethodIds that we do not have names for
-     * @return the 3xn array, containing triplets of {class name, method name, method signature} strings for
+     * @return the 4xn array, containing quadruplets of {class name, method name, method signature, native flag} strings for
      *         given jmethodIds
      */
     public synchronized String[][] getMethodNamesForJMethodIds(int[] methodIds)
         throws ClientUtils.TargetAppOrVMTerminated {
+        final int PACKEDARR_ITEMS = 4; // must match PACKEDARR_ITEMS in Stacks.c
         checkForTargetVMAlive();
 
         GetMethodNamesForJMethodIdsCommand cmd = new GetMethodNamesForJMethodIdsCommand(methodIds);
@@ -684,7 +720,7 @@ public class ProfilerClient implements CommonConstants {
 
         MethodNamesResponse resp = (MethodNamesResponse) getAndCheckLastResponse("Unknown problem when trying to get method names for jmethodIds"); // NOI18N
 
-        return StringUtils.convertPackedStringsIntoStringArrays(resp.getPackedData(), resp.getPackedArrayOffsets(), 3);
+        return StringUtils.convertPackedStringsIntoStringArrays(resp.getPackedData(), resp.getPackedArrayOffsets(), PACKEDARR_ITEMS);
     }
 
     public synchronized HeapHistogram getHeapHistogram() throws ClientUtils.TargetAppOrVMTerminated {
@@ -696,6 +732,25 @@ public class ProfilerClient implements CommonConstants {
         return histogramManager.getHistogram(resp);
     }
     
+    public synchronized ThreadDump takeThreadDump() throws ClientUtils.TargetAppOrVMTerminated {
+        ThreadDumpResponse resp;
+        
+        checkForTargetVMAlive();
+        sendSimpleCmdToServer(Command.TAKE_THREAD_DUMP);
+        resp = (ThreadDumpResponse) getAndCheckLastResponse("Unknown problem when trying to take thread dump"); // NOI18N
+        return new ThreadDump(resp.isJDK15(), resp.getTime(), resp.getThreads());
+    }
+
+    public synchronized byte[][] getCachedClassFileBytes(String[] classes, int[] classLoaderIds) throws ClientUtils.TargetAppOrVMTerminated {        
+        GetClassFileBytesResponse resp;
+        
+        checkForTargetVMAlive();
+        GetClassFileBytesCommand cmd = new GetClassFileBytesCommand(classes, classLoaderIds);
+        sendComplexCmdToServer(cmd);
+        resp = (GetClassFileBytesResponse) getAndCheckLastResponse("Unknown problem when trying to get cached class file bytes"); // NOI18N
+        return resp.getClassBytes();
+    }
+
     public synchronized MonitoredData getMonitoredData() {
         try {
             checkForTargetVMAlive();
@@ -775,7 +830,7 @@ public class ProfilerClient implements CommonConstants {
                 return;
             }
 
-            Response resp = null;
+            Response resp;
             checkForTargetAppRunning();
 
             long curTime = System.currentTimeMillis();
@@ -1132,13 +1187,13 @@ public class ProfilerClient implements CommonConstants {
         memCctProvider = provider;
     }
 
+    public void registerJdbcCCTProvider(JdbcCCTProvider provider) {
+        jdbcCctProvider = provider;
+    }
+
     public void removeAllInstrumentation(boolean cleanupClient)
                                   throws InstrumentationException {
         synchronized (instrumentationLock) {
-            if (getCurrentInstrType() == INSTR_NONE) {
-                return;
-            }
-
             commandOnStartup = null;
 
             if (cleanupClient) {
@@ -1200,6 +1255,7 @@ public class ProfilerClient implements CommonConstants {
                                          throws ClientUtils.TargetAppOrVMTerminated {
         SetChangeableInstrParamsCommand cmd = new SetChangeableInstrParamsCommand(settings.isLockContentionMonitoringEnabled(),
                                                                                   settings.getNProfiledThreadsLimit(),
+                                                                                  settings.getStackDepthLimit(),
                                                                                   settings.getSamplingInterval(),
                                                                                   settings.getAllocTrackEvery(),
                                                                                   settings.getAllocStackTraceLimit(),
@@ -1536,12 +1592,19 @@ public class ProfilerClient implements CommonConstants {
         status.maxHeapSize = resp.getMaxHeapSize();
         status.startupTimeMillis = resp.getStartupTimeMillis();
         status.startupTimeInCounts = resp.getStartupTimeInCounts();
+        status.canInstrumentConstructor = resp.canInstrumentConstructor();
 
-        if (!status.remoteProfiling && settings.getTargetJDKVersionString() != CommonConstants.JDK_19_STRING) {
+        if (!status.remoteProfiling
+            && settings.getTargetJDKVersionString() != CommonConstants.JDK_19_STRING
+            && settings.getTargetJDKVersionString() != CommonConstants.JDK_110_STRING
+            && settings.getTargetJDKVersionString() != CommonConstants.JDK_110_STRING) {
             settings.setWorkingDir(resp.getWorkingDir());
             settings.setVMClassPaths(resp.getJavaClassPath(), resp.getJavaExtDirs(), resp.getBootClassPath());
-            ClassRepository.initClassPaths(settings.getWorkingDir(), settings.getVMClassPaths());
+        } else {
+            settings.setWorkingDir("");
+            settings.setVMClassPaths("", null, null);
         }
+        ClassRepository.initClassPaths(settings.getWorkingDir(), settings.getVMClassPaths());
 
         return true;
     }
@@ -1573,7 +1636,7 @@ public class ProfilerClient implements CommonConstants {
     //-------------------------------- Private implementation ------------------------------------------
     private void clearPreviousInstrumentationInServer()
                                                throws InstrumentationException, ClientUtils.TargetAppOrVMTerminated {
-        Response resp = null;
+        Response resp;
         checkForTargetAppRunning();
 
         // First send the command that will make the application stop emitting events
@@ -1881,7 +1944,7 @@ public class ProfilerClient implements CommonConstants {
         synchronized (instrumentationLock) {
             AppStatusHandler.AsyncDialog waitDialog = null;
             try {
-                InstrumentMethodGroupResponse imgr = null;
+                InstrumentMethodGroupResponse imgr;
 
                 if (!serverClassesInitialized) { // Check if it is a fake command from server, used to just pre-initialize
                                                  // some internal server classes
