@@ -177,7 +177,7 @@ public class TargetAppRunner implements CommonConstants {
                             AsyncMessageCommand msg = (AsyncMessageCommand) cmd;
 
                             if (msg.isPositive()) {
-                                appStatusHandler.displayNotification(msg.getMessage());
+//                                appStatusHandler.displayNotification(msg.getMessage());
                             } else {
                                 appStatusHandler.displayError(msg.getMessage());
 
@@ -339,13 +339,15 @@ public class TargetAppRunner implements CommonConstants {
     }
 
     public boolean hasSupportedJDKForHeapDump() {
-        // not supported for JDK other than 1.9 & 1.8 & 1.7 & 1.6 & 1.5.0_12 and up
+        // not supported for JDK other than 10 & 1.9 & 1.8 & 1.7 & 1.6 & 1.5.0_12 and up
         String jdkVersion = getProfilerEngineSettings().getTargetJDKVersionString();
 
         if (CommonConstants.JDK_16_STRING.equals(jdkVersion)
            || CommonConstants.JDK_17_STRING.equals(jdkVersion)
            || CommonConstants.JDK_18_STRING.equals(jdkVersion)
-           || CommonConstants.JDK_19_STRING.equals(jdkVersion)) {
+           || CommonConstants.JDK_19_STRING.equals(jdkVersion)
+           || CommonConstants.JDK_100_STRING.equals(jdkVersion)
+           || CommonConstants.JDK_110_STRING.equals(jdkVersion)) {
             return true;
         }
 
@@ -703,8 +705,10 @@ public class TargetAppRunner implements CommonConstants {
         if (jdkVer.equals(JDK_16_STRING)
             || jdkVer.equals(JDK_17_STRING)
             || jdkVer.equals(JDK_18_STRING)
-            || jdkVer.equals(JDK_19_STRING)) {
-            // for now the 1.6 and 1.7 and 1.8 profiling uses the same agent as 1.5
+            || jdkVer.equals(JDK_19_STRING)
+            || jdkVer.equals(JDK_100_STRING)
+            || jdkVer.equals(JDK_110_STRING)) {
+            // for now the 1.6 and 1.7 and 1.8 and 9 and 10 and 11 profiling uses the same agent as 1.5
             jdkVer = JDK_15_STRING;
         }
 
@@ -754,11 +758,14 @@ public class TargetAppRunner implements CommonConstants {
 
         commands.add(settings.getTargetJVMExeFile());
 
-        if (settings.getTargetJDKVersionString().equals(Platform.JDK_15_STRING)
-                || settings.getTargetJDKVersionString().equals(Platform.JDK_16_STRING)
-                || settings.getTargetJDKVersionString().equals(Platform.JDK_17_STRING)
-                || settings.getTargetJDKVersionString().equals(Platform.JDK_18_STRING)
-                || settings.getTargetJDKVersionString().equals(Platform.JDK_19_STRING)) {
+        String jdk = settings.getTargetJDKVersionString();
+        if (jdk.equals(Platform.JDK_15_STRING)
+            || jdk.equals(Platform.JDK_16_STRING)
+            || jdk.equals(Platform.JDK_17_STRING)
+            || jdk.equals(Platform.JDK_18_STRING)
+            || jdk.equals(Platform.JDK_19_STRING)
+            || jdk.equals(Platform.JDK_100_STRING)
+            || jdk.equals(Platform.JDK_110_STRING)) {
             String jfNativeLibFullName = Platform.getAgentNativeLibFullName(settings.getJFluidRootDirName(), false,
                                                                             settings.getTargetJDKVersionString(),
                                                                             settings.getSystemArchitecture());
@@ -812,8 +819,7 @@ public class TargetAppRunner implements CommonConstants {
             commands.add(mainArgs[i]);
         }
 
-        String[] cmdArray = new String[commands.size()];
-        commands.toArray(cmdArray);
+        String[] cmdArray = (String[])commands.toArray(new String[0]);
 
         MiscUtils.printInfoMessage("Starting target application..."); // NOI18N
         MiscUtils.printVerboseInfoMessage(cmdArray);
