@@ -97,6 +97,8 @@ public interface CommonConstants {
     public static final int JDK_17 = 4; // we treat JDK 17 the same as JDK 1.5 for now
     public static final int JDK_18 = 6;
     public static final int JDK_19 = 7;
+    public static final int JDK_100 = 8;
+    public static final int JDK_110 = 9;
     public static final int JDK_CVM = 5;
     public static final int JDK_UNSUPPORTED = -1;
 
@@ -106,6 +108,8 @@ public interface CommonConstants {
     public static final String JDK_17_STRING = "jdk17"; // NOI18N
     public static final String JDK_18_STRING = "jdk18"; // NOI18N
     public static final String JDK_19_STRING = "jdk19"; // NOI18N
+    public static final String JDK_100_STRING = "jdk100"; // NOI18N
+    public static final String JDK_110_STRING = "jdk110"; // NOI18N
     public static final String JDK_CVM_STRING = "cvm";  // NOI18N
     public static final String JDK_UNSUPPORTED_STRING = "UNSUPPORTED_JDK"; // NOI18N
 
@@ -157,6 +161,7 @@ public interface CommonConstants {
     public static final byte THREAD_DUMP_END = 32;
     public static final byte THREAD_INFO_IDENTICAL = 33;
     public static final byte THREAD_INFO = 34;
+    public static final byte MARKER_ENTRY_PARAMETERS = 35;
 
     // The following are used when storing unstamped method entry/exit events in the "compact" format, when both
     // event code and method id are packed in a single char. See more comments in ProfilerRuntimeCPUSampledInstr.java
@@ -213,6 +218,7 @@ public interface CommonConstants {
     static final String JAVA_LANG_REFLECT_METHOD_SLASHED_CLASS_NAME = "java/lang/reflect/Method"; // NOI18N
     static final String INVOKE_METHOD_NAME = "invoke"; // NOI18N
     static final String INVOKE_METHOD_SIGNATURE = "(Ljava/lang/Object;[Ljava/lang/Object;)Ljava/lang/Object;"; // NOI18N
+    static final String OBJECT_SLASHED_CLASS_NAME = "java/lang/Object";
 
     // Kinds of injections that we make into TA methods
     public static final int INJ_RECURSIVE_NORMAL_METHOD = 0;
@@ -305,5 +311,8 @@ public interface CommonConstants {
     public static final int AGENT_VERSION_73 = 13;
     public static final int AGENT_VERSION_74 = 14;
     public static final int AGENT_VERSION_80 = 15;
-    public static final int CURRENT_AGENT_VERSION = AGENT_VERSION_80;
+    public static final int AGENT_VERSION_81 = 16;
+    public static final int AGENT_VERSION_82 = 17;
+    public static final int AGENT_VERSION_90 = 18;
+    public static final int CURRENT_AGENT_VERSION = AGENT_VERSION_90;
 }
