@@ -182,8 +182,10 @@ public class Platform implements CommonConstants {
 
         is64bitArch = architecture == ARCH_64;
 
-        if (jdkString.equals(JDK_17_STRING) || jdkString.equals(JDK_18_STRING) || jdkString.equals(JDK_19_STRING)) {
-            // for now, we use the same libs for 1.6 and 1.7 and 1.8 and 1.9
+        if (jdkString.equals(JDK_17_STRING) || jdkString.equals(JDK_18_STRING)
+            || jdkString.equals(JDK_19_STRING) || jdkString.equals(JDK_100_STRING)
+            || jdkString.equals(JDK_110_STRING)) {
+            // for now, we use the same libs for 1.6 and 1.7 and 1.8 and 1.9 and 10 and 11
             jdkString = JDK_16_STRING;
         }
 
@@ -305,8 +307,12 @@ public class Platform implements CommonConstants {
             jdkVersion = JDK_18;
         } else if (javaVersion.startsWith("1.9")) { // NOI18N
             jdkVersion = JDK_19;
-        } else if (javaVersion.startsWith("9")) { // NOI18N
+        } else if (javaVersion.equals("9") || javaVersion.startsWith("9.")) { // NOI18N
             jdkVersion = JDK_19;
+        } else if (javaVersion.equals("10") || javaVersion.startsWith("10.")) { // NOI18N
+            jdkVersion = JDK_100;
+        } else if (javaVersion.equals("11") || javaVersion.equals("11-ea") || javaVersion.startsWith("11.")) { // NOI18N
+            jdkVersion = JDK_110;
         } else if (javaVersion.equals("CVM")) { // NOI18N
             jdkVersion = JDK_CVM;
         } else {
@@ -329,7 +335,8 @@ public class Platform implements CommonConstants {
     /**
      * Returns the string for, essentially, JFluid directory corresponding to a particular JDK version the TA runs on.
      * Currently it's "jdk15" for JDK 1.5 version and "jdk16" for JDK 1.6 version, "jdk17" for JDK 1.7 version,
-     *  "jdk18" for JDK 1.8 version and "cvm" for CVM
+     *  "jdk18" for JDK 1.8 version and "jdk19" for JDK 9 version and
+     *  "jdk100" for JDK 10 version and "cvm" for CVM
      */
     public static String getJDKVersionString(String javaVersionString) {
         int jdkVersionNumber = getJDKVersionNumber(javaVersionString);
@@ -340,6 +347,8 @@ public class Platform implements CommonConstants {
             case JDK_17: return JDK_17_STRING;
             case JDK_18: return JDK_18_STRING;
             case JDK_19: return JDK_19_STRING;
+            case JDK_100: return JDK_100_STRING;
+            case JDK_110: return JDK_110_STRING;
             case JDK_CVM: return JDK_CVM_STRING;
             case JDK_UNSUPPORTED: return JDK_UNSUPPORTED_STRING;
         }
@@ -476,7 +485,7 @@ public class Platform implements CommonConstants {
 
             if (!d.exists()) {
                 if (!d.mkdir()) {
-                    throw new IOException("Could not create directory" + customDir); // NOI18N
+                    throw new IOException("Could not create directory " + customDir); // NOI18N
                 }
             }
 
@@ -488,7 +497,7 @@ public class Platform implements CommonConstants {
 
             if (!d.exists()) {
                 if (!d.mkdir()) {
-                    throw new IOException("Could not create directory" + dir); // NOI18N
+                    throw new IOException("Could not create directory " + dir); // NOI18N
                 }
             }
 
@@ -590,13 +599,15 @@ public class Platform implements CommonConstants {
     }
 
     /**
-     * Returns true if the given JVM version supports dynaimc attach
+     * Returns true if the given JVM version supports dynamic attach
      */
     public static boolean supportsDynamicAttach(String jdkVersionString) {
         return CommonConstants.JDK_16_STRING.equals(jdkVersionString) 
                || CommonConstants.JDK_17_STRING.equals(jdkVersionString)
                || CommonConstants.JDK_18_STRING.equals(jdkVersionString)
-               || CommonConstants.JDK_19_STRING.equals(jdkVersionString);
+               || CommonConstants.JDK_19_STRING.equals(jdkVersionString)
+               || CommonConstants.JDK_100_STRING.equals(jdkVersionString)
+               || CommonConstants.JDK_110_STRING.equals(jdkVersionString);
     }
 
     /**
@@ -609,6 +620,8 @@ public class Platform implements CommonConstants {
 		   jdkVersionString.equals(JDK_17_STRING) ||
 		   jdkVersionString.equals(JDK_18_STRING) ||
 		   jdkVersionString.equals(JDK_19_STRING) ||
+		   jdkVersionString.equals(JDK_100_STRING) ||
+		   jdkVersionString.equals(JDK_110_STRING) ||
 		   jdkVersionString.equals(JDK_CVM_STRING)));
     }
 
@@ -627,7 +640,7 @@ public class Platform implements CommonConstants {
 		(jdkVersionNumber == JDK_16) ||
 		(jdkVersionNumber == JDK_17) ||
 		(jdkVersionNumber == JDK_18) ||
-		(jdkVersionNumber == JDK_19) ||
+		(jdkVersionNumber >= JDK_19) ||
 		(jdkVersionNumber == JDK_CVM));
     }
 }
