@@ -43,16 +43,20 @@
 
 package org.netbeans.lib.profiler.heap;
 
+import java.io.BufferedOutputStream;
+import java.io.DataInputStream;
+import java.io.DataOutputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.NoSuchElementException;
 import java.util.Properties;
 
 /**
@@ -93,6 +97,17 @@ class HprofHeap implements Heap {
     static final int OBJECT_ARRAY_DUMP = 0x22;
     static final int PRIMITIVE_ARRAY_DUMP = 0x23;
 
+    //  HPROF HEAP 1.0.3 tags
+    static final int HEAP_DUMP_INFO                = 0xfe;
+    static final int ROOT_INTERNED_STRING          = 0x89;
+    static final int ROOT_FINALIZING               = 0x8a;
+    static final int ROOT_DEBUGGER                 = 0x8b;
+    static final int ROOT_REFERENCE_CLEANUP        = 0x8c;
+    static final int ROOT_VM_INTERNAL              = 0x8d;
+    static final int ROOT_JNI_MONITOR              = 0x8e;
+    static final int UNREACHABLE                   = 0x90; /* deprecated */
+    static final int PRIMITIVE_ARRAY_NODATA_DUMP   = 0xc3;
+
     // basic type
     static final int OBJECT = 2;
     static final int BOOLEAN = 4;
@@ -105,29 +120,41 @@ class HprofHeap implements Heap {
     static final int LONG = 11;
     private static final boolean DEBUG = false;
 
+    private static final String SNAPSHOT_ID = "NBPHD";
+    private static final int SNAPSHOT_VERSION  = 2;
+    
     //~ Instance fields ----------------------------------------------------------------------------------------------------------
 
     HprofByteBuffer dumpBuffer;
     LongMap idToOffsetMap;
     private NearestGCRoot nearestGCRoot;
+    final HprofGCRoots gcRoots;
     private ComputedSummary computedSummary;
-    private Map gcRoots;
+    private final Object computedSummaryLock = new Object();
     private DominatorTree domTree;
-    final private Object gcRootLock = new Object();
     private TagBounds allInstanceDumpBounds;
     private TagBounds heapDumpSegment;
     private TagBounds[] heapTagBounds;
     private TagBounds[] tagBounds = new TagBounds[0xff];
     private boolean instancesCountComputed;
+    private final Object instancesCountLock = new Object();
     private boolean referencesComputed;
+    private final Object referencesLock = new Object();
     private boolean retainedSizeComputed;
+    private final Object retainedSizeLock = new Object();
     private boolean retainedSizeByClassComputed;
+    private final Object retainedSizeByClassLock = new Object();
     private int idMapSize;
     private int segment;
 
+    // for serialization
+    File heapDumpFile;
+    CacheDirectory cacheDirectory;
+    
     //~ Constructors -------------------------------------------------------------------------------------------------------------
 
-    HprofHeap(File dumpFile, int seg) throws FileNotFoundException, IOException {
+    HprofHeap(File dumpFile, int seg, CacheDirectory cacheDir) throws FileNotFoundException, IOException {
+        cacheDirectory = cacheDir;
         dumpBuffer = HprofByteBuffer.createHprofByteBuffer(dumpFile);
         segment = seg;
         fillTagBounds(dumpBuffer.getHeaderSize());
@@ -137,8 +164,10 @@ class HprofHeap implements Heap {
             fillHeapTagBounds();
         }
 
-        idToOffsetMap = new LongMap(idMapSize,dumpBuffer.getIDSize(),dumpBuffer.getFoffsetSize());
+        idToOffsetMap = new LongMap(idMapSize,dumpBuffer.getIDSize(),dumpBuffer.getFoffsetSize(), cacheDirectory);
         nearestGCRoot = new NearestGCRoot(this);
+        gcRoots = new HprofGCRoots(this);
+        heapDumpFile = dumpFile;
     }
 
     //~ Methods ------------------------------------------------------------------------------------------------------------------
@@ -173,28 +202,14 @@ class HprofHeap implements Heap {
     
     public GCRoot getGCRoot(Instance instance) {
        Long instanceId = Long.valueOf(instance.getInstanceId());
-       return getGCRoot(instanceId);
+       return gcRoots.getGCRoot(instanceId);
     }
 
     public Collection getGCRoots() {
-        synchronized (gcRootLock) {
-            if (heapDumpSegment == null) {
-                return Collections.EMPTY_LIST;
-            }
-            if (gcRoots == null) {
-                gcRoots = computeGCRootsFor(heapTagBounds[ROOT_UNKNOWN]);
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_JNI_GLOBAL]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_JNI_LOCAL]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_JAVA_FRAME]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_NATIVE_STACK]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_STICKY_CLASS]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_THREAD_BLOCK]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_MONITOR_USED]));
-                gcRoots.putAll(computeGCRootsFor(heapTagBounds[ROOT_THREAD_OBJECT]));
-            }
-
-            return gcRoots.values();
+        if (heapDumpSegment == null) {
+            return Collections.EMPTY_LIST;
         }
+        return gcRoots.getGCRoots();
     }
 
     public Instance getInstanceByID(long instanceID) {
@@ -203,50 +218,12 @@ class HprofHeap implements Heap {
         }
 
         computeInstances();
-
-        ClassDump classDump;
-        ClassDumpSegment classDumpBounds = getClassDumpSegment();
-        int idSize = dumpBuffer.getIDSize();
-        int classIdOffset = 0;
         LongMap.Entry entry = idToOffsetMap.get(instanceID);
 
         if (entry == null) {
             return null;
         }
-
-        long start = entry.getOffset();
-        assert start != 0L;
-
-        long[] offset = new long[] { start };
-
-        int tag = readDumpTag(offset);
-
-        if (tag == INSTANCE_DUMP) {
-            classIdOffset = idSize + 4;
-        } else if (tag == OBJECT_ARRAY_DUMP) {
-            classIdOffset = idSize + 4 + 4;
-        } else if (tag == PRIMITIVE_ARRAY_DUMP) {
-            classIdOffset = idSize + 4 + 4;
-        }
-
-        if (tag == PRIMITIVE_ARRAY_DUMP) {
-            classDump = classDumpBounds.getPrimitiveArrayClass(dumpBuffer.get(start + 1 + classIdOffset));
-
-            return new PrimitiveArrayDump(classDump, start);
-        } else {
-            long classId = dumpBuffer.getID(start + 1 + classIdOffset);
-            classDump = classDumpBounds.getClassDumpByID(classId);
-        }
-
-        if (tag == INSTANCE_DUMP) {
-            return new InstanceDump(classDump, start);
-        } else if (tag == OBJECT_ARRAY_DUMP) {
-            return new ObjectArrayDump(classDump, start);
-        } else if (tag == CLASS_DUMP) {
-            return new ClassDumpInstance(classDump);
-        } else {
-            throw new IllegalArgumentException("Illegal tag " + tag); // NOI18N
-        }
+        return getInstanceByOffset(new long[] {entry.getOffset()});
     }
 
     public JavaClass getJavaClassByID(long javaclassId) {
@@ -267,6 +244,42 @@ class HprofHeap implements Heap {
         return getClassDumpSegment().getJavaClassesByRegExp(regexp);
     }
     
+    
+    private class InstancesIterator implements Iterator {
+        private long[] offset;
+        private Instance nextInstance;
+        
+        private InstancesIterator() {
+            offset = new long[] { allInstanceDumpBounds.startOffset };
+        }
+
+        public boolean hasNext() {
+            while (offset[0] < allInstanceDumpBounds.endOffset && nextInstance == null) {
+                nextInstance = getInstanceByOffset(offset);
+            }
+            return nextInstance != null;
+        }
+
+        public Object next() {
+            if (hasNext()) {
+                Instance ni = nextInstance;
+
+                nextInstance = null;
+                return ni;
+            }
+            throw new NoSuchElementException();
+        }
+    }
+        
+    public Iterator getAllInstancesIterator() {
+        // make sure java classes are initialized
+        List classes = getAllClasses();
+        if (classes.isEmpty()) {
+            return Collections.EMPTY_LIST.iterator();
+        }
+        return new InstancesIterator();
+    }
+    
     public synchronized HeapSummary getSummary() {
         TagBounds summaryBound = tagBounds[HEAP_SUMMARY];
 
@@ -274,8 +287,10 @@ class HprofHeap implements Heap {
             return new Summary(dumpBuffer, summaryBound.startOffset);
         }
 
-        if (computedSummary == null) {
-            computedSummary = new ComputedSummary(this);
+        synchronized (computedSummaryLock) {
+            if (computedSummary == null) {
+                computedSummary = new ComputedSummary(this);
+            }
         }
 
         return computedSummary;
@@ -296,6 +311,83 @@ class HprofHeap implements Heap {
         return null;
     }
 
+    public boolean isRetainedSizeComputed() {
+        return retainedSizeComputed;
+    }
+
+    public boolean isRetainedSizeByClassComputed() {
+        return retainedSizeByClassComputed;
+    }
+
+    //---- Serialization support
+    void writeToFile() {
+        if (!cacheDirectory.isTemporary()) {
+            try {
+                DataOutputStream out;
+                File outFile = cacheDirectory.getHeapDumpAuxFile();
+                out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(outFile), 32768));
+                writeToStream(out);
+                out.close();
+            } catch (IOException ex) {
+                ex.printStackTrace(System.err);
+            }
+        }
+    }
+    
+    void writeToStream(DataOutputStream out) throws IOException {
+        out.writeUTF(SNAPSHOT_ID);
+        out.writeInt(SNAPSHOT_VERSION);
+        out.writeUTF(heapDumpFile.getAbsolutePath());
+        nearestGCRoot.writeToStream(out);
+        allInstanceDumpBounds.writeToStream(out);
+        heapDumpSegment.writeToStream(out);
+        TagBounds.writeToStream(heapTagBounds, out);
+        TagBounds.writeToStream(tagBounds, out);
+        out.writeBoolean(instancesCountComputed);
+        out.writeBoolean(referencesComputed);
+        out.writeBoolean(retainedSizeComputed);
+        out.writeBoolean(retainedSizeByClassComputed);
+        out.writeInt(idMapSize);
+        out.writeInt(segment);        
+        idToOffsetMap.writeToStream(out);
+        out.writeBoolean(domTree != null);
+        if (domTree != null) {
+            domTree.writeToStream(out);
+        }
+    }
+
+    HprofHeap(DataInputStream dis, CacheDirectory cacheDir) throws IOException {
+        String id = dis.readUTF();
+        if (!SNAPSHOT_ID.equals(id)) {
+            throw new IOException("Invalid HPROF dump id "+id);
+        }
+        int version = dis.readInt();
+        if (version != SNAPSHOT_VERSION) {
+            throw new IOException("Invalid HPROF version "+SNAPSHOT_VERSION+" loaded "+version);            
+        }
+        heapDumpFile = cacheDir.getHeapFile(dis.readUTF());
+        cacheDirectory = cacheDir;
+        dumpBuffer = HprofByteBuffer.createHprofByteBuffer(heapDumpFile);
+        nearestGCRoot = new NearestGCRoot(this, dis);
+        allInstanceDumpBounds = new TagBounds(dis);
+        heapDumpSegment = new TagBounds(dis);
+        heapTagBounds = new TagBounds[0x100];
+        TagBounds.readFromStream(dis, this, heapTagBounds);
+        TagBounds.readFromStream(dis, this, tagBounds);        
+        instancesCountComputed = dis.readBoolean();
+        referencesComputed = dis.readBoolean();
+        retainedSizeComputed = dis.readBoolean();
+        retainedSizeByClassComputed = dis.readBoolean();
+        idMapSize = dis.readInt();
+        segment = dis.readInt();
+        idToOffsetMap = new LongMap(dis, cacheDirectory);
+        if (dis.readBoolean()) {
+            domTree = new DominatorTree(this, dis);
+        }
+        gcRoots = new HprofGCRoots(this);
+        getClassDumpSegment().extractSpecialClasses();            
+    }
+    
     ClassDumpSegment getClassDumpSegment() {
         return (ClassDumpSegment) heapTagBounds[CLASS_DUMP];
     }
@@ -324,16 +416,6 @@ class HprofHeap implements Heap {
         computeRetainedSize();
         return idToOffsetMap.get(instance.getInstanceId()).getRetainedSize();
     }
-    
-    GCRoot getGCRoot(Long instanceId) {
-        synchronized (gcRootLock) {
-            if (gcRoots == null) {
-                getGCRoots();
-            }
-
-            return (GCRoot) gcRoots.get(instanceId);
-        }
-    }
 
     int getValueSize(final byte type) {
         switch (type) {
@@ -360,11 +442,68 @@ class HprofHeap implements Heap {
         }
     }
 
-    synchronized void computeInstances() {
+    Instance getInstanceByOffset(long[] offset) {
+        return getInstanceByOffset(offset, null, -1);
+    }
+
+    Instance getInstanceByOffset(long[] offset, ClassDump instanceClassDump, long instanceClassId) {
+        long start = offset[0];
+        assert start != 0L;
+        ClassDump classDump;
+        ClassDumpSegment classDumpBounds = getClassDumpSegment();
+        int idSize = dumpBuffer.getIDSize();
+        int classIdOffset = 0;
+
+        int tag = readDumpTag(offset);
+
+        if (tag == INSTANCE_DUMP) {
+            classIdOffset = idSize + 4;
+        } else if (tag == OBJECT_ARRAY_DUMP) {
+            classIdOffset = idSize + 4 + 4;
+        } else if (tag == PRIMITIVE_ARRAY_DUMP) {
+            classIdOffset = idSize + 4 + 4;
+        }
+
+        if (tag == PRIMITIVE_ARRAY_DUMP) {
+            classDump = classDumpBounds.getPrimitiveArrayClass(dumpBuffer.get(start + 1 + classIdOffset));
+            if (instanceClassId != -1 && classDump.getJavaClassId() != instanceClassId) {
+                return null;
+            }
+
+            return new PrimitiveArrayDump(classDump, start);
+        } else {
+            long classId = dumpBuffer.getID(start + 1 + classIdOffset);
+            if (instanceClassId != -1 && classId != instanceClassId) {
+                return null;
+            }
+            if (instanceClassDump == null) {
+                classDump = classDumpBounds.getClassDumpByID(classId);
+            } else {
+                classDump = instanceClassDump;        
+            }
+        }
+
+        if (classDump == null) {
+            return null;
+        }
+        if (tag == INSTANCE_DUMP) {
+            return new InstanceDump(classDump, start);
+        } else if (tag == OBJECT_ARRAY_DUMP) {
+            return new ObjectArrayDump(classDump, start);
+        } else if (tag == CLASS_DUMP) {
+            return new ClassDumpInstance(classDump);
+        }
+        // other heap dump tags, ROOT_ etc.
+        return null;
+    }
+
+    void computeInstances() {
+        synchronized (instancesCountLock) {
         if (instancesCountComputed) {
             return;
         }
 
+        HeapProgress.progressStart();
         ClassDumpSegment classDumpBounds = getClassDumpSegment();
         int idSize = dumpBuffer.getIDSize();
         long[] offset = new long[] { allInstanceDumpBounds.startOffset };
@@ -407,8 +546,10 @@ class HprofHeap implements Heap {
             }
             HeapProgress.progress(counter,allInstanceDumpBounds.startOffset,start,allInstanceDumpBounds.endOffset);
         }
-        HeapProgress.progressFinish();
         instancesCountComputed = true;
+        writeToFile();
+        }
+        HeapProgress.progressFinish();
     }
 
     List findReferencesFor(long instanceId) {
@@ -416,22 +557,18 @@ class HprofHeap implements Heap {
         computeReferences();
         
         List refs = new ArrayList();
-        List refIds = idToOffsetMap.get(instanceId).getReferences();
-        Iterator refIdsIt = refIds.iterator();
+        LongIterator refIdsIt = idToOffsetMap.get(instanceId).getReferences();
         int idSize = dumpBuffer.getIDSize();
         ClassDumpSegment classDumpBounds = getClassDumpSegment();
         long[] offset = new long[1];
         
         while (refIdsIt.hasNext()) {
-            long foundInstanceId = ((Long)refIdsIt.next()).longValue();
+            long foundInstanceId = refIdsIt.next();
             offset[0] = idToOffsetMap.get(foundInstanceId).getOffset();
-            int classIdOffset = 0;
-            int instanceIdOffset = 0;
             long start = offset[0];
             int tag = readDumpTag(offset);
 
             if (tag == INSTANCE_DUMP) {
-                classIdOffset = idSize + 4;
                 int size = dumpBuffer.getInt(start + 1 + idSize + 4 + idSize);
                 byte[] fields = new byte[size];
                 dumpBuffer.get(start + 1 + idSize + 4 + idSize + 4, fields);
@@ -459,14 +596,12 @@ class HprofHeap implements Heap {
                 }
             } else if (tag == OBJECT_ARRAY_DUMP) {
                 int elements = dumpBuffer.getInt(start + 1 + idSize + 4);
-                int i;
+                long classId = dumpBuffer.getID(start + 1 + idSize + 4 + 4);
+                ClassDump classDump = classDumpBounds.getClassDumpByID(classId);
                 long position = start + 1 + idSize + 4 + 4 + idSize;
 
-                for (i = 0; i < elements; i++, position += idSize) {
+                for (int i = 0; i < elements; i++, position += idSize) {
                     if (dumpBuffer.getID(position) == instanceId) {
-                        long classId = dumpBuffer.getID(start + 1 + idSize + 4 + 4);
-                        ClassDump classDump = classDumpBounds.getClassDumpByID(classId);
-
                         refs.add(new HprofArrayValue(classDump, start, i));
                     }
                 }
@@ -479,11 +614,13 @@ class HprofHeap implements Heap {
         return refs;
     }
 
-    synchronized void computeReferences() {
+    void computeReferences() {
+        synchronized (referencesLock) {
         if (referencesComputed) {
             return;
         }
 
+        HeapProgress.progressStart();
         ClassDumpSegment classDumpBounds = getClassDumpSegment();
         int idSize = dumpBuffer.getIDSize();
         long[] offset = new long[] { allInstanceDumpBounds.startOffset };
@@ -562,11 +699,14 @@ class HprofHeap implements Heap {
             }
         }
         idToOffsetMap.flush();
-        HeapProgress.progressFinish();        
         referencesComputed = true;
+        writeToFile();
+        }
+        HeapProgress.progressFinish();        
     }
     
-    synchronized void computeRetainedSize() {
+    void computeRetainedSize() {
+        synchronized (retainedSizeLock) {
         if (retainedSizeComputed) {
             return;
         }
@@ -595,10 +735,11 @@ class HprofHeap implements Heap {
             boolean isTreeObj = instanceEntry.isTreeObj();
             long instSize = 0;
             
-            if (!isTreeObj && (instanceEntry.getNearestGCRootPointer() != 0 || getGCRoot(new Long(instanceId)) != null)) {
+            if (!isTreeObj && (instanceEntry.getNearestGCRootPointer() != 0 || gcRoots.getGCRoot(new Long(instanceId)) != null)) {
                 long origSize = instanceEntry.getRetainedSize();
                 if (origSize < 0) origSize = 0;
-                instSize = getInstanceByID(instanceId).getSize();
+                Instance instance = getInstanceByID(instanceId);
+                instSize = instance != null ? instance.getSize() : getClassDumpSegment().getMinimumInstanceSize();
                 instanceEntry.setRetainedSize(origSize + instSize);
             }
             if (idom != 0) {
@@ -623,9 +764,12 @@ class HprofHeap implements Heap {
             }
         }
         retainedSizeComputed = true;
+        writeToFile();
+        }
     }
 
-    synchronized void computeRetainedSizeByClass() {
+    void computeRetainedSizeByClass() {
+        synchronized (retainedSizeByClassLock) {
         if (retainedSizeByClassComputed) {
             return;
         }
@@ -648,23 +792,27 @@ class HprofHeap implements Heap {
             }
             long instanceId = dumpBuffer.getID(start + instanceIdOffset);
             Instance i = getInstanceByID(instanceId);
-            ClassDump javaClass = (ClassDump) i.getJavaClass();
-            if (javaClass != null && !domTree.hasInstanceInChain(tag, i)) {
-                javaClass.addSizeForInstance(i);
+            if (i != null) {
+                ClassDump javaClass = (ClassDump) i.getJavaClass();
+                if (javaClass != null && !domTree.hasInstanceInChain(tag, i)) {
+                    javaClass.addSizeForInstance(i);
+                }
             }
         }
         // all done, release domTree
         domTree = null;
         retainedSizeByClassComputed = true;
+        writeToFile();
+        }
     }
 
-    synchronized Instance getNearestGCRootPointer(Instance instance) {
+    Instance getNearestGCRootPointer(Instance instance) {
         return nearestGCRoot.getNearestGCRootPointer(instance);
     }
     
     int readDumpTag(long[] offset) {
         long position = offset[0];
-        int dumpTag = dumpBuffer.get(position++);
+        int dumpTag = dumpBuffer.get(position++) & 0xFF;
         long size = 0;
         long tagOffset = position;
         int idSize = dumpBuffer.getIDSize();
@@ -917,6 +1065,100 @@ class HprofHeap implements Heap {
 
                 break;
             }
+
+             /* HPROF HEAP 1.0.3 tags */
+            case HEAP_DUMP_INFO: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_HEAP_DUMP_INFO"); // NOI18N
+                    int heapId = dumpBuffer.getInt(position);
+                    position += 4;
+
+                    long stringID = dumpBuffer.getID(position);
+                    position += idSize;
+                    System.out.println(" Dump info id " + heapId + " String ID " + stringID); // NOI18N
+                }
+
+                size = 4 + idSize;
+
+                break;
+            }
+            case ROOT_INTERNED_STRING: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_INTERNED_STRING"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+          }
+            case ROOT_FINALIZING: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_FINALIZING"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case ROOT_DEBUGGER: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_DEBUGGER"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case ROOT_REFERENCE_CLEANUP: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_REFERENCE_CLEANUP"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case ROOT_VM_INTERNAL: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_VM_INTERNAL"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case ROOT_JNI_MONITOR: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_ROOT_JNI_MONITOR"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case UNREACHABLE: {
+
+                if (DEBUG) {
+                    System.out.println("Tag HPROF_UNREACHABLE"); // NOI18N
+                }
+
+                size = idSize;
+
+                break;
+            }
+            case PRIMITIVE_ARRAY_NODATA_DUMP: {
+                    throw new IllegalArgumentException(
+                        "Don't know how to load a nodata array");
+                //break;
+            }
+
             default:throw new IllegalArgumentException("Invalid dump tag " + dumpTag + " at position " + (position - 1)); // NOI18N              
         }
 
@@ -931,36 +1173,19 @@ class HprofHeap implements Heap {
 
         //int time = dumpBuffer.getInt(start+1);
         long len = dumpBuffer.getInt(start + 1 + 4) & 0xFFFFFFFFL;  // len is unsigned int
-        offset[0] = start + 1 + 4 + 4 + len;
+         // only HEAP_DUMP_END can have zero length
+        if (len == 0 && tag != HEAP_DUMP_END && dumpBuffer.version != HprofByteBuffer.JAVA_PROFILE_1_0_3) {
+            // broken tag length
+            offset[0] = -1;
+        } else {
+            offset[0] = start + 1 + 4 + 4 + len;
+        }
 
         return tag;
     }
 
-    private Map computeGCRootsFor(TagBounds tagBounds) {
-        Map roots = new HashMap();
-
-        if (tagBounds != null) {
-            int rootTag = tagBounds.tag;
-            long[] offset = new long[] { tagBounds.startOffset };
-
-            while (offset[0] < tagBounds.endOffset) {
-                long start = offset[0];
-
-                if (readDumpTag(offset) == rootTag) {
-                    HprofGCRoot root;
-                    if (rootTag == ROOT_THREAD_OBJECT) {
-                        root = new ThreadObjectHprofGCRoot(this, start);                        
-                    } else if (rootTag == ROOT_JAVA_FRAME) {
-                        root = new JavaFrameHprofGCRoot(this, start);
-                    } else {
-                        root = new HprofGCRoot(this, start);
-                    }
-                    roots.put(Long.valueOf(root.getInstanceId()), root);
-                }
-            }
-        }
-
-        return roots;
+    TagBounds getHeapTagBound(int heapTag) {
+        return heapTagBounds[heapTag];
     }
 
     private TagBounds computeHeapDumpStart() throws IOException {
@@ -1004,6 +1229,7 @@ class HprofHeap implements Heap {
             return;
         }
 
+        HeapProgress.progressStart();
         heapTagBounds = new TagBounds[0x100];
 
         long[] offset = new long[] { heapDumpSegment.startOffset + 1 + 4 + 4 };
@@ -1042,7 +1268,7 @@ class HprofHeap implements Heap {
         HeapProgress.progressFinish();
     }
 
-    private void fillTagBounds(long tagStart) {
+    private void fillTagBounds(long tagStart) throws IOException {
         long[] offset = new long[] { tagStart };
 
         while (offset[0] < dumpBuffer.capacity()) {
@@ -1051,6 +1277,10 @@ class HprofHeap implements Heap {
             TagBounds bounds = tagBounds[tag];
             long end = offset[0];
 
+            if (end == -1) {
+                // tag with zero-length -> broken heap dump
+                throw new IOException("Heap dump is broken.\nTag 0x"+Integer.toHexString(tag)+" at offset "+start+" has zero length.");
+            }
             if (bounds == null) {
                 TagBounds newBounds;
 
