@@ -41,6 +41,9 @@
  */
 package org.netbeans.lib.profiler.heap;
 
+import java.io.DataInputStream;
+import java.io.DataOutputStream;
+import java.io.IOException;
 import java.util.Arrays;
 import java.util.Map;
 
@@ -222,7 +225,7 @@ class LongHashMap {
             if (item == k)
                 return tab[i + 1];
             if (item == 0)
-                return 0;
+                return -1;
             i = nextKeyIndex(i, len);
         }
     }
@@ -309,6 +312,7 @@ class LongHashMap {
      */
     long put(long key, long value) {
         assert key != 0;
+        assert value != -1;
         long k = key;
         long[] tab = table;
         int len = tab.length;
@@ -329,7 +333,7 @@ class LongHashMap {
         tab[i + 1] = value;
         if (++size >= threshold)
             resize(len); // len == 2 * current capacity.
-        return 0;
+        return -1;
     }
 
     /**
@@ -575,5 +579,24 @@ class LongHashMap {
         return result;
     }
 
+    //---- Serialization support
+    void writeToStream(DataOutputStream out) throws IOException {
+        out.writeInt(modCount);
+        out.writeInt(size);
+        out.writeInt(threshold);
+        out.writeInt(table.length);
+        for (int i = 0; i < table.length; i++) {
+            out.writeLong(table[i]);
+        }
+    }
 
+    LongHashMap(DataInputStream dis) throws IOException {
+        modCount = dis.readInt();
+        size = dis.readInt();
+        threshold = dis.readInt();
+        table = new long[dis.readInt()];
+        for (int i = 0; i < table.length; i++) {
+            table[i] = dis.readLong();
+        }
+    }
 }
