@@ -43,19 +43,24 @@
 
 package org.netbeans.lib.profiler.instrumentation;
 
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 import org.netbeans.lib.profiler.ProfilerEngineSettings;
+import org.netbeans.lib.profiler.classfile.BaseClassInfo;
+import org.netbeans.lib.profiler.classfile.ClassRepository;
 import org.netbeans.lib.profiler.classfile.DynamicClassInfo;
 import org.netbeans.lib.profiler.classfile.PlaceholderClassInfo;
 import org.netbeans.lib.profiler.client.RuntimeProfilingPoint;
-import org.netbeans.lib.profiler.global.InstrumentationFilter;
+import org.netbeans.lib.profiler.filters.InstrumentationFilter;
 import org.netbeans.lib.profiler.global.ProfilingSessionStatus;
-import java.util.ArrayList;
-import java.util.Enumeration;
-import java.util.Hashtable;
+import org.netbeans.lib.profiler.wireprotocol.RootClassLoadedCommand;
 
 /**
  * Base class for two "recursive" method scanners, implementing the "eager" and "lazy" transitive call subgraph revelation and
- * instrumentation schemes. This class contains functionality used by both scaners.
+ * instrumentation schemes. This class contains functionality used by both scanners.
  *
  * @author Tomas Hurka
  * @author Misha Dmitriev
@@ -94,7 +99,7 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
 
     //~ Instance fields ----------------------------------------------------------------------------------------------------------
 
-    protected Hashtable instrClasses = new Hashtable();
+    protected Map instrClasses = new HashMap();
     protected InstrumentationFilter instrFilter;
     protected byte[] codeBytes;
     protected boolean dontInstrumentEmptyMethods;
@@ -155,8 +160,7 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
      * This method should initialize instrumentation-related data structures, register given classes as loaded, and return
      * the initial set of methods to instrument in the format given by createInstrumentedMethodPack().
      */
-    abstract Object[] getInitialMethodsToInstrument(String[] loadedClasses, int[] loadedClassLoaderIds,
-                                                           byte[][] cachedClassFileBytes, RootMethods rootMethods);
+    abstract Object[] getInitialMethodsToInstrument(RootClassLoadedCommand rootLoaded, RootMethods rootMethods);
 
     public abstract Object[] getMethodsToInstrumentUponClassLoad(String className, int classLoaderId, boolean threadInCallGraph);
 
@@ -178,9 +182,7 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
     }
 
     protected void addToSubclassList(DynamicClassInfo clazz, DynamicClassInfo addedClassInfo) {
-        String superClassName = clazz.getSuperclassName();
-        int loaderId = clazz.getLoaderId();
-        DynamicClassInfo superClass = javaClassForName(superClassName, loaderId);
+        DynamicClassInfo superClass = getSuperClass(clazz);
         clazz.setSuperClass(superClass);
 
         if ((superClass != null) && !clazz.isInterface()) {
@@ -190,16 +192,16 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
                 findAndMarkOverridingMethodsReachable(superClass, addedClassInfo);
             }
 
-            if (superClassName != "java/lang/Object") { // NOI18N
+            if (superClass.getName() != OBJECT_SLASHED_CLASS_NAME) {
                 addToSubclassList(superClass, addedClassInfo);
             }
         }
 
-        String[] interfaces = clazz.getInterfaceNames();
+        DynamicClassInfo[] interfaces = getInterfaces(clazz);
 
         if (interfaces != null) {
             for (int i = 0; i < interfaces.length; i++) {
-                DynamicClassInfo superInterface = javaClassForName(interfaces[i], loaderId);
+                DynamicClassInfo superInterface = interfaces[i];
                 clazz.setSuperInterface(superInterface, i);
 
                 if (superInterface != null) {
@@ -456,8 +458,8 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
             // Check if java.lang.reflect.Method is already among classes to instrument
             int idx = 0;
 
-            for (Enumeration e = instrClasses.elements(); e.hasMoreElements(); idx++) {
-                DynamicClassInfo clazz = (DynamicClassInfo) e.nextElement();
+            for (Iterator e = instrClasses.values().iterator(); e.hasNext(); idx++) {
+                DynamicClassInfo clazz = (DynamicClassInfo) e.next();
 
                 if (clazz.getName() == JAVA_LANG_REFLECT_METHOD_SLASHED_CLASS_NAME) {
                     reflectMethodClassIdx = idx;
@@ -466,7 +468,7 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
                 }
             }
             if (reflectMethodClassIdx == -1) {
-                reflectMethodClass = javaClassForName(JAVA_LANG_REFLECT_METHOD_DOTTED_CLASS_NAME, 0);
+                reflectMethodClass = javaClassForName(JAVA_LANG_REFLECT_METHOD_SLASHED_CLASS_NAME, 0);
                 if (reflectMethodClass != null) {
                     nInstrClasses++;
                 }
@@ -484,8 +486,8 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
         int classIdx = 0;
         int methodIdx = 0;
 
-        for (Enumeration e = instrClasses.elements(); e.hasMoreElements();) {
-            DynamicClassInfo clazz = (DynamicClassInfo) e.nextElement();
+        for (Iterator e = instrClasses.values().iterator(); e.hasNext();) {
+            DynamicClassInfo clazz = (DynamicClassInfo) e.next();
             int nMethods = clazz.getMethodNames().length;
             instrMethodClasses[classIdx] = clazz.getName().replace('/', '.').intern(); // NOI18N
             instrClassLoaderIds[classIdx] = clazz.getLoaderId();
@@ -635,4 +637,43 @@ public abstract class RecursiveMethodInstrumentor extends ClassManager {
             nInstrClasses++;
         }
     }
+    
+    private DynamicClassInfo getSuperClass(DynamicClassInfo clazz) {
+        String superClassName = clazz.getSuperclassName();
+        
+        if (clazz.isLoaded()) {
+            List classes = ClassRepository.getAllClassVersions(superClassName);
+            if (classes != null && classes.size() == 1) {
+                BaseClassInfo bci = (BaseClassInfo)classes.get(0);
+                if (bci instanceof DynamicClassInfo) return (DynamicClassInfo)bci;
+            }
+        }
+        return javaClassForName(superClassName, clazz.getLoaderId());
+    }
+    
+    private DynamicClassInfo[] getInterfaces(DynamicClassInfo clazz) {
+        String[] interfaceNames = clazz.getInterfaceNames();
+        
+        if (interfaceNames != null) {
+            DynamicClassInfo[] interfaces = new DynamicClassInfo[interfaceNames.length];
+            int loaderId = clazz.getLoaderId();
+            boolean isLoaded = clazz.isLoaded();
+            
+            for (int i = 0; i < interfaceNames.length; i++) {
+                if (isLoaded) {
+                    List classes = ClassRepository.getAllClassVersions(interfaceNames[i]);
+                    if (classes != null && classes.size()==1) {
+                        BaseClassInfo bci = (BaseClassInfo)classes.get(0);
+                        if (bci instanceof DynamicClassInfo) {
+                            interfaces[i] = (DynamicClassInfo)classes.get(0);
+                            continue;
+                        }
+                    }
+                }
+                interfaces[i] = javaClassForName(interfaceNames[i], loaderId);
+            }
+            return interfaces;
+        }
+        return null;
+    }
 }
