@@ -51,8 +51,8 @@ import org.netbeans.lib.profiler.ProfilerClient;
 import org.netbeans.lib.profiler.TargetAppRunner;
 import org.netbeans.lib.profiler.client.ProfilingPointsProcessor;
 import org.netbeans.lib.profiler.client.RuntimeProfilingPoint;
+import org.netbeans.lib.profiler.filters.InstrumentationFilter;
 import org.netbeans.lib.profiler.global.CommonConstants;
-import org.netbeans.lib.profiler.global.InstrumentationFilter;
 import org.netbeans.lib.profiler.marker.Mark;
 import org.netbeans.lib.profiler.results.BaseCallGraphBuilder;
 import org.netbeans.lib.profiler.results.RuntimeCCTNode;
@@ -64,7 +64,6 @@ import org.netbeans.lib.profiler.results.cpu.cct.nodes.ServletRequestCPUCCTNode;
 import org.netbeans.lib.profiler.results.cpu.cct.nodes.SimpleCPUCCTNode;
 import org.netbeans.lib.profiler.results.cpu.cct.nodes.ThreadCPUCCTNode;
 import org.netbeans.lib.profiler.results.cpu.cct.nodes.TimedCPUCCTNode;
-import org.netbeans.lib.profiler.results.cpu.marking.MarkingEngine;
 
 
 /**
@@ -176,7 +175,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
                 }
             }
 
-            return (CPUCCTContainer[]) ccts.toArray(new CPUCCTContainer[ccts.size()]);
+            return (CPUCCTContainer[]) ccts.toArray(new CPUCCTContainer[0]);
         } finally {
             threadInfos.endTrans();
         }
@@ -195,7 +194,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
     }
 
     public void methodEntry(final int methodId, final int threadId, final int methodType, final long timeStamp0,
-                            final long timeStamp1) {
+                            final long timeStamp1, final List parameters, int[] methodIds) {
         if (!isReady() || (threadInfos.threadInfos == null)) {
             return;
         }
@@ -218,7 +217,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
                 break;
             }
             case METHODTYPE_MARKER: {
-                markerMethodEntry(methodId, ti, timeStamp0, timeStamp1);
+                markerMethodEntry(methodId, ti, timeStamp0, timeStamp1, parameters, methodIds);
 
                 break;
             }
@@ -227,7 +226,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
         batchNotEmpty = true;
     }
 
-    public void methodEntryUnstamped(final int methodId, final int threadId, final int methodType) {
+    public void methodEntryUnstamped(final int methodId, final int threadId, final int methodType, final List parameters, final int[] methodIds) {
         if (!isReady() || (threadInfos.threadInfos == null)) {
             return;
         }
@@ -245,7 +244,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
                 break;
             }
             case METHODTYPE_MARKER: {
-                markerMethodEntry(methodId, ti);
+                markerMethodEntry(methodId, ti, parameters, methodIds);
 
                 break;
             }
@@ -255,7 +254,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
     }
 
     public void methodExit(final int methodId, final int threadId, final int methodType, final long timeStamp0,
-                           final long timeStamp1) {
+                           final long timeStamp1, final Object retVal) {
         if (!isReady() || (threadInfos.threadInfos == null)) {
             return;
         }
@@ -845,7 +844,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
             rootGrossTimeAbs += (time0 - ti.rootMethodEntryTimeAbs);
 
             //System.err.println("*** dumpAbsTimeStamp = " + status.dumpAbsTimeStamp + ", ti.topMethodEntryTime0 = "
-            // + ti.topMethodEntryTime0 + "ti.rootMethodEntryTimeAbs = " + ti.rootMethodEntryTimeAbs); // NOI18N
+            // + ti.topMethodEntryTime0 + ", ti.rootMethodEntryTimeAbs = " + ti.rootMethodEntryTimeAbs); // NOI18N
             //System.err.println("*** Adjusted rootGrossTimeAbs = " + rootGrossTimeAbs); // NOI18N
         }
 
@@ -962,12 +961,12 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
 
             @Override
             public int getMinMethodId() {
-                return profilerClient.getStatus().getStartingMethodId();
+                return 1;
             }
 
             @Override
             public int getMaxMethodId() {
-                return profilerClient.getStatus().getNInstrMethods() + profilerClient.getStatus().getStartingMethodId() - 1;
+                return profilerClient.getStatus().getNInstrMethods();
             }
 
             @Override
@@ -1035,8 +1034,9 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
     }
 
     private TimedCPUCCTNode markerMethodEntry(final int methodId, final ThreadInfo ti, long timeStamp0, long timeStamp1,
-                                              boolean stamped) {
-        Mark mark = MarkingEngine.getDefault().markMethod(methodId, status);
+                                              boolean stamped, List parameters, int[] methodIds) {
+//        Mark mark = MarkingEngine.getDefault().markMethod(methodId, status);
+        Mark mark = Mark.DEFAULT;
 
         if (LOGGER.isLoggable(Level.FINEST)) {
             LOGGER.log(Level.FINEST, "MarkerMEntry{0} for tId = {1}, time: {2}, method:  {3}, inRoot: {4}, rootEntryTimeThread: {5}", new Object[]{(!stamped) ? "(unstamped)" : "", (int) ti.threadId, timeStamp0, debugMethod(methodId), ti.rootMethodEntryTimeAbs, ti.rootMethodEntryTimeThreadCPU});
@@ -1144,7 +1144,7 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
                 ProfilerClient client = getClient();
 
                 if (client != null) {
-                    if (!client.getSettings().getInstrumentationFilter().passesFilter(jvmClassName)) {
+                    if (!client.getSettings().getInstrumentationFilter().passes(jvmClassName)) {
                         curNode.setFilteredStatus(TimedCPUCCTNode.FILTERED_YES);
                     }
                 } else {
@@ -1158,12 +1158,12 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
         return curNode;
     }
 
-    private TimedCPUCCTNode markerMethodEntry(final int methodId, final ThreadInfo ti, long timeStamp0, long timeStamp1) {
-        return markerMethodEntry(methodId, ti, timeStamp0, timeStamp1, true);
+    private TimedCPUCCTNode markerMethodEntry(final int methodId, final ThreadInfo ti, long timeStamp0, long timeStamp1, List parameters, int[] methodIds) {
+        return markerMethodEntry(methodId, ti, timeStamp0, timeStamp1, true, parameters, methodIds);
     }
 
-    private TimedCPUCCTNode markerMethodEntry(final int methodId, final ThreadInfo ti) {
-        return markerMethodEntry(methodId, ti, 0, 0, false);
+    private TimedCPUCCTNode markerMethodEntry(final int methodId, final ThreadInfo ti, List parameters, int[] methodIds) {
+        return markerMethodEntry(methodId, ti, 0, 0, false, parameters, methodIds);
     }
 
     private TimedCPUCCTNode plainMethodEntry(final int methodId, final ThreadInfo ti, long timeStamp0, long timeStamp1,
@@ -1219,11 +1219,11 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
         if (!ti.isInRoot()) {
             try {
                 methodInfoMapper.lock(false);
-                String jvmClassName = methodInfoMapper.getInstrMethodClass(((MethodCPUCCTNode) curNode).getMethodId()).replace('.', '/');
+                String jvmClassName = methodInfoMapper.getInstrMethodClass(((MethodCPUCCTNode) curNode).getMethodId()).replace('.', '/'); // NOI18N
                 ProfilerClient client = getClient();
 
                 if (client != null) {
-                    if (!client.getSettings().getInstrumentationFilter().passesFilter(jvmClassName)) {
+                    if (!client.getSettings().getInstrumentationFilter().passes(jvmClassName)) {
                         curNode.setFilteredStatus(TimedCPUCCTNode.FILTERED_YES);
                     }
                 } else {
@@ -1342,7 +1342,8 @@ public class CPUCallGraphBuilder extends BaseCallGraphBuilder implements CPUProf
             LOGGER.log(Level.FINEST, "RootMEntry for tId = {0}, time: {1}, method:  {2}", new Object[]{(int) ti.threadId, timeStamp0, debugMethod(methodId)});
         }
 
-        Mark mark = MarkingEngine.getDefault().markMethod(methodId, status);
+//        Mark mark = MarkingEngine.getDefault().markMethod(methodId, status);
+        Mark mark = Mark.DEFAULT;
 
         TimedCPUCCTNode curNode = ti.peek();
 
