@@ -53,8 +53,8 @@ import java.util.logging.Level;
 import org.netbeans.lib.profiler.ProfilerClient;
 import org.netbeans.lib.profiler.ProfilerLogger;
 import org.netbeans.lib.profiler.client.ClientUtils.TargetAppOrVMTerminated;
+import org.netbeans.lib.profiler.filters.InstrumentationFilter;
 import org.netbeans.lib.profiler.global.CommonConstants;
-import org.netbeans.lib.profiler.global.InstrumentationFilter;
 import org.netbeans.lib.profiler.results.ProfilingResultListener;
 import org.netbeans.lib.profiler.results.cpu.StackTraceSnapshotBuilder.SampledThreadInfo;
 import org.netbeans.lib.profiler.results.locks.AbstractLockDataFrameProcessor;
@@ -220,15 +220,15 @@ public class CPUSamplingDataFrameProcessor extends AbstractLockDataFrameProcesso
                     break;
                 }
             }
-            try {
-                methodIdsTable.getNamesForMethodIds(client);
-            } catch (TargetAppOrVMTerminated ex) {
-                ProfilerLogger.log(ex.getMessage());
-                return;
-            }
-            processCollectedDumps(methodIdsTable,threadDumps);
-            threadDumps.clear();
         }
+        try {
+            methodIdsTable.getNamesForMethodIds(client);
+        } catch (TargetAppOrVMTerminated ex) {
+            ProfilerLogger.log(ex.getMessage());
+            return;
+        }
+        processCollectedDumps(methodIdsTable,threadDumps);
+        threadDumps.clear();
     }
 
     public void startup(ProfilerClient client) {
@@ -246,6 +246,11 @@ public class CPUSamplingDataFrameProcessor extends AbstractLockDataFrameProcesso
         builder = new StackTraceSnapshotBuilder(ccgb[0],client.getSettings().getInstrumentationFilter(),client.getStatus());
     }
     
+    public void shutdown() {
+        super.shutdown();
+        builder = null;
+    }
+  
     private static Thread.State getThreadState(int threadState) {
         switch (threadState) {
             case CommonConstants.THREAD_STATUS_UNKNOWN:
@@ -285,8 +290,9 @@ public class CPUSamplingDataFrameProcessor extends AbstractLockDataFrameProcesso
                     if (el == null) {
                         JMethodIdTableEntry entry = methodIdTable.getEntry(methodId);
                         String method = formatter.formatMethodName(entry.className, entry.methodName, entry.methodSig).toFormatted();
-                        String className = entry.className.replace('/','.');
-                        el = new StackTraceElement(className, method, entry.methodSig, -1);
+                        String className = entry.className.replace('/','.'); // NOI18N
+                        String methodName = method+StackTraceSnapshotBuilder.NAME_SIG_SPLITTER+entry.methodSig;
+                        el = new StackTraceElement(className, methodName, null, entry.isNative?-2:-1);
                         stackTraceElements.put(Integer.valueOf(methodId),el);
                     }
                     stackTrace[i] = el;
@@ -317,7 +323,7 @@ public class CPUSamplingDataFrameProcessor extends AbstractLockDataFrameProcesso
         
         ThreadDump(long ts, Map<Integer,ThreadInfo> threadsMap) {
             timestamp = ts;
-            threadDumps = threadsMap.values().toArray(new ThreadInfo[threadsMap.size()]);
+            threadDumps = threadsMap.values().toArray(new ThreadInfo[0]);
         }
     }    
 }
