@@ -43,6 +43,11 @@
 
 package org.netbeans.lib.profiler.results.memory;
 
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
 import org.netbeans.lib.profiler.ProfilerClient;
 import org.netbeans.lib.profiler.client.ClientUtils;
 import org.netbeans.lib.profiler.results.CCTNode;
@@ -51,6 +56,7 @@ import org.netbeans.lib.profiler.utils.formatting.MethodNameFormatterFactory;
 import org.netbeans.lib.profiler.results.ExportDataDumper;
 import java.util.ResourceBundle;
 import org.netbeans.lib.profiler.results.FilterSortSupport;
+import org.netbeans.lib.profiler.utils.Wildcards;
 
 
 /**
@@ -63,7 +69,7 @@ import org.netbeans.lib.profiler.results.FilterSortSupport;
  * @author Tomas Hurka
  * @author Misha Dmitriev
  */
-public class PresoObjAllocCCTNode implements CCTNode {
+public class PresoObjAllocCCTNode extends CCTNode {
     //~ Static fields/initializers -----------------------------------------------------------------------------------------------
 
     public static final String VM_ALLOC_CLASS = "org.netbeans.lib.profiler.server.ProfilerRuntimeMemory"; // NOI18N
@@ -76,24 +82,41 @@ public class PresoObjAllocCCTNode implements CCTNode {
     public static final int SORT_BY_ALLOC_OBJ_SIZE = 2;
     public static final int SORT_BY_ALLOC_OBJ_NUMBER = 3;
     
-    protected static final char MASK_FILTERED_NODE = 0x8;
+//    protected static final char MASK_FILTERED_NODE = 0x8;
 
     //~ Instance fields ----------------------------------------------------------------------------------------------------------
 
     public long nCalls;
     public long totalObjSize;
-    PresoObjAllocCCTNode parent;
+    public PresoObjAllocCCTNode parent;
     String className;
     String methodName;
-    String methodSig;
+//    String methodSig;
     String nodeName;
-    PresoObjAllocCCTNode[] children;
+    public PresoObjAllocCCTNode[] children;
     int methodId;
+    JMethodIdTable.JMethodIdTableEntry entry;
     
     protected char flags;
 
     //~ Constructors -------------------------------------------------------------------------------------------------------------
-
+    
+    public static PresoObjAllocCCTNode rootNode(PresoObjAllocCCTNode[] children) {
+        PresoObjAllocCCTNode root = new PresoObjAllocCCTNode();
+        root.setChildren(children);
+        return root;
+    }
+    
+    public PresoObjAllocCCTNode(String className, long nCalls, long totalObjSize) {
+        this.className = className;
+        this.nCalls = nCalls;
+        this.totalObjSize = totalObjSize;
+        
+        methodName = Wildcards.ALLWILDCARD;
+    }
+    
+    PresoObjAllocCCTNode() {}
+    
     protected PresoObjAllocCCTNode(RuntimeMemoryCCTNode rtNode) {
         methodId = rtNode.methodId;
 
@@ -103,6 +126,63 @@ public class PresoObjAllocCCTNode implements CCTNode {
             totalObjSize += rtTermNode.totalObjSize;
         }
     }
+    
+    
+    protected final void setChildren(PresoObjAllocCCTNode[] children) {
+        this.children = children;
+        for (PresoObjAllocCCTNode child : children) child.parent = this;
+    }
+    
+    
+    // --- Filtering support
+    
+    public CCTNode createFilteredNode() {
+        PresoObjAllocCCTNode filtered = new PresoObjAllocCCTNode();
+        setupFilteredNode(filtered);        
+        return filtered;
+    }
+    
+    protected void setupFilteredNode(PresoObjAllocCCTNode filtered) {
+        filtered.setFilteredNode();
+         
+        filtered.parent = parent;
+
+        filtered.nCalls = nCalls;
+        filtered.totalObjSize = totalObjSize;
+
+        Collection<PresoObjAllocCCTNode> _childrenL = resolveChildren(this);
+        filtered.children = _childrenL.toArray(new PresoObjAllocCCTNode[0]);
+    }
+    
+    public void merge(CCTNode node) {
+        if (node instanceof PresoObjAllocCCTNode) {
+            PresoObjAllocCCTNode _node = (PresoObjAllocCCTNode)node;
+            
+            nCalls += _node.nCalls;
+            totalObjSize += _node.totalObjSize;
+
+            List<CCTNode> ch = new ArrayList();
+            
+            // Include current children
+            if (children != null) ch.addAll(Arrays.asList(children));
+            
+            // Add or merge new children
+            for (PresoObjAllocCCTNode child : resolveChildren(_node)) {
+                int idx = ch.indexOf(child);
+                if (idx == -1) ch.add(child);
+                else ch.get(idx).merge(child);
+            }
+            
+            children = ch.toArray(new PresoObjAllocCCTNode[0]);
+        }
+    }
+
+    protected static Collection<PresoObjAllocCCTNode> resolveChildren(PresoObjAllocCCTNode node) {
+        PresoObjAllocCCTNode[] chldrn = (PresoObjAllocCCTNode[])node.getChildren();
+        return chldrn == null ? Collections.EMPTY_LIST : Arrays.asList(chldrn);
+    }
+    
+    // ---
 
     //~ Methods ------------------------------------------------------------------------------------------------------------------
 
@@ -121,11 +201,11 @@ public class PresoObjAllocCCTNode implements CCTNode {
         JMethodIdTable.getDefault().getNamesForMethodIds(profilerClient);
     }
 
-    public static PresoObjAllocCCTNode createPresentationCCTFromSnapshot(MemoryResultsSnapshot snapshot,
+    public static PresoObjAllocCCTNode createPresentationCCTFromSnapshot(JMethodIdTable methodIdTable,
                                                                          RuntimeMemoryCCTNode rootRuntimeNode,
                                                                          String classTypeName) {
         PresoObjAllocCCTNode rootNode = generateMirrorNode(rootRuntimeNode);
-        assignNamesToNodesFromSnapshot(snapshot, rootNode, classTypeName);
+        assignNamesToNodesFromSnapshot(methodIdTable, rootNode, classTypeName);
 
         return rootNode;
     }
@@ -162,7 +242,7 @@ public class PresoObjAllocCCTNode implements CCTNode {
     }
 
     public String[] getMethodClassNameAndSig() {
-        return new String[] { className, methodName, methodSig };
+        return new String[] { getClassName(), getMethodName(), getMethodSig()};
     }
 
     public int getNChildren() {
@@ -174,15 +254,24 @@ public class PresoObjAllocCCTNode implements CCTNode {
     }
 
     public String getNodeName() {
-        if (isFilteredNode()) {
-            return FilterSortSupport.FILTERED_OUT_LBL;
-        } else if (methodId != 0) {
-            return nodeName;
-        } else if (className != null) {
-            return className;
-        } else {
-            return UKNOWN_NODENAME;
+        if (nodeName == null) {
+            if (isFiltered()) {
+                nodeName = FilterSortSupport.FILTERED_OUT_LBL;
+            } else if (methodId != 0) {
+                if (VM_ALLOC_CLASS.equals(getClassName()) && VM_ALLOC_METHOD.equals(getMethodName())) { // special handling of ProfilerRuntimeMemory.traceVMObjectAlloc
+                    nodeName = VM_ALLOC_TEXT;
+                } else {
+                    nodeName = MethodNameFormatterFactory.getDefault().getFormatter().formatMethodName(
+                                        getClassName(), getMethodName(), getMethodSig()).toFormatted();
+                }
+            } else if (getClassName() != null) {
+                nodeName = getClassName();
+            } else {
+                nodeName = UKNOWN_NODENAME;
+            }
         }
+        
+        return nodeName;
     }
 
     public CCTNode getParent() {
@@ -190,80 +279,99 @@ public class PresoObjAllocCCTNode implements CCTNode {
     }
 
     public void sortChildren(int sortBy, boolean sortOrder) {
-        int nChildren = getNChildren();
-
-        if (nChildren == 0) {
-            return;
-        }
-
-        for (int i = 0; i < nChildren; i++) {
-            children[i].sortChildren(sortBy, sortOrder);
-        }
-
-        if (nChildren > 1) {
-            switch (sortBy) {
-                case SORT_BY_NAME:
-                    sortChildrenByName(sortOrder);
-
-                    break;
-                case SORT_BY_ALLOC_OBJ_SIZE:
-                    sortChildrenByAllocObjSize(sortOrder);
-
-                    break;
-                case SORT_BY_ALLOC_OBJ_NUMBER:
-                    sortChildrenByAllocObjNumber(sortOrder);
-
-                    break;
-            }
-        }
+//        int nChildren = getNChildren();
+//
+//        if (nChildren == 0) {
+//            return;
+//        }
+//
+//        for (int i = 0; i < nChildren; i++) {
+//            children[i].sortChildren(sortBy, sortOrder);
+//        }
+//
+//        if (nChildren > 1) {
+//            switch (sortBy) {
+//                case SORT_BY_NAME:
+//                    sortChildrenByName(sortOrder);
+//
+//                    break;
+//                case SORT_BY_ALLOC_OBJ_SIZE:
+//                    sortChildrenByAllocObjSize(sortOrder);
+//
+//                    break;
+//                case SORT_BY_ALLOC_OBJ_NUMBER:
+//                    sortChildrenByAllocObjNumber(sortOrder);
+//
+//                    break;
+//            }
+//        }
     }
 
     public String toString() {
         return getNodeName();
     }
     
-    public void setFilteredNode() {
-        flags |= MASK_FILTERED_NODE;
-    }
+//    public void setFilteredNode() {
+//        flags |= MASK_FILTERED_NODE;
+//    }
+//    
+//    public void resetFilteredNode() {
+//        flags &= ~MASK_FILTERED_NODE;
+//    }
+//
+//    public boolean isFilteredNode() {
+//        return (flags & MASK_FILTERED_NODE) != 0;
+//    }
     
-    public void resetFilteredNode() {
-        flags &= ~MASK_FILTERED_NODE;
-    }
-
-    public boolean isFilteredNode() {
-        return (flags & MASK_FILTERED_NODE) != 0;
-    }
-    
-    void merge(PresoObjAllocCCTNode node) {
-        nCalls += node.nCalls;
-        totalObjSize += totalObjSize;
-        
-        if (node.children != null) {
-            for (PresoObjAllocCCTNode ch : node.children)
-                ch.parent = this;
-            
-            int chl = children == null ? 0 : children.length;
-            int newchl = node.children.length;
-            PresoObjAllocCCTNode[] newch = new PresoObjAllocCCTNode[chl + newchl];
-            if (children != null) System.arraycopy(children, 0, newch, 0, chl);
-            System.arraycopy(node.children, 0, newch, chl, newchl);
-            children = newch;
-        }
-    }
+//    void merge(PresoObjAllocCCTNode node) {
+//        nCalls += node.nCalls;
+//        totalObjSize += totalObjSize;
+//        
+//        if (node.children != null) {
+//            for (PresoObjAllocCCTNode ch : node.children)
+//                ch.parent = this;
+//            
+//            int chl = children == null ? 0 : children.length;
+//            int newchl = node.children.length;
+//            PresoObjAllocCCTNode[] newch = new PresoObjAllocCCTNode[chl + newchl];
+//            if (children != null) System.arraycopy(children, 0, newch, 0, chl);
+//            System.arraycopy(node.children, 0, newch, chl, newchl);
+//            children = newch;
+//        }
+//    }
     
     public boolean equals(Object o) {
+        if (o == this) return true;
         if (!(o instanceof PresoObjAllocCCTNode)) return false;
-        return getNodeName().equals(((PresoObjAllocCCTNode)o).getNodeName());
+        PresoObjAllocCCTNode other = (PresoObjAllocCCTNode)o;
+        if (isFiltered()) {
+            return other.isFiltered();
+        }
+        if (other.isFiltered()) {
+            return false;
+        }
+        if (methodId == 0) {
+            return getNodeName().equals(other.getNodeName());
+        }
+        if (other.methodId == 0) {
+            return false;
+        }
+        return entry.className.equals(other.entry.className) &&
+               entry.methodName.equals(other.entry.methodName) &&
+               entry.methodSig.equals(other.entry.methodSig);
     }
     
     public int hashCode() {
-        return getNodeName().hashCode();
+        if (methodId == 0 || isFiltered()) {
+            return getNodeName().hashCode();
+        }
+        return entry.className.hashCode() ^ entry.methodName.hashCode() ^ entry.methodSig.hashCode();
     }
 
-    protected static void assignNamesToNodesFromSnapshot(MemoryResultsSnapshot snapshot, PresoObjAllocCCTNode rootNode,
+    protected static void assignNamesToNodesFromSnapshot(JMethodIdTable methodIdTable, PresoObjAllocCCTNode rootNode,
                                                          String classTypeName) {
         rootNode.className = StringUtils.userFormClassName(classTypeName);
-        rootNode.setFullClassAndMethodInfo(snapshot.getJMethodIdTable());
+        rootNode.setFullClassAndMethodInfo(methodIdTable);
     }
 
     protected static void assignNamesToNodesFromVM(ProfilerClient profilerClient, PresoObjAllocCCTNode rootNode,
@@ -310,22 +418,12 @@ public class PresoObjAllocCCTNode implements CCTNode {
 
     protected boolean setFullClassAndMethodInfo(JMethodIdTable methodIdTable) {
         if (methodId != 0) {
-            JMethodIdTable.JMethodIdTableEntry entry = methodIdTable.getEntry(methodId);
-            className = entry.className.replace('/', '.'); // NOI18N
-            methodName = entry.methodName;
-            methodSig = entry.methodSig;
-
-            if (VM_ALLOC_CLASS.equals(className) && VM_ALLOC_METHOD.equals(methodName)) { // special handling of ProfilerRuntimeMemory.traceVMObjectAlloc
-                nodeName = VM_ALLOC_TEXT;
-            } else {
-                nodeName = MethodNameFormatterFactory.getDefault().getFormatter()
-                                                     .formatMethodName(className, methodName, methodSig).toFormatted();
-            }
+            entry = methodIdTable.getEntry(methodId);
         }
 
         // If any object allocations that happen in our own code are caught (which shouldn't happen),
         // make sure to conceal this data here.
-        boolean thisNodeOk = !"org.netbeans.lib.profiler.server.ProfilerServer".equals(className); // NOI18N
+        boolean thisNodeOk = entry!=null && !"org/netbeans/lib/profiler/server/ProfilerServer".equals(entry.className); // NOI18N
         boolean childrenOk = true;
 
         if (children != null) {
@@ -362,7 +460,7 @@ public class PresoObjAllocCCTNode implements CCTNode {
                 children = newChildren;
             }
 
-            if ((methodName == null) || (methodName.equals("main") && methodSig // NOI18N
+            if ((getMethodName() == null) || (getMethodName().equals("main") && getMethodSig() // NOI18N
                 .equals("([Ljava/lang/String;)V"))) { // NOI18N
 
                 return true;
@@ -394,109 +492,109 @@ public class PresoObjAllocCCTNode implements CCTNode {
         }
     }
 
-    protected void sortChildrenByAllocObjNumber(boolean sortOrder) {
-        int len = children.length;
-        long[] values = new long[len];
-
-        for (int i = 0; i < len; i++) {
-            values[i] = children[i].nCalls;
-        }
-
-        sortLongs(values, sortOrder);
-    }
-
-    protected void sortChildrenByAllocObjSize(boolean sortOrder) {
-        int len = children.length;
-        long[] values = new long[len];
-
-        for (int i = 0; i < len; i++) {
-            values[i] = children[i].totalObjSize;
-        }
-
-        sortLongs(values, sortOrder);
-    }
-
-    protected void sortChildrenByName(boolean sortOrder) {
-        int len = children.length;
-        String[] values = new String[len];
-
-        for (int i = 0; i < len; i++) {
-            values[i] = children[i].getNodeName();
-        }
-
-        sortStrings(values, sortOrder);
-    }
-
-    protected void sortFloats(float[] values, boolean sortOrder) {
-        int len = values.length;
-
-        // Just the insertion sort - we will never get really large arrays here
-        for (int i = 0; i < len; i++) {
-            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
-                float tmp = values[j];
-                values[j] = values[j - 1];
-                values[j - 1] = tmp;
-
-                PresoObjAllocCCTNode tmpCh = children[j];
-                children[j] = children[j - 1];
-                children[j - 1] = tmpCh;
-            }
-        }
-    }
-
-    protected void sortInts(int[] values, boolean sortOrder) {
-        int len = values.length;
-
-        // Just the insertion sort - we will never get really large arrays here
-        for (int i = 0; i < len; i++) {
-            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
-                int tmp = values[j];
-                values[j] = values[j - 1];
-                values[j - 1] = tmp;
-
-                PresoObjAllocCCTNode tmpCh = children[j];
-                children[j] = children[j - 1];
-                children[j - 1] = tmpCh;
-            }
-        }
-    }
-
-    protected void sortLongs(long[] values, boolean sortOrder) {
-        int len = values.length;
-
-        // Just the insertion sort - we will never get really large arrays here
-        for (int i = 0; i < len; i++) {
-            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
-                long tmp = values[j];
-                values[j] = values[j - 1];
-                values[j - 1] = tmp;
-
-                PresoObjAllocCCTNode tmpCh = children[j];
-                children[j] = children[j - 1];
-                children[j - 1] = tmpCh;
-            }
-        }
-    }
-
-    protected void sortStrings(String[] values, boolean sortOrder) {
-        int len = values.length;
-
-        // Just the insertion sort - we will never get really large arrays here
-        for (int i = 0; i < len; i++) {
-            for (int j = i;
-                     (j > 0)
-                     && ((sortOrder == false) ? (values[j - 1].compareTo(values[j]) < 0) : (values[j - 1].compareTo(values[j]) > 0));
-                     j--) {
-                String tmp = values[j];
-                values[j] = values[j - 1];
-                values[j - 1] = tmp;
-
-                PresoObjAllocCCTNode tmpCh = children[j];
-                children[j] = children[j - 1];
-                children[j - 1] = tmpCh;
-            }
-        }
-    }
+//    protected void sortChildrenByAllocObjNumber(boolean sortOrder) {
+//        int len = children.length;
+//        long[] values = new long[len];
+//
+//        for (int i = 0; i < len; i++) {
+//            values[i] = children[i].nCalls;
+//        }
+//
+//        sortLongs(values, sortOrder);
+//    }
+//
+//    protected void sortChildrenByAllocObjSize(boolean sortOrder) {
+//        int len = children.length;
+//        long[] values = new long[len];
+//
+//        for (int i = 0; i < len; i++) {
+//            values[i] = children[i].totalObjSize;
+//        }
+//
+//        sortLongs(values, sortOrder);
+//    }
+//
+//    protected void sortChildrenByName(boolean sortOrder) {
+//        int len = children.length;
+//        String[] values = new String[len];
+//
+//        for (int i = 0; i < len; i++) {
+//            values[i] = children[i].getNodeName();
+//        }
+//
+//        sortStrings(values, sortOrder);
+//    }
+//
+//    protected void sortFloats(float[] values, boolean sortOrder) {
+//        int len = values.length;
+//
+//        // Just the insertion sort - we will never get really large arrays here
+//        for (int i = 0; i < len; i++) {
+//            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
+//                float tmp = values[j];
+//                values[j] = values[j - 1];
+//                values[j - 1] = tmp;
+//
+//                PresoObjAllocCCTNode tmpCh = children[j];
+//                children[j] = children[j - 1];
+//                children[j - 1] = tmpCh;
+//            }
+//        }
+//    }
+//
+//    protected void sortInts(int[] values, boolean sortOrder) {
+//        int len = values.length;
+//
+//        // Just the insertion sort - we will never get really large arrays here
+//        for (int i = 0; i < len; i++) {
+//            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
+//                int tmp = values[j];
+//                values[j] = values[j - 1];
+//                values[j - 1] = tmp;
+//
+//                PresoObjAllocCCTNode tmpCh = children[j];
+//                children[j] = children[j - 1];
+//                children[j - 1] = tmpCh;
+//            }
+//        }
+//    }
+//
+//    protected void sortLongs(long[] values, boolean sortOrder) {
+//        int len = values.length;
+//
+//        // Just the insertion sort - we will never get really large arrays here
+//        for (int i = 0; i < len; i++) {
+//            for (int j = i; (j > 0) && ((sortOrder == false) ? (values[j - 1] < values[j]) : (values[j - 1] > values[j])); j--) {
+//                long tmp = values[j];
+//                values[j] = values[j - 1];
+//                values[j - 1] = tmp;
+//
+//                PresoObjAllocCCTNode tmpCh = children[j];
+//                children[j] = children[j - 1];
+//                children[j - 1] = tmpCh;
+//            }
+//        }
+//    }
+//
+//    protected void sortStrings(String[] values, boolean sortOrder) {
+//        int len = values.length;
+//
+//        // Just the insertion sort - we will never get really large arrays here
+//        for (int i = 0; i < len; i++) {
+//            for (int j = i;
+//                     (j > 0)
+//                     && ((sortOrder == false) ? (values[j - 1].compareTo(values[j]) < 0) : (values[j - 1].compareTo(values[j]) > 0));
+//                     j--) {
+//                String tmp = values[j];
+//                values[j] = values[j - 1];
+//                values[j - 1] = tmp;
+//
+//                PresoObjAllocCCTNode tmpCh = children[j];
+//                children[j] = children[j - 1];
+//                children[j - 1] = tmpCh;
+//            }
+//        }
+//    }
 
     public void exportXMLData(ExportDataDumper eDD,String indent) {
         String newline = System.getProperty("line.separator"); // NOI18N
@@ -558,7 +656,7 @@ public class PresoObjAllocCCTNode implements CCTNode {
         for (int i=0; i<depth; i++) {
             result.append(indent); // to simulate the tree structure in CSV
         }
-        result.append((nodeName==null)?(className):(nodeName)).append(quote).append(separator);
+        result.append(getNodeName()).append(quote).append(separator);
         result.append(quote).append(totalObjSize).append(quote).append(separator);
         result.append(quote).append(nCalls).append(quote).append(separator);
         result.append(quote).append((getParent()==null)?("none"):(((PresoObjAllocCCTNode)getParent()).getNodeName())).append(newLine); // NOI18N
@@ -570,4 +668,50 @@ public class PresoObjAllocCCTNode implements CCTNode {
             }
         }
     }
+
+    String getClassName() {
+        if (className == null && entry != null) {
+            className = entry.className.replace('/', '.'); // NOI18N
+        }
+        return className;
+    }
+
+    String getMethodName() {
+        if (methodName == null && entry != null) {
+            methodName = entry.methodName;
+            if (entry.isNative) {
+                methodName = methodName.concat(JMethodIdTable.NATIVE_SUFFIX);
+            }   
+        }
+        return methodName;
+    }
+
+    String getMethodSig() {
+        if (entry != null) {
+            return entry.methodSig;
+        }
+        return null;
+    }
+    
+    static class Handle {
+
+        final PresoObjAllocCCTNode node;
+        
+        Handle(PresoObjAllocCCTNode n) {
+            node = n;
+        }
+
+        public int hashCode() {
+            return node.hashCode();
+        }
+
+        public boolean equals(Object obj) {
+            if (obj == this) return true;
+            
+            return node.equals(((Handle)obj).node);
+        }
+        
+        
+        
+    }
 }
