@@ -46,10 +46,13 @@ package org.netbeans.lib.profiler.server;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.lang.ref.WeakReference;
 import java.lang.reflect.Method;
 import java.net.URL;
 import java.text.MessageFormat;
+import java.util.HashMap;
 import java.util.LinkedHashMap;
+import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.WeakHashMap;
 import org.netbeans.lib.profiler.global.CommonConstants;
@@ -85,7 +88,7 @@ public class ProfilerInterface implements CommonConstants {
 
         public void run() {
             RootClassLoadedCommand cmd = new RootClassLoadedCommand(new String[] { "*FAKE_CLASS_1*", "*FAKE_CLASS_2*" }, // NOI18N
-                                                                    new int[] { 0, 0 }, null, 2, new int[] { -1 }); // NOI18N
+                                                                    new int[] { 0, 0 }, null, new int[] { 0, 0 }, new int[2][], 2, new int[] { -1 });
             profilerServer.sendComplexCmdToClient(cmd);
 
             InstrumentMethodGroupResponse imgr = (InstrumentMethodGroupResponse) profilerServer.getLastResponse();
@@ -138,7 +141,7 @@ public class ProfilerInterface implements CommonConstants {
                         initiateInstrumentation(instrType);
                         break;
                     case INSTR_NONE_MEMORY_SAMPLING:
-                        if (Histogram.isAvailable()) {
+                        if (Histogram.initialize(Platform.getJDKVersionNumber() >= Platform.JDK_19)) {
                             profilerServer.notifyClientOnResultsAvailability();
                         }
                         break;
@@ -180,13 +183,13 @@ public class ProfilerInterface implements CommonConstants {
                 e.printStackTrace(System.err);
             }
 
-            if (Platform.getJDKVersionNumber() == CommonConstants.JDK_19) {
+            if (Platform.getJDKVersionNumber() >= CommonConstants.JDK_19) {
                 try {
                     // preload classes for classLoadHook()
-                    Class.forName("java.lang.reflect.WeakPairMap$Pair"); // NOI18N
-                    Class.forName("java.lang.reflect.WeakPairMap$WeakRefPeer");    // NOI18N
-                    Class.forName("java.lang.reflect.WeakPairMap$Pair$Weak"); // NOI18N
-                    Class.forName("java.lang.reflect.WeakPairMap$Pair$Weak$1"); // NOI18N
+                    Class.forName("java.lang.WeakPairMap$Pair"); // NOI18N
+                    Class.forName("java.lang.WeakPairMap$WeakRefPeer");    // NOI18N
+                    Class.forName("java.lang.WeakPairMap$Pair$Weak"); // NOI18N
+                    Class.forName("java.lang.WeakPairMap$Pair$Weak$1"); // NOI18N
                 } catch (ClassNotFoundException e) {
                     e.printStackTrace(System.err);
                 }
@@ -208,9 +211,12 @@ public class ProfilerInterface implements CommonConstants {
                 } // ignore
             }
 
+            // reset loadedClassesArray so that next time it contains current loaded classes
+            loadedClassesArray = null;
+            loadedClassesLoaders = null;
             Classes.enableClassLoadHook();
 
-            boolean instrSpawnedThreads = cmd.getInstrSpawnedThreads();
+            instrSpawnedThreads = cmd.getInstrSpawnedThreads();
 
             if (targetAppRunning || hasAnyCoreClassNames(cmd.getRootClassNames()) || instrSpawnedThreads
                     || (instrType == INSTR_OBJECT_ALLOCATIONS) || (instrType == INSTR_OBJECT_LIVENESS)) {
@@ -250,6 +256,10 @@ public class ProfilerInterface implements CommonConstants {
 
                     rootClassLoaded = true; // See the comment in classLoadHook why it's worth setting rootClassLoaded
                                             // to true after the first instrumentation, not before
+                } else {
+                    // if root class is not loaded, reset loadedClassesArray so that next time it contains current loaded classes
+                    loadedClassesArray = null;
+                    loadedClassesLoaders = null;
                 }
             }
             Monitors.exitServerState();
@@ -278,6 +288,17 @@ public class ProfilerInterface implements CommonConstants {
     //~ Static fields/initializers -----------------------------------------------------------------------------------------------
 
     private static final int REDEFINE_CHUNK_SIZE = 500;
+    
+    static final char BOOLEAN = 'Z'; // NOI18N
+    static final char CHAR = 'C'; // NOI18N
+    static final char BYTE = 'B'; // NOI18N
+    static final char SHORT = 'S'; // NOI18N
+    static final char INT = 'I'; // NOI18N
+    static final char LONG = 'J'; // NOI18N
+    static final char FLOAT = 'F'; // NOI18N
+    static final char DOUBLE = 'D'; // NOI18N
+    static final char VOID = 'V'; // NOI18N
+    static final char REFERENCE = 'L'; // NOI18N
 
     // -----
     // I18N String constants
@@ -286,7 +307,8 @@ public class ProfilerInterface implements CommonConstants {
     private static String INTERNAL_ERROR_MSG = "Internal error:\nExpected InstrumentMethodGroupResponse, got response of class {0},\nvalue = {1}\nAll instrumentation will be removed"; // NOI18N
     private static String UNEXPECTED_EXCEPTION_MSG = "Unexpected exception caught when trying to instrument classes.\nOriginal exception:\n{0}\nStack trace:\n\n{1}"; // NOI18N
     private static String INSTRUMENTATION_SUCCESSFUL_MSG = "Deferred instrumentation performed successfully"; // NOI18N
-                                                                                                              // -----
+    private static String HISTOGRAM_NOT_AVAILABLE_MSG = "Histogram is not available."; // NOI18N
+    // -----
 
     static {
         ResourceBundle messages = ProfilerServer.getProfilerServerResourceBundle();
@@ -295,6 +317,7 @@ public class ProfilerInterface implements CommonConstants {
             INTERNAL_ERROR_MSG = messages.getString("ProfilerInterface_InternalErrorMsg"); // NOI18N
             UNEXPECTED_EXCEPTION_MSG = messages.getString("ProfilerInterface_UnexpectedExceptionMsg"); // NOI18N
             INSTRUMENTATION_SUCCESSFUL_MSG = messages.getString("ProfilerInterface_InstrumentationSuccessfulMsg"); // NOI18N
+            HISTOGRAM_NOT_AVAILABLE_MSG = messages.getString("ProfilerInterface_HistogramNotAvailableMsg");     // NOI18N
         }
     }
 
@@ -302,17 +325,20 @@ public class ProfilerInterface implements CommonConstants {
     private static final boolean DEBUG = Boolean.getBoolean("org.netbeans.lib.profiler.server.ProfilerInterface.classLoadHook"); // NOI18N
     private static final boolean INSTRUMENT_JFLUID_CLASSES = Boolean.getBoolean("org.netbeans.lib.profiler.server.instrumentJFluidClasses"); // NOI18N
 
+    private static final byte[] EMPTY = new byte[0];
+    
     // The lock used to serialize requests from server to client. May be used outside this class.
     public static final TransactionalSupport serialClientOperationsLock = new TransactionalSupport();
     private static ProfilerServer profilerServer;
     private static ProfilingSessionStatus status;
     private static EventBufferManager evBufManager;
-    private static Class[] loadedClassesArray; // Temporary array, used to send all loaded class names to client
+    private static WeakReference[] loadedClassesArray; // Temporary array, used to send all loaded class names to client
                                                // on instrumentation initiation.
     private static int[] loadedClassesLoaders; // Ditto, for loaders
     private static WeakHashMap reflectMethods; // Cache of methods called using reflection
     private static boolean targetAppSuspended = false;
     private static boolean instrumentReflection = false;
+    private static boolean instrSpawnedThreads;
     private static int[] packedArrayOffsets;
     private static int nSystemThreads;
     private static Thread initInstrumentationThread;
@@ -389,8 +415,9 @@ public class ProfilerInterface implements CommonConstants {
     }
 
     public static MethodNamesResponse getMethodNamesForJMethodIds(int[] methodIds) {
+        final int PACKEDARR_ITEMS = 4; // must match PACKEDARR_ITEMS in Stacks.c
         int nMethods = methodIds.length;
-        int len = nMethods * 3;
+        int len = nMethods * PACKEDARR_ITEMS;
         packedArrayOffsets = new int[len];
 
         byte[] packedData = Stacks.getMethodNamesForJMethodIds(nMethods, methodIds, packedArrayOffsets);
@@ -439,20 +466,15 @@ public class ProfilerInterface implements CommonConstants {
         return ProfilerRuntime.profiledTargetAppThreadsExist();
     }
 
-    /**
-     * Deactivate the injected code for the current instrumentation type, and clean up all the supporting data structures
-     * maintained by the corresponding ProfilerRuntimeXXX class.
-     */
-    public static void deactivateInjectedCode() {
+    static void disableProfiling() {
         int instrType = getCurrentInstrType();
 
-        if (instrType == INSTR_NONE) {
-            return;
-        }
-
-        disableProfilerHooks();
-
         switch (instrType) {
+            case INSTR_NONE:
+            case INSTR_NONE_MEMORY_SAMPLING:
+                ProfilerRuntime.clearDataStructures();
+                break;
+                
             case INSTR_NONE_SAMPLING:
                 ProfilerRuntimeSampler.shutdown();
                 break;
@@ -483,8 +505,16 @@ public class ProfilerInterface implements CommonConstants {
                 ProfilerRuntimeObjLiveness.enableProfiling(false);
 
                 break;
-        }
+        }        
+    }
 
+    /**
+     * Deactivate the injected code for the current instrumentation type, and clean up all the supporting data structures
+     * maintained by the corresponding ProfilerRuntimeXXX class.
+     */
+    public static void deactivateInjectedCode() {
+        disableProfilerHooks();
+        disableProfiling();
         status.resetInstrClassAndMethodInfo();
         setCurrentInstrType(INSTR_NONE);
     }
@@ -493,6 +523,7 @@ public class ProfilerInterface implements CommonConstants {
         Classes.setWaitTrackingEnabled(false);
         Classes.setParkTrackingEnabled(false);
         Classes.setSleepTrackingEnabled(false);
+        Classes.setVMObjectAllocEnabled(false);
         Classes.disableClassLoadHook();
         ProfilerRuntimeCPU.setJavaLangReflectMethodInvokeInterceptEnabled(false);
         ClassLoaderManager.setNotifyToolAboutUnloadedClasses(false);
@@ -503,9 +534,9 @@ public class ProfilerInterface implements CommonConstants {
             GC.runGC();
 
             try {
+                // Give WeakReference collector thread a chance to register some (hopefully most of) object GCs
                 Thread.sleep(500);
 
-                // Give WeakReference collector thread a chance to register some (hopefully most of) object GCs
             } catch (Exception ex) {
             }
         }
@@ -521,14 +552,16 @@ public class ProfilerInterface implements CommonConstants {
      * is the only thread that we can reliably characterize as the profiler's own.
      */
     public static void initProfilerInterface(ProfilingSessionStatus status, Thread specialThread) {
+        boolean jdk15 = Platform.getJDKVersionNumber() == Platform.JDK_15;
         Timers.initialize();
         Classes.initialize();
         GC.initialize();
         Stacks.initialize();
         Threads.initialize();
-        HeapDump.initialize(Platform.getJDKVersionNumber() == Platform.JDK_15);
+        HeapDump.initialize(jdk15);
+        ThreadDump.initialize(jdk15);
         ClassLoaderManager.initialize(profilerServer);
-        ClassLoaderManager.addLoader(ClassLoader.getSystemClassLoader());
+        ClassLoaderManager.addSystemClassLoader();
         reflectMethods = new WeakHashMap();
 
         evBufManager = new EventBufferManager(profilerServer);
@@ -562,8 +595,8 @@ public class ProfilerInterface implements CommonConstants {
                     reflectiveMethodInvokeHook(method);
                 }
 
-                public int handleFirstTimeVMObjectAlloc(String className, int classLoaderId) {
-                    return firstTimeVMObjectAlloc(className, classLoaderId);
+                public int handleFirstTimeVMObjectAlloc(String className, int definingClassLoaderId) {
+                    return firstTimeVMObjectAlloc(className, definingClassLoaderId);
                 }
 
                 public void handleEventBufferDump(byte[] eventBuffer, int startPos, int curPtrPos) {
@@ -689,10 +722,61 @@ public class ProfilerInterface implements CommonConstants {
         return detachStarted;
     }
         
-    static HeapHistogramResponse computeHistogram() {
-        return heapHistgramManager.computeHistogram(Histogram.getRawHistogram());
+    static Response computeHistogram() {
+        Response resp = null;
+        
+        if (Histogram.isAvailable()) {
+            resp = heapHistgramManager.computeHistogram(Histogram.getRawHistogram());
+        }
+        if (resp == null) {
+            resp = new Response(HISTOGRAM_NOT_AVAILABLE_MSG);
+            
+        }
+        return resp;
     }
 
+    static byte[][] getClassFileBytes(String[] classNames, int[] classLoaderIds) {
+        int MAX_CLASSES = 1000;
+        Class[] nonSystemClasses = new Class[MAX_CLASSES+1]; // classes loaded by classloaders other that bootstrap and system
+        int nonSystemIndex = 0;
+        byte[][] bytes = new byte[classNames.length][];
+        Class[] classes = new Class[classNames.length];
+
+        for (int i = 0; i < loadedClassesArray.length; i++) {
+            Class loadedClass = getOrdinaryClass(loadedClassesArray[i]);
+            if (loadedClass == null) {
+                // class was unloaded or has special name
+                continue;
+            }
+            int classLoaderId = loadedClassesLoaders[i];
+            String name = loadedClass.getName();
+            
+            // At the client side we treat classes loaded by the bootstrap and by the system classloaders in the same way
+            if (classLoaderId == -1) classLoaderId = 0;
+            for (int j = 0; j < classNames.length; j++) {
+                if (classLoaderIds[j] == classLoaderId && classNames[j].equals(name)) {
+                    classes[j] = loadedClass;
+                    nonSystemClasses[nonSystemIndex++] = loadedClass;
+                    break;
+                }
+            }
+            if (nonSystemIndex == MAX_CLASSES) {
+                cacheLoadedClasses(nonSystemClasses,nonSystemIndex);
+                nonSystemIndex = 0;
+                profilerServer.sendSimpleCmdToClient(Command.STILL_ALIVE);
+            }
+        }
+        if (nonSystemIndex > 0) {
+            cacheLoadedClasses(nonSystemClasses,nonSystemIndex);
+        }
+        for (int i = 0; i < classes.length; i++) {
+            if (classes[i] != null) {
+                bytes[i] = getClassFileBytes(classes[i], classLoaderIds[i]);
+            }
+        }
+        return bytes;
+    }
+    
     private static boolean getAndInstrumentClasses(boolean rootClassInstrumentation) {
         Response r = profilerServer.getLastResponse();
 
@@ -787,10 +871,10 @@ public class ProfilerInterface implements CommonConstants {
             int MAX_CLASSES = 1000;
             Class[] nonSystemClasses = new Class[MAX_CLASSES+1]; // classes loaded by classloaders other that bootstrap and system
             boolean dynamic = profilerServer.isDynamic();
-
-            loadedClassesArray = Classes.getAllLoadedClasses();
-            loadedClassesLoaders = new int[loadedClassesArray.length];
-
+            Class[] allClasses = Classes.getAllLoadedClasses();
+            loadedClassesArray = new WeakReference[allClasses.length];
+            loadedClassesLoaders = new int[allClasses.length];
+            
             Monitors.DeterminateProgress progress = Monitors.enterServerState(CommonConstants.SERVER_PREPARING, loadedClassesArray.length);
             try {
                 for (int i = 0; i < loadedClassesArray.length; i++) {
@@ -798,32 +882,9 @@ public class ProfilerInterface implements CommonConstants {
                     if(detachStarted) {
                         return;
                     }
-                    Class clazz = loadedClassesArray[i];
+                    Class clazz = allClasses[i];
+                    loadedClassesArray[i] = new WeakReference(clazz);
                     loadedClassesLoaders[i] = ClassLoaderManager.registerLoader(clazz);
-
-                    if (dynamic) {
-                        if (loadedClassesLoaders[i] > 0) { // bootstrap classloader has index -1 and system classloader has index 0
-                            nonSystemClasses[nonSystemIndex++] = clazz;
-                        } else {    // system or bootstrap class loader
-                            String name = clazz.getName();
-                            
-                            if (name.startsWith("[") || internalClassName(name)) {  // NOI18N
-                                continue;
-                            }
-                            if (ClassBytesLoader.getClassFileURL(name) == null) {
-                                // system class without file on disk -> must be cached
-                                nonSystemClasses[nonSystemIndex++] = clazz;
-                            }
-                        }
-                        if (nonSystemIndex == MAX_CLASSES) {
-                            cacheLoadedClasses(nonSystemClasses,nonSystemIndex);
-                            nonSystemIndex = 0;
-                        }
-                    }
-                }
-
-                if (nonSystemIndex > 0) {
-                    cacheLoadedClasses(nonSystemClasses,nonSystemIndex);
                 }
             } finally {
                 Monitors.exitServerState();
@@ -866,23 +927,23 @@ public class ProfilerInterface implements CommonConstants {
         }
 
         if (type == Integer.TYPE) {
-            sb.append('I'); // NOI18N
+            sb.append(INT);
         } else if (type == Boolean.TYPE) {
-            sb.append('Z'); // NOI18N
+            sb.append(BOOLEAN);
         } else if (type == Byte.TYPE) {
-            sb.append('B'); // NOI18N
+            sb.append(BYTE);
         } else if (type == Character.TYPE) {
-            sb.append('C'); // NOI18N
+            sb.append(CHAR);
         } else if (type == Long.TYPE) {
-            sb.append('J'); // NOI18N
+            sb.append(LONG);
         } else if (type == Float.TYPE) {
-            sb.append('F'); // NOI18N
+            sb.append(FLOAT);
         } else if (type == Double.TYPE) {
-            sb.append('D'); // NOI18N
+            sb.append(DOUBLE);
         } else if (type == Void.TYPE) {
-            sb.append('V'); // NOI18N
+            sb.append(VOID);
         } else {
-            sb.append('L'); // NOI18N
+            sb.append(REFERENCE);
             sb.append(type.getName().replace('.', '/')); // NOI18N
             sb.append(';'); // NOI18N
         }
@@ -890,7 +951,8 @@ public class ProfilerInterface implements CommonConstants {
 
     private static boolean checkForLoadedRootClasses() {
         for (int i = 0; i < loadedClassesArray.length; i++) {
-            if (isRootClass(loadedClassesArray[i].getName())) {
+            Class loadedClass = (Class) loadedClassesArray[i].get();
+            if (loadedClass != null && isRootClass(loadedClass.getName())) {
                 return true;
             }
         }
@@ -1100,7 +1162,7 @@ public class ProfilerInterface implements CommonConstants {
         }
     }
 
-    private static int firstTimeVMObjectAlloc(String className, int classLoaderId) {
+    private static int firstTimeVMObjectAlloc(String className, int definingClassLoaderId) {
         GetClassIdResponse resp;
 
         if (internalClassName(className)) {
@@ -1110,12 +1172,7 @@ public class ProfilerInterface implements CommonConstants {
         serialClientOperationsLock.beginTrans(true);
 
         try {
-            if (classLoaderId > 0) { // neither bootstrap nor system class loader
-                                     // get defining classloader 
-                classLoaderId = ClassLoaderManager.getDefiningLoaderForClass(className, classLoaderId);
-            }
-
-            GetClassIdCommand cmd = new GetClassIdCommand(className, classLoaderId);
+            GetClassIdCommand cmd = new GetClassIdCommand(className, definingClassLoaderId);
             profilerServer.sendComplexCmdToClient(cmd);
             resp = (GetClassIdResponse) profilerServer.getLastResponse();
         }  finally {
@@ -1402,6 +1459,10 @@ public class ProfilerInterface implements CommonConstants {
     * classLoaderId = 0 means that it is a system or bootstrap classloader
     */
     private static byte[] getClassFileBytes(Class clazz, int classLoaderId) {
+        return getClassFileBytes(clazz, classLoaderId, true);
+    }
+
+    private static byte[] getClassFileBytes(Class clazz, int classLoaderId, boolean force) {
         byte[] classFileBytes = null;
         URL classURL = null;
         
@@ -1411,6 +1472,7 @@ public class ProfilerInterface implements CommonConstants {
                 if (DEBUG) {
                     System.err.println("Cannot get classbytes for "+clazz.getName()+" loader "+classLoaderId);
                 }
+                if (!force) return EMPTY;
                 cacheLoadedClass(clazz);
                 classFileBytes = getCachedClassFileBytes(clazz);
                 if (classFileBytes == null) {
@@ -1420,7 +1482,7 @@ public class ProfilerInterface implements CommonConstants {
                     }
                 }
             }
-        } else if (status.remoteProfiling || Platform.getJDKVersionNumber() == Platform.JDK_19) {
+        } else if (status.remoteProfiling || Platform.getJDKVersionNumber() >= Platform.JDK_19) {
             classFileBytes = ClassBytesLoader.getClassFileBytes(classURL);
         }
         return classFileBytes;
@@ -1435,27 +1497,81 @@ public class ProfilerInterface implements CommonConstants {
         String[] loadedClassNames = new String[len];
         int[] loaders = new int[len];
         byte[][] cachedClassFileBytes = new byte[len][];
-        int idx = 0;
+        int[] loadedClassesSuper = new int[len];
+        int[][] loadedClassesInterfaces = new int[len][];
+        int instrType = getCurrentInstrType();
+        boolean isLazyInstrType = profilerServer.isDynamic()
+                && (instrType == INSTR_RECURSIVE_FULL || instrType == INSTR_RECURSIVE_SAMPLED)
+                && status.instrScheme == INSTRSCHEME_LAZY 
+                && !instrSpawnedThreads;
+        boolean isMemoryProfiling = profilerServer.isDynamic()
+                && (instrType == INSTR_OBJECT_ALLOCATIONS || instrType == INSTR_OBJECT_LIVENESS);
+        Map classIndex = new HashMap(loadedClassesArray.length*4/3);
+        Class[] classesArray = new Class[len];
 
-        for (int i = 0; i < loadedClassesArray.length; i++) {
-            String name = loadedClassesArray[i].getName();
-
-            if (name.startsWith("[") || internalClassName(name)) { // NOI18N
+        for (int i = 0; i < len; i++) {
+            Class clazz = getOrdinaryClass(loadedClassesArray[i]);
+            if (clazz == null) {
+                // class was unloaded or has special name
                 continue;
             }
-
-            loadedClassNames[idx] = name;
-            loaders[idx] = loadedClassesLoaders[i];
-            cachedClassFileBytes[idx] = getClassFileBytes(loadedClassesArray[i], loaders[idx]);
-            idx++;
+            int index = classIndex.size();
+            classesArray[index] = clazz;
+            loaders[index] = loadedClassesLoaders[i];
+            classIndex.put(clazz, new Integer(index));
         }
-        RootClassLoadedCommand cmd = new RootClassLoadedCommand(loadedClassNames, loaders, cachedClassFileBytes, idx,
-                                                                ClassLoaderManager.getParentLoaderIdTable());
+        for (int i = 0; i < classIndex.size(); i++) {
+            Class clazz = classesArray[i];
+            String name = clazz.getName();
+            boolean forceClassFile = (!isLazyInstrType && !isMemoryProfiling) || isRootClass(name);
+            loadedClassNames[i] = name;
+            cachedClassFileBytes[i] = getClassFileBytes(clazz, loaders[i], forceClassFile);
+
+            if (!forceClassFile) {
+                Class superClass = clazz.getSuperclass();
+                Class[] interfaces = clazz.getInterfaces();
+                
+                if (superClass != null) {
+                    Integer clsId = (Integer) classIndex.get(superClass);
+                    if (clsId != null) {
+                        loadedClassesSuper[i] = clsId.intValue();
+                    } else {
+                        //System.out.println("Super class of class "+name+" not found: "+superClass.getName());
+                        loadedClassesSuper[i] = -1;
+                    }
+                } else {
+                    loadedClassesSuper[i] = -1;
+                }
+                loadedClassesInterfaces[i] = new int[interfaces.length];
+                for (int j = 0; j < interfaces.length; j++) {
+                    Integer clsId = (Integer)classIndex.get(interfaces[j]);
+                    if (clsId != null) {
+                        loadedClassesInterfaces[i][j] = clsId.intValue();
+                    } else {
+                        //System.out.println("Interface of class "+name+" not found: "+interfaces[j].getName());
+                        loadedClassesSuper[i] = -1;
+                    }
+                }
+            }
+        }
+        RootClassLoadedCommand cmd = new RootClassLoadedCommand(loadedClassNames, loaders, cachedClassFileBytes, 
+                                                loadedClassesSuper, loadedClassesInterfaces, classIndex.size(),
+                                                ClassLoaderManager.getParentLoaderIdTable());
         profilerServer.sendComplexCmdToClient(cmd);
-        loadedClassesArray = null; // Free memory
-        loadedClassesLoaders = null; // Ditto
     }
 
+    private static Class getOrdinaryClass(WeakReference classRef) {
+        Class clazz = (Class) classRef.get();
+        if (clazz != null) {
+            String name = clazz.getName();
+
+            if (!name.startsWith("[") && !internalClassName(name)) {
+                return clazz;
+            }             
+        }
+        return null;
+    }
+    
     private static void updateInstrClassAndMethodNames(InstrumentMethodGroupData imgb, boolean firstTime) {
         status.beginTrans(false);
 
