@@ -43,9 +43,11 @@
 
 package org.netbeans.lib.profiler.server;
 
-import org.netbeans.lib.profiler.global.Platform;
-import org.netbeans.lib.profiler.server.system.Timers;
 import java.lang.reflect.Method;
+import java.util.List;
+import org.netbeans.lib.profiler.global.Platform;
+import org.netbeans.lib.profiler.server.system.Stacks;
+import org.netbeans.lib.profiler.server.system.Timers;
 
 /**
  * This class contains the functionality that is common for all CPU profiling methods available in JFluid.
@@ -57,8 +59,11 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
     //~ Static fields/initializers -----------------------------------------------------------------------------------------------
 
     private static final boolean DEBUG = false;
+    private static final int MAX_STRING_LENGTH = 2048;
+    static final Object NO_RET_VALUE = new Object();
     private static int nProfiledThreadsLimit;
     protected static int nProfiledThreadsAllowed;
+    protected static int stackDepthLimit = Integer.MAX_VALUE;
     protected static boolean enableFirstTimeMethodInvoke;
 
     // The following flag is used to prevent deadlock inside getThreadInfo() by forcing immediate return from methodEntry() etc.
@@ -106,6 +111,10 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
         nProfiledThreadsLimit = nProfiledThreadsAllowed = num;
     }
 
+    public static void setStackDepthLimit(int num) {
+        stackDepthLimit = num;
+    }
+
     public static void setWaitAndSleepTracking(boolean waitTracking, boolean sleepTracking) {
         waitTrackingEnabled = waitTracking;
         sleepTrackingEnabled = sleepTracking;
@@ -218,6 +227,12 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
         ProfilerRuntime.clearDataStructures();
         nProfiledThreadsAllowed = nProfiledThreadsLimit;
     }
+    
+    protected static void createNewDataStructures() {
+        ProfilerRuntime.createNewDataStructures();
+        // top level Marker method has stacktrace 
+        Stacks.createNativeStackFrameBuffer(ProfilerRuntimeMemory.MAX_STACK_FRAMES);
+    }
 
     protected static long currentTimeInCounts() {
         return Timers.getCurrentTimeInCounts();
@@ -379,9 +394,21 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
 
     // ---------------------------------- Writing profiler events --------------------------------------
     static void writeTimeStampedEvent(byte eventType, ThreadInfo ti, char methodId) {
+        int[] stackFrameIds = null;
+        int currentStackDepth = 0;
+        if (eventType == MARKER_ENTRY) {
+            // top-level marker method has stack trace
+            if (ti.stackDepth == 0) {
+                stackFrameIds = new int[ProfilerRuntimeMemory.MAX_STACK_FRAMES];
+                currentStackDepth = Stacks.getCurrentStackFrameIds(ti.getThread(), stackFrameIds.length, stackFrameIds);
+                currentStackDepth -= ProfilerRuntimeMemory.NO_OF_PROFILER_FRAMES;
+            } else {
+                stackFrameIds = new int[0];
+            }
+        }
         int curPos = ti.evBufPos; // It's important to use a local copy for evBufPos, so that evBufPos is at event boundary at any moment
 
-        if (curPos > ThreadInfo.evBufPosThreshold) {
+        if (curPos + currentStackDepth*4 > ThreadInfo.evBufPosThreshold) {
             copyLocalBuffer(ti);
             curPos = ti.evBufPos;
         }
@@ -426,6 +453,20 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
                                    + (int) methodId + ", timestamp: " + threadTimeStamp); // NOI18N
             }
         }
+        if (stackFrameIds != null) {
+            evBuf[curPos++] = (byte) ((currentStackDepth >> 16) & 0xFF);
+            evBuf[curPos++] = (byte) ((currentStackDepth >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((currentStackDepth) & 0xFF);
+            int frameIdx = ProfilerRuntimeMemory.NO_OF_PROFILER_FRAMES;
+
+            for (int i = 0; i < currentStackDepth; i++) {
+                evBuf[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 24) & 0xFF);
+                evBuf[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 16) & 0xFF);
+                evBuf[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 8) & 0xFF);
+                evBuf[curPos++] = (byte) ((stackFrameIds[frameIdx]) & 0xFF);
+                frameIdx++;
+            }
+        }
 
         ti.evBufPos = curPos;
     }
@@ -434,6 +475,45 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
         return writeWaitTimeEvent(eventType, ti, null);
     }
     
+    static void writeParametersEvent(ThreadInfo ti) {
+        List pars = ti.getParameters();
+        int parsLength = 0;
+        if (pars != null) {
+            for (int i = 0; i < pars.size(); i++) {
+                parsLength += 1 + getParSize(pars.get(i));
+            }
+        }
+        int fullInfoLen = 1 + 1 + parsLength;
+        int curPos = ti.evBufPos; // It's important to use a local copy for evBufPos, so that evBufPos is at event boundary at any moment
+
+        if ((curPos + fullInfoLen) > ThreadInfo.evBufPosThreshold) {
+            copyLocalBuffer(ti);
+            curPos = ti.evBufPos;
+        }
+
+        byte[] evBuf = ti.evBuf;
+
+        evBuf[curPos++] = MARKER_ENTRY_PARAMETERS;
+        if (pars != null) {
+            evBuf[curPos++] = (byte) pars.size();
+
+            for (int i = 0; i < pars.size(); i++) {
+                curPos = writeParameter(evBuf, curPos, pars.get(i));
+            }
+            ti.clearParameters();
+        } else {
+            evBuf[curPos++] = 0;            
+        }
+        ti.evBufPos = curPos;
+    }
+
+    static void writeRetValue(Object ret, ThreadInfo ti) {
+        if (ret != NO_RET_VALUE) {
+            ti.addParameter(ret != null ? converToString(ret) : "");
+            writeParametersEvent(ti);
+        }
+    }
+    
     private static void servletDoMethodHook(ThreadInfo ti, Object request) {
         String servletPath = null;
         String method = null;
@@ -464,4 +544,265 @@ public class ProfilerRuntimeCPU extends ProfilerRuntime {
 
         writeServletDoMethod(ti, method, servletPath, requestedSessionId);
     }
+    
+    public static void addParameter(boolean b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(Boolean.valueOf(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(char b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Character(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(byte b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Byte(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(short b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Short(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(int b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Integer(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(long b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Long(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(float b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Float(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(double b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(new Double(b));
+        ti.inProfilingRuntimeMethod--; 
+    }
+
+    public static void addParameter(Object b) {
+        if (recursiveInstrumentationDisabled) {
+            return;
+        }
+
+        ThreadInfo ti = ThreadInfo.getThreadInfo();
+
+        if (ti.inProfilingRuntimeMethod > 0) {
+            return;
+        }
+
+        ti.inProfilingRuntimeMethod++;
+        ti.addParameter(b != null ? converToString(b) : "");
+        ti.inProfilingRuntimeMethod--; 
+    }
+    
+    private static int getParSize(Object p) {
+        Class type = p.getClass();
+        if (type == Integer.class) {
+            return 4;
+        } else if (type == Boolean.class) {
+            return 1;
+        } else if (type == Byte.class) {
+            return 1;
+        } else if (type == Character.class) {
+            return 2;
+        } else if (type == Long.class) {
+            return 8;
+        } else if (type == Float.class) {
+            return 4;
+        } else if (type == Double.class) {
+            return 8;
+        } else {
+            return 2 + truncatedByteLength((String) p);
+        }
+    }
+    
+    private static int writeParameter(byte[] evBuf, int curPos, Object p) {
+        Class type = p.getClass();
+        if (type == Integer.class) {
+            int vp = ((Integer)p).intValue();
+            evBuf[curPos++] = ProfilerInterface.INT;
+            evBuf[curPos++] = (byte) ((vp >> 24) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 16) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp) & 0xFF);
+        } else if (type == Boolean.class) {
+            boolean vp = ((Boolean)p).booleanValue();
+            evBuf[curPos++] = ProfilerInterface.BOOLEAN;
+            evBuf[curPos++] = (byte) (vp ? 1 : 0);
+        } else if (type == Byte.class) {
+            byte vp = ((Byte)p).byteValue();
+            evBuf[curPos++] = ProfilerInterface.BYTE;
+            evBuf[curPos++] = vp;
+        } else if (type == Character.class) {
+            char vp = ((Character)p).charValue();
+            evBuf[curPos++] = ProfilerInterface.CHAR;
+            evBuf[curPos++] = (byte) ((vp >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp) & 0xFF);
+        } else if (type == Long.class) {
+            long vp = ((Long)p).longValue();
+            evBuf[curPos++] = ProfilerInterface.LONG;
+            evBuf[curPos++] = (byte) ((vp >> 56) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 48) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 40) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 32) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 24) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 16) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp) & 0xFF); 
+        } else if (type == Float.class) {
+            int vp = Float.floatToIntBits(((Float)p).floatValue());
+            evBuf[curPos++] = ProfilerInterface.FLOAT;
+            evBuf[curPos++] = (byte) ((vp >> 24) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 16) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp) & 0xFF);
+        } else if (type == Double.class) {
+            long vp = Double.doubleToLongBits(((Double)p).doubleValue());
+            evBuf[curPos++] = ProfilerInterface.DOUBLE;
+            evBuf[curPos++] = (byte) ((vp >> 56) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 48) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 40) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 32) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 24) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 16) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((vp) & 0xFF);
+        } else {    
+            String sp = (String) p;
+            int lengthBytes = truncatedByteLength(sp);
+            evBuf[curPos++] = ProfilerInterface.REFERENCE;
+            evBuf[curPos++] = (byte) ((lengthBytes >> 8) & 0xFF);
+            evBuf[curPos++] = (byte) ((lengthBytes) & 0xFF);
+            for (int i = 0; i < lengthBytes/2; i++) {
+                char ch = sp.charAt(i);
+                evBuf[curPos++] = (byte) ((ch >> 8) & 0xFF);
+                evBuf[curPos++] = (byte) ((ch) & 0xFF);
+            }
+        }
+        return curPos;
+    }
+    
+    static String converToString(Object o) {
+        String clazz = o.getClass().getName();
+        
+        if (clazz.startsWith("java.lang.")) {
+            return o.toString();
+        }
+        if (clazz.equals("java.sql.Date")) {
+            return String.valueOf(((java.sql.Date)o).getTime());
+        }
+        if (clazz.equals("java.sql.Timestamp")) {
+            return String.valueOf(((java.sql.Timestamp)o).getTime());            
+        }
+        if (clazz.equals("java.math.BigDecimal")) {
+            return o.toString();
+        }
+        return getObjectId(o, clazz);
+    }
+    
+    private static int truncatedByteLength(String s) {
+        int length = s.length()*2;
+        
+        if (length < MAX_STRING_LENGTH) {
+            return length;
+        }
+        return MAX_STRING_LENGTH;
+    }
+    
+    private static String getObjectId(Object o, String clazz) {
+        return clazz + "@" + Integer.toHexString(System.identityHashCode(o));
+    }
 }
