@@ -66,10 +66,13 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     protected static short[] allocatedInstThreshold;
     protected static char[] objectSize;
     protected static short samplingInterval;
-    protected static int samplingDepth;
-    private static int stackDepth;
+    protected static int stackSamplingDepth;
+    static final Object classIdMapLock = new Object();
+    private static int currentStackDepth;
     private static int[] stackFrameIds;
     private static Map classIdMap;
+    private static volatile boolean resultsAvailable;
+    private static final boolean DEBUG = false;
 
     // -------------------------------------- Miscellaneous support routines ------------------------------------------
     private static long randSeed;
@@ -108,7 +111,7 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     }
 
     /** Negative parameter means the depth is not limited. val == 0 means we should not take stack samples */
-    public static void setSamplingDepth(int val) {
+    public static void setStackSamplingDepth(int val) {
         if (val < 0) {
             val = MAX_STACK_FRAMES;
         } else if (val > 0) {
@@ -118,10 +121,10 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
                 val = MAX_STACK_FRAMES;
             }
         } else {
-            stackDepth = 0;
+            currentStackDepth = 0;
         }
 
-        samplingDepth = val;
+        stackSamplingDepth = val;
     }
 
     public static void setSamplingInterval(short val) {
@@ -143,6 +146,7 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
         }
 
         classIdMap = new HashMap();
+        resultsAvailable = false;
     }
 
     public static void traceVMObjectAlloc(Object instance, Class clazz) {
@@ -152,11 +156,7 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
         }
 
         ThreadInfo ti = ThreadInfo.getThreadInfo();
-        String className;
-        String classNameId;
-        int classLoaderId;
-        Integer classIdInt;
-        char classId;
+        int classId;
         boolean isObjectLiveness;
 
         if (ti.inProfilingRuntimeMethod > 0) {
@@ -164,52 +164,71 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
         }
 
         if (!ti.isInitialized()) {
-            ti.initialize(true);
+            ti.initialize();
+            if (lockContentionMonitoringEnabled) writeThreadCreationEvent(ti);
         }
 
         ti.inProfilingRuntimeMethod++;
-        className = clazz.getName();
-        classLoaderId = ClassLoaderManager.registerLoader(clazz);
-        classNameId = new StringBuffer(className).append('#').append(classLoaderId).toString();
-        classIdInt = (Integer) classIdMap.get(classNameId);
+        classId = getClassId(clazz);
+        if (classId == -1) {
+            // System.err.println("*** JFluid warning: Invalid classId for class:"+classNameId);
+            ti.inProfilingRuntimeMethod--;
 
-        if (classIdInt == null) {
-            int newClassId = externalActionsHandler.handleFirstTimeVMObjectAlloc(className, classLoaderId);
-
-            if (newClassId != -1) {
-                classIdInt = new Integer(newClassId);
-                classIdMap.put(classNameId, classIdInt);
-            } else {
-                // System.err.println("*** JFluid warning: Invalid classId for class:"+classNameId);
-                ti.inProfilingRuntimeMethod--;
-
-                return;
-            }
+            return;
         }
 
         isObjectLiveness = ProfilerInterface.getCurrentInstrType() == INSTR_OBJECT_LIVENESS;
-        classId = (char) classIdInt.intValue();
         ti.inProfilingRuntimeMethod--;
 
         if (isObjectLiveness) {
-            ProfilerRuntimeObjLiveness.traceObjAlloc(instance, classId);
+            ProfilerRuntimeObjLiveness.traceObjAlloc(instance, (char)classId);
         } else {
-            ProfilerRuntimeObjAlloc.traceObjAlloc(instance, classId);
+            ProfilerRuntimeObjAlloc.traceObjAlloc(instance, (char)classId);
         }
     }
 
+    static int getClassId(Class clazz) {
+        String className = clazz.getName();
+        int definingClassLoaderId = ClassLoaderManager.registerLoader(clazz);
+        String classNameId = new StringBuffer(className).append('#').append(definingClassLoaderId).toString();
+        Integer classIdInt;
+        
+        synchronized (classIdMapLock) {
+            classIdInt = (Integer) classIdMap.get(classNameId);
+        }
+
+        if (classIdInt == null) {
+            int newClassId = externalActionsHandler.handleFirstTimeVMObjectAlloc(className, definingClassLoaderId);
+
+            classIdInt = new Integer(newClassId);
+            synchronized (classIdMapLock) {
+                classIdMap.put(classNameId, classIdInt);
+            }
+            if (newClassId == -1) {
+                if (DEBUG) System.out.println("ProfilerRuntimeMemory.DEBUG: Invalid classId for class:"+classNameId);
+//                if (classNameId.startsWith("org.netbeans.lib.profiler")) Thread.dumpStack();
+//                if (classNameId.startsWith("[")) Thread.dumpStack();
+            }
+        }
+        return classIdInt.intValue();
+    }
+    
+    static boolean isInternalClass(Class clazz) {
+        return clazz == ThreadInfo.class || clazz == ThreadInfo[].class;
+    }
+    
     // ------------------------------------------ Stack trace obtaining -----------------------------------------------
 
     /** This is used in Object Allocation profiling mode */
-    protected static synchronized void getAndSendCurrentStackTrace(char classId, long objSize) {
+    protected static synchronized void getAndSendCurrentStackTrace(int classId, long objSize) {
         if (eventBuffer == null) {
             return; // Chances are that instrumentation has been removed while we were executing instrumentation code
         }
 
         synchronized (eventBuffer) { // Note that we have to use synchronization here due to the static stackFrameIds[] array
 
-            if (samplingDepth != 0) {
-                stackDepth = Stacks.getCurrentStackFrameIds(Thread.currentThread(), samplingDepth, stackFrameIds);
+            if (stackSamplingDepth != 0) {
+                currentStackDepth = Stacks.getCurrentStackFrameIds(Thread.currentThread(), stackSamplingDepth, stackFrameIds);
             }
 
             writeObjAllocStackTraceEvent(classId, objSize);
@@ -217,15 +236,15 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     }
 
     /** This is used in Object Liveness profiling mode */
-    protected static synchronized void getAndSendCurrentStackTrace(char classId, char epoch, int objCount, long objSize) {
+    protected static synchronized void getAndSendCurrentStackTrace(int classId, char epoch, int objCount, long objSize) {
         if (eventBuffer == null) {
             return; // Chances are that instrumentation has been removed while we were executing instrumentation code
         }
 
         synchronized (eventBuffer) { // Note that we have to use synchronization here due to the static stackFrameIds[] array
 
-            if (samplingDepth != 0) {
-                stackDepth = Stacks.getCurrentStackFrameIds(Thread.currentThread(), samplingDepth, stackFrameIds);
+            if (stackSamplingDepth != 0) {
+                currentStackDepth = Stacks.getCurrentStackFrameIds(Thread.currentThread(), stackSamplingDepth, stackFrameIds);
             }
 
             writeObjLivenessStackTraceEvent(classId, epoch, objCount, objSize);
@@ -272,6 +291,7 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
         stackFrameIds = new int[MAX_STACK_FRAMES];
         Stacks.createNativeStackFrameBuffer(MAX_STACK_FRAMES);
         classIdMap = new HashMap();
+        resultsAvailable = false;
     }
 
     protected static void enableProfiling(boolean v) {
@@ -324,22 +344,23 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     // ---------------------------------------- Writing profiler events -----------------------------------------
 
     /** Note that there is no synchronized(eventBuffer) in this method, since synchronization is already required by its callers */
-    protected static void writeObjAllocStackTraceEvent(char classId, long objSize) {
+    protected static void writeObjAllocStackTraceEvent(int classId, long objSize) {
         if (eventBuffer == null) {
             return; // Instrumentation removal happened when we were in instrumentation 
         }
 
-        if (stackDepth != 0) {
-            stackDepth -= NO_OF_PROFILER_FRAMES; // Top frames are our own methods
+        if (currentStackDepth != 0) {
+            currentStackDepth -= NO_OF_PROFILER_FRAMES; // Top frames are our own methods
         }
 
-        if (globalEvBufPos == 0) {
+        if (!resultsAvailable) {
+            resultsAvailable = true;
             ProfilerServer.notifyClientOnResultsAvailability();
         }
 
         int curPos = globalEvBufPos;
 
-        if ((curPos + 16 + (stackDepth * 4)) > globalEvBufPosThreshold) { // Dump the buffer
+        if ((curPos + 16 + (currentStackDepth * 4)) > globalEvBufPosThreshold) { // Dump the buffer
             externalActionsHandler.handleEventBufferDump(eventBuffer, 0, curPos);
             curPos = 0;
         }
@@ -385,22 +406,23 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     }
 
     /** Note that there is no synchronized(eventBuffer) in this method, since synchronization is already required by its callers */
-    protected static void writeObjLivenessStackTraceEvent(char classId, char epoch, int objCount, long objSize) {
+    protected static void writeObjLivenessStackTraceEvent(int classId, char epoch, int objCount, long objSize) {
         if (eventBuffer == null) {
             return; // Instrumentation removal happened when we were in instrumentation 
         }
 
-        if (stackDepth != 0) {
-            stackDepth -= NO_OF_PROFILER_FRAMES; // Top 4 frames are our own methods
+        if (currentStackDepth != 0) {
+            currentStackDepth -= NO_OF_PROFILER_FRAMES; // Top 4 frames are our own methods
         }
 
-        if (globalEvBufPos == 0) {
+        if (!resultsAvailable) {
+            resultsAvailable = true;
             ProfilerServer.notifyClientOnResultsAvailability();
         }
 
         int curPos = globalEvBufPos;
 
-        if ((curPos + 24 + (stackDepth * 4)) > globalEvBufPosThreshold) { // Dump the buffer
+        if ((curPos + 24 + (currentStackDepth * 4)) > globalEvBufPosThreshold) { // Dump the buffer
             externalActionsHandler.handleEventBufferDump(eventBuffer, 0, curPos);
             curPos = 0;
         }
@@ -426,9 +448,9 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
     }
 
     private static int writeStack(int curPos) {
-        eventBuffer[curPos++] = (byte) ((stackDepth >> 16) & 0xFF);
-        eventBuffer[curPos++] = (byte) ((stackDepth >> 8) & 0xFF);
-        eventBuffer[curPos++] = (byte) ((stackDepth) & 0xFF);
+        eventBuffer[curPos++] = (byte) ((currentStackDepth >> 16) & 0xFF);
+        eventBuffer[curPos++] = (byte) ((currentStackDepth >> 8) & 0xFF);
+        eventBuffer[curPos++] = (byte) ((currentStackDepth) & 0xFF);
 
         /// A variant when we send non-reversed call graph
         //int base = depth + NO_OF_PROFILER_FRAMES - 1;
@@ -438,7 +460,7 @@ public class ProfilerRuntimeMemory extends ProfilerRuntime {
         //}
         int frameIdx = NO_OF_PROFILER_FRAMES;
 
-        for (int i = 0; i < stackDepth; i++) {
+        for (int i = 0; i < currentStackDepth; i++) {
             eventBuffer[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 24) & 0xFF);
             eventBuffer[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 16) & 0xFF);
             eventBuffer[curPos++] = (byte) ((stackFrameIds[frameIdx] >> 8) & 0xFF);
