@@ -301,27 +301,7 @@ public class MiscUtils implements CommonConstants {
 
         outString = outString.substring(pos + 1);
 
-        if (outString.startsWith("1.5")) { // NOI18N
-
-            return JDK_15_STRING;
-        } else if (outString.startsWith("1.6")) { // NOI18N
-
-            return JDK_16_STRING;
-        } else if (outString.startsWith("1.7")) { // NOI18N
-
-            return JDK_17_STRING;
-        } else if (outString.startsWith("1.8")) { // NOI18N
-
-            return JDK_18_STRING;
-        } else if (outString.startsWith("1.9")) { // NOI18N
-
-            return JDK_19_STRING;
-        } else if (outString.startsWith("9")) { // NOI18N
-
-            return JDK_19_STRING;
-        } else {
-            throw new IOException(VM_INCOMPATIBLE_MSG + printOutString);
-        }
+        return Platform.getJDKVersionString(outString);
     }
 
     /** For a string representing a class path, remove all entries that don't correspond to existing files, and return the remaining ones. */
@@ -376,10 +356,10 @@ public class MiscUtils implements CommonConstants {
                     try {
                         getClassPathFromManifest(name,list);
                     } catch (URISyntaxException ex) {
-                        System.out.println("Error processing "+name);
+                        System.out.println("Error processing "+name);   // NOI18N
                         ex.printStackTrace();
                     } catch (IOException ex) {
-                        System.out.println("Error processing "+name);
+                        System.out.println("Error processing "+name);   // NOI18N
                         ex.printStackTrace();
                     }
                 }
@@ -389,7 +369,7 @@ public class MiscUtils implements CommonConstants {
     }
 
     private static void getClassPathFromManifest(String jarPath,List pathList) throws IOException, URISyntaxException {
-        if (jarPath.toLowerCase().endsWith(".jar")) {
+        if (jarPath.toLowerCase().endsWith(".jar")) {   // NOI18N
             File pathFile = new File(jarPath);
             JarFile jarFile = new JarFile(pathFile);
             Manifest manifest = jarFile.getManifest();
@@ -453,17 +433,17 @@ public class MiscUtils implements CommonConstants {
             return false;
         }
 
-        if (isSupported15or16or17or18or19orCvm(jdkVersionString)) {
+        if (isSupportedJDK(jdkVersionString)) {
             return true;
         }
         // CVM is recognized via java.vm.name system property
-        return isSupported15or16or17or18or19orCvm(vmNameString);
+        return isSupportedJDK(vmNameString);
     }
 
     // This method is used for checking running JVM if supported.
     // jvmVersionString should be enough to decide that
     public static boolean isSupportedRunningJVMVersion(String jdkVersionString) {
-        return isSupported15or16or17or18or19orCvm(jdkVersionString);
+        return isSupportedJDK(jdkVersionString);
     }
 
     public static void setVerbosePrint() {
@@ -539,7 +519,7 @@ public class MiscUtils implements CommonConstants {
                     files.close();
                 }
             } catch (IOException ex) {
-                System.err.println("deleteHeapTempFiles failed");
+                System.err.println("deleteHeapTempFiles failed");   // NOI18N
                 ex.printStackTrace();
             }
         }
@@ -642,11 +622,15 @@ public class MiscUtils implements CommonConstants {
         return (new Date()).toString();
     }
 
-    private static boolean isSupported15or16or17or18or19orCvm(String jdkVersionString) {
-        if (jdkVersionString.startsWith("1.9")) { // NOI18N
+    private static boolean isSupportedJDK(String jdkVersionString) {
+        if (jdkVersionString.startsWith("11")) { // NOI18N
+            return true;
+        } else if (jdkVersionString.startsWith("10")) { // NOI18N
             return true;
         } else if (jdkVersionString.startsWith("9")) { // NOI18N
             return true;
+        } else if (jdkVersionString.startsWith("1.9")) { // NOI18N
+            return true;
         } else if (jdkVersionString.startsWith("1.8")) { // NOI18N
             return true;
         } else if (jdkVersionString.startsWith("1.7")) { // NOI18N
@@ -654,13 +638,13 @@ public class MiscUtils implements CommonConstants {
         } else if (jdkVersionString.startsWith("1.6")) { // NOI18N
             return true;
         } else if (jdkVersionString.startsWith("1.5")) { // NOI18N
-            if (jdkVersionString.equals("1.5.0") || jdkVersionString.startsWith("1.5.0_01") ||
+            if (jdkVersionString.equals("1.5.0") || jdkVersionString.startsWith("1.5.0_01") ||  // NOI18N
                 jdkVersionString.startsWith("1.5.0_02") || jdkVersionString.startsWith("1.5.0_03")) { // NOI18N
                 return false;
             } else {
                 return true;
             }
-        } else if (jdkVersionString.equals("CVM")) {
+        } else if (jdkVersionString.equals("CVM")) {    // NOI18N
             return true;
         }
         return false;
