@@ -1,45 +1,20 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
- * Other names may be trademarks of their respective owners.
- *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common
- * Development and Distribution License("CDDL") (collectively, the
- * "License"). You may not use this file except in compliance with the
- * License. You can obtain a copy of the License at
- * http://www.netbeans.org/cddl-gplv2.html
- * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
- * specific language governing permissions and limitations under the
- * License.  When distributing the software, include this License Header
- * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
- * particular file as subject to the "Classpath" exception as provided
- * by Oracle in the GPL Version 2 section of the License file that
- * accompanied this code. If applicable, add the following below the
- * License Header, with the fields enclosed by brackets [] replaced by
- * your own identifying information:
- * "Portions Copyrighted [year] [name of copyright owner]"
- *
- * Contributor(s):
- *
- * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
- * Microsystems, Inc. All Rights Reserved.
- *
- * If you wish your version of this file to be governed by only the CDDL
- * or only the GPL Version 2, indicate your decision by adding
- * "[Contributor] elects to include this software in this distribution
- * under the [CDDL or GPL Version 2] license." If you do not indicate a
- * single choice of license, a recipient has the option to distribute
- * your version of this file under either the CDDL, the GPL Version 2 or
- * to extend the choice of license to its licensees as provided above.
- * However, if you add GPL Version 2 code and therefore, elected the GPL
- * Version 2 license, then the option applies only if the new code is
- * made subject to such option by the copyright holder.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.netbeans.nbbuild;
 
@@ -59,7 +34,9 @@ import java.net.URL;
 import java.text.DateFormat;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.regex.Matcher;
@@ -189,12 +166,9 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
             builder.setEntityResolver(this);
 
             if (generateTemplate) {
-                InputStream resource = Arch.class.getResourceAsStream("Arch-api-questions.xml");
-                try {
+                try (InputStream resource = Arch.class.getResourceAsStream("Arch-api-questions.xml")) {
                     q = builder.parse(resource);
                     qSource = new DOMSource (q);
-                } finally {
-                    resource.close();
                 }
             } else {
                 q = builder.parse (questionsFile);
@@ -233,7 +207,7 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
         if (generateTemplate) {
             log ("Input file " + questionsFile + " does not exist. Generating it with skeleton answers.");
             try {
-                SortedSet<String> s = new TreeSet<String>(questions.keySet());
+                SortedSet<String> s = new TreeSet<>(questions.keySet());
                 generateTemplateFile(questionsVersion, s);
             } catch (IOException ex) {
                 throw new BuildException (ex);
@@ -273,7 +247,7 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
         
         {
             // check all answers have their questions
-            SortedSet<String> s = new TreeSet<String>(questions.keySet());
+            SortedSet<String> s = new TreeSet<>(questions.keySet());
             s.removeAll (answers.keySet ());
             if (!s.isEmpty()) {
                 if ("true".equals (this.getProject().getProperty ("arch.generate"))) {
@@ -287,9 +261,7 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
                     qSource = new StreamSource (questionsFile);
                     try {
                         q = builder.parse(questionsFile);
-                    } catch (IOException ex) {
-                        throw new BuildException(ex);
-                    } catch (SAXException ex) {
+                    } catch (IOException | SAXException ex) {
                         throw new BuildException(ex);
                     }
                 } else {
@@ -431,9 +403,7 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
                         TransformerFactory fack = TransformerFactory.newInstance();
                         Transformer t = fack.newTransformer(defXSL);
                         t.transform(prjSrc, res);
-                    } catch (IOException ex) {
-                        throw new BuildException (ex);
-                    } catch (TransformerException ex) {
+                    } catch (IOException | TransformerException ex) {
                         throw new BuildException (ex);
                     }
                     
@@ -496,44 +466,40 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
             trans = TransformerFactory.newInstance();
             trans.setURIResolver(this);
             Transformer t = trans.newTransformer(ss);
-            OutputStream os = new BufferedOutputStream (new FileOutputStream (output));
-            StreamResult r = new StreamResult (os);
-            if (stylesheet == null) {
-                stylesheet = this.getProject ().getProperty ("arch.stylesheet");
+            try (OutputStream os = new BufferedOutputStream (new FileOutputStream (output))) {
+                StreamResult r = new StreamResult (os);
+                if (stylesheet == null) {
+                    stylesheet = this.getProject ().getProperty ("arch.stylesheet");
+                }
+                if (stylesheet != null) {
+                    t.setParameter("arch.stylesheet", stylesheet);
+                }
+                if (overviewlink != null) {
+                    t.setParameter("arch.overviewlink", overviewlink);
+                }
+                if (footer != null) {
+                    t.setParameter("arch.footer", footer);
+                }
+                t.setParameter("arch.answers.date", DateFormat.getDateInstance().format(new Date(questionsFile.lastModified())));
+                
+                String archTarget = output.toString();
+                int slash = archTarget.lastIndexOf(File.separatorChar);
+                if (slash > 0) {
+                    archTarget = archTarget.substring (slash + 1);
+                }
+                String archPref = getProject ().getProperty ("arch.target");
+                if (archPref != null) {
+                    archTarget = archPref + "/" + archTarget;
+                }
+                
+                t.setParameter("arch.target", archTarget);
+                String when = getProject().getProperty("arch.when");
+                if (when != null) {
+                    t.setParameter("arch.when", when);
+                }
+                t.transform(qSource, r);
             }
-            if (stylesheet != null) {
-                t.setParameter("arch.stylesheet", stylesheet);
-            }
-            if (overviewlink != null) {
-                t.setParameter("arch.overviewlink", overviewlink);
-            }
-            if (footer != null) {
-                t.setParameter("arch.footer", footer);
-            }
-            t.setParameter("arch.answers.date", DateFormat.getDateInstance().format(new Date(questionsFile.lastModified())));
-            
-            String archTarget = output.toString();
-            int slash = archTarget.lastIndexOf(File.separatorChar);
-            if (slash > 0) {
-                archTarget = archTarget.substring (slash + 1);
-            }
-            String archPref = getProject ().getProperty ("arch.target");
-            if (archPref != null) {
-                archTarget = archPref + "/" + archTarget;
-            }
-            
-            t.setParameter("arch.target", archTarget);
-            String when = getProject().getProperty("arch.when");
-            if (when != null) {
-                t.setParameter("arch.when", when);
-            }
-            t.transform(qSource, r);
-            os.close ();
-        } catch (IOException ex) {
-            throw new BuildException (ex);
-        } catch (TransformerConfigurationException ex) {
-            throw new BuildException (ex);
-        } catch (TransformerException ex) {
+        } catch (IOException | TransformerException ex) {
             throw new BuildException (ex);
         }
     }
@@ -562,11 +528,11 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
             sb.insert(m.start(1), version);
         }
         
-        Writer w = new OutputStreamWriter (new FileOutputStream (questionsFile.toString ()));
-        w.write(sb.toString());
-        writeQuestions (w, missing);
-        w.write("</api-answers>\n");
-        w.close();
+        try (Writer w = new OutputStreamWriter (new FileOutputStream (questionsFile.toString ()))) {
+            w.write(sb.toString());
+            writeQuestions (w, missing);
+            w.write("</api-answers>\n");
+        }
     }
 
     private void writeQuestions(Writer w, SortedSet<String> missing) throws IOException {
@@ -617,28 +583,26 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
             nbRoot = "http://hg.netbeans.org/main/raw-file/tip/";
         }
         
-        Writer w = new FileWriter (questionsFile);
-        
-        w.write ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-        w.write ("<!DOCTYPE api-answers PUBLIC \"-//NetBeans//DTD Arch Answers//EN\" \""); w.write (nbRoot); w.write ("nbbuild/antsrc/org/netbeans/nbbuild/Arch.dtd\" [\n");
-        w.write ("  <!ENTITY api-questions SYSTEM \""); w.write (nbRoot); w.write ("nbbuild/antsrc/org/netbeans/nbbuild/Arch-api-questions.xml\">\n");
-        w.write ("]>\n");
-        w.write ("\n");
-        w.write ("<api-answers\n");
-        w.write ("  question-version=\""); w.write (versionOfQuestions); w.write ("\"\n");
-        w.write ("  author=\"yourname@netbeans.org\"\n");
-        w.write (">\n\n");
-        w.write ("  &api-questions;\n");        
-        
-        writeQuestions (w, missing);
-        
-        w.write ("</api-answers>\n");
-        
-        w.close ();
+        try (Writer w = new FileWriter (questionsFile)) {
+            w.write ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+            w.write ("<!DOCTYPE api-answers PUBLIC \"-//NetBeans//DTD Arch Answers//EN\" \""); w.write (nbRoot); w.write ("nbbuild/antsrc/org/netbeans/nbbuild/Arch.dtd\" [\n");
+            w.write ("  <!ENTITY api-questions SYSTEM \""); w.write (nbRoot); w.write ("nbbuild/antsrc/org/netbeans/nbbuild/Arch-api-questions.xml\">\n");
+            w.write ("]>\n");
+            w.write ("\n");
+            w.write ("<api-answers\n");
+            w.write ("  question-version=\""); w.write (versionOfQuestions); w.write ("\"\n");
+            w.write ("  author=\"yourname@netbeans.org\"\n");
+            w.write (">\n\n");
+            w.write ("  &api-questions;\n");
+            
+            writeQuestions (w, missing);
+            
+            w.write ("</api-answers>\n");
+        }
     }
 
     private static Map<String,Element> readElements (Document q, String name) {
-        Map<String,Element> map = new HashMap<String,Element>();
+        Map<String,Element> map = new HashMap<>();
        
         NodeList list = q.getElementsByTagName(name);
         for (int i = 0; i < list.getLength(); i++) {
@@ -703,13 +667,15 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
         return null;
     }
     
-    private static Map<String,String> publicIds;
+    private static Set<String> localDTDs;
     static {
-        publicIds = new HashMap<String,String>();
-        publicIds.put("xhtml1-strict.dtd", "Arch-fake-xhtml.dtd");
-        publicIds.put("Arch.dtd", "Arch.dtd");
-        publicIds.put("Arch-api-questions.xml", "Arch-api-questions.xml");
-        
+        localDTDs = new HashSet<>();
+        localDTDs.add("Arch.dtd");
+        localDTDs.add("Arch-api-questions.xml");
+        localDTDs.add("xhtml1-strict.dtd");
+        localDTDs.add("xhtml-lat1.ent");
+        localDTDs.add("xhtml-special.ent");
+        localDTDs.add("xhtml-symbol.ent");
     }
 
     public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
@@ -718,46 +684,15 @@ public class Arch extends Task implements ErrorHandler, EntityResolver, URIResol
         int idx = systemId.lastIndexOf('/');
         String last = systemId.substring(idx + 1);
         
-        if (last.equals("xhtml1-strict.dtd")) {
-            // try to find relative libraries
-            String dtd = "nbbuild/external/xhtml1-dtds/xhtml1-strict.dtd".replace('/', File.separatorChar);
-            File f = questionsFile.getParentFile();
-            while (f != null) {
-                File check = new File(f, dtd);
-                if (check.isFile()) {
-                    String r = check.toURI().toString();
-                    log("Replacing entity " + publicId + " at " + systemId + " with " + r);
-                    return new InputSource(r);
-                }
-                f = f.getParentFile();
-            }
-        }
-        
-        String replace = publicIds.get(last);
-        if (replace == null) {
-            log("Not replacing id", Project.MSG_VERBOSE);
-            return null;
-        }
-        
-        try {
-            URL u = new URL(systemId);
-            u.openStream();
-            log("systemId " + systemId + " exists, leaving", Project.MSG_VERBOSE);
-            return null;
-        } catch (IOException ex) {
-            // ok
-        }
-        
-        InputSource is;
-        log("Replacing entity " + publicId + " at " + systemId + " with " + replace);
-        if (replace.startsWith("http://")) {
-            is = new InputSource(new URL(replace).openStream());
-            is.setSystemId(replace);
+        if (localDTDs.contains(last)) {
+            log("Resolved to embedded DTD");
+            InputSource is = new InputSource(Arch.class.getResourceAsStream(last));
+            is.setSystemId(systemId);
+            return is;
         } else {
-            is = new InputSource(Arch.class.getResourceAsStream(replace));
-            is.setSystemId(replace);
+            log("Not resolved");
+            return null;
         }
-        return is;
     }
 
     public Source resolve(String href, String base) throws TransformerException {
