@@ -1,43 +1,20 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
- * Other names may be trademarks of their respective owners.
- *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common
- * Development and Distribution License("CDDL") (collectively, the
- * "License"). You may not use this file except in compliance with the
- * License. You can obtain a copy of the License at
- * http://www.netbeans.org/cddl-gplv2.html
- * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
- * specific language governing permissions and limitations under the
- * License.  When distributing the software, include this License Header
- * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
- * particular file as subject to the "Classpath" exception as provided
- * by Oracle in the GPL Version 2 section of the License file that
- * accompanied this code. If applicable, add the following below the
- * License Header, with the fields enclosed by brackets [] replaced by
- * your own identifying information:
- * "Portions Copyrighted [year] [name of copyright owner]"
- *
- * If you wish your version of this file to be governed by only the CDDL
- * or only the GPL Version 2, indicate your decision by adding
- * "[Contributor] elects to include this software in this distribution
- * under the [CDDL or GPL Version 2] license." If you do not indicate a
- * single choice of license, a recipient has the option to distribute
- * your version of this file under either the CDDL, the GPL Version 2 or
- * to extend the choice of license to its licensees as provided above.
- * However, if you add GPL Version 2 code and therefore, elected the GPL
- * Version 2 license, then the option applies only if the new code is
- * made subject to such option by the copyright holder.
- *
- * Contributor(s):
- *
- * Portions Copyrighted 2009 Sun Microsystems, Inc.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 package org.netbeans.nbbuild;
@@ -90,7 +67,7 @@ import org.xml.sax.helpers.DefaultHandler;
  * @author Jaroslav Tulach <jtulach@netbeans.org>
  */
 public class AutoUpdate extends Task {
-    private List<Modules> modules = new ArrayList<Modules>();
+    private List<Modules> modules = new ArrayList<>();
     private FileSet nbmSet;
     private File download;
     private File dir;
@@ -157,7 +134,7 @@ public class AutoUpdate extends Task {
             }
             DirectoryScanner s = nbmSet.getDirectoryScanner(getProject());
             File basedir = s.getBasedir();
-            units = new HashMap<String, ModuleItem>();
+            units = new HashMap<>();
             for (String incl : s.getIncludedFiles()) {
                 File nbm = new File(basedir, incl);
                 try {
@@ -279,8 +256,7 @@ public class AutoUpdate extends Task {
                 module_version.setAttribute("origin", "Ant"); // XXX set to URL origin
                 module_version.setAttribute("specification_version", uu.getSpecVersion());
 
-                JarFile  zf = new JarFile(tmp);
-                try {
+                try (JarFile zf = new JarFile(tmp)) {
                 Manifest manifest = zf.getManifest();
                 if (manifest == null || manifest.getMainAttributes().getValue("Bundle-SymbolicName") == null) { // regular NBM
                 Enumeration<? extends ZipEntry> en = zf.entries();
@@ -349,21 +325,17 @@ public class AutoUpdate extends Task {
                     String relName = "config/Modules/" + dash + ".xml";
                     File configModulesXml = new File(whereTo, relName);
                     configModulesXml.getParentFile().mkdirs();
-                    PrintWriter w = new PrintWriter(configModulesXml);
-                    try {
+                    try (PrintWriter w = new PrintWriter(configModulesXml)) {
                         w.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE module PUBLIC \"-//NetBeans//DTD Module Status 1.0//EN\"\n                        \"http://www.netbeans.org/dtds/module-status-1_0.dtd\">\n<module name=\"");
                         w.print(uu.getCodeName());
                         w.print("\">\n    <param name=\"autoload\">true</param>\n    <param name=\"eager\">false</param>\n    <param name=\"jar\">modules/");
                         w.print(dash);
                         w.print(".jar</param>\n    <param name=\"reloadable\">false</param>\n</module>\n");
-                    } finally {
-                        w.close();
                     }
                     Element file = (Element) module_version.appendChild(doc.createElement("file"));
                     file.setAttribute("crc", String.valueOf(getFileCRC(configModulesXml)));
                     file.setAttribute("name", relName);
                     relName = "modules/" + dash + ".jar";
-                    zf.close();
                     File bundle = new File(whereTo, relName);
                     bundle.getParentFile().mkdirs();
                     FileUtils.getFileUtils().copyFile(tmp, bundle);
@@ -371,8 +343,6 @@ public class AutoUpdate extends Task {
                     file.setAttribute("crc", String.valueOf(getFileCRC(bundle)));
                     file.setAttribute("name", relName);
                 }
-                } finally {
-                    zf.close();
                 }
                 File tracking = new File(new File(whereTo, "update_tracking"), dash + ".xml");
                 log("Writing tracking file " + tracking, Project.MSG_VERBOSE);
@@ -461,9 +431,7 @@ public class AutoUpdate extends Task {
             //https://netbeans.org/bugzilla/show_bug.cgi?id=119861
             result = process.waitFor();
             process.destroy();
-        } catch (IOException e) {
-            e.printStackTrace();
-        } catch (InterruptedException e) {
+        } catch (IOException | InterruptedException e) {
             e.printStackTrace();
         }
         return result == 0;
@@ -510,7 +478,7 @@ public class AutoUpdate extends Task {
     }
 
     private Map<String,List<String>> findExistingModules(File... clusters) {
-        Map<String,List<String>> all = new HashMap<String, List<String>>();
+        Map<String,List<String>> all = new HashMap<>();
         for (File c : clusters) {
             File mc = new File(c, "update_tracking");
             final File[] arr = mc.listFiles();
@@ -564,7 +532,7 @@ public class AutoUpdate extends Task {
                     if (name == null || version == null) {
                         throw new BuildException("Cannot find version in " + config);
                     }
-                    arr = new ArrayList<String>();
+                    arr = new ArrayList<>();
                     arr.add(version);
                     toAdd.put(name, arr);
                     return;
