@@ -1,45 +1,20 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
- * Other names may be trademarks of their respective owners.
- *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common
- * Development and Distribution License("CDDL") (collectively, the
- * "License"). You may not use this file except in compliance with the
- * License. You can obtain a copy of the License at
- * http://www.netbeans.org/cddl-gplv2.html
- * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
- * specific language governing permissions and limitations under the
- * License.  When distributing the software, include this License Header
- * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
- * particular file as subject to the "Classpath" exception as provided
- * by Oracle in the GPL Version 2 section of the License file that
- * accompanied this code. If applicable, add the following below the
- * License Header, with the fields enclosed by brackets [] replaced by
- * your own identifying information:
- * "Portions Copyrighted [year] [name of copyright owner]"
- *
- * Contributor(s):
- *
- * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
- * Microsystems, Inc. All Rights Reserved.
- *
- * If you wish your version of this file to be governed by only the CDDL
- * or only the GPL Version 2, indicate your decision by adding
- * "[Contributor] elects to include this software in this distribution
- * under the [CDDL or GPL Version 2] license." If you do not indicate a
- * single choice of license, a recipient has the option to distribute
- * your version of this file under either the CDDL, the GPL Version 2 or
- * to extend the choice of license to its licensees as provided above.
- * However, if you add GPL Version 2 code and therefore, elected the GPL
- * Version 2 license, then the option applies only if the new code is
- * made subject to such option by the copyright holder.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 package org.netbeans.nbbuild;
@@ -100,11 +75,8 @@ public class FixTestDependencies extends Task {
                 throw new BuildException("project.xml file doesn't exist.");
             }
             byte xmlBytes [] = new byte[(int)projectXmlFile.length()];
-            FileInputStream prjFis = new FileInputStream(projectXmlFile);
-            try {
+            try (FileInputStream prjFis = new FileInputStream(projectXmlFile)) {
                 prjFis.read(xmlBytes);
-            } finally {
-                prjFis.close();
             }
             String xml = new String(xmlBytes);
             String oldXsd = "<data xmlns=\"http://www.netbeans.org/ns/nb-module-project/2";
@@ -138,10 +110,10 @@ public class FixTestDependencies extends Task {
                 // test if project.xml contains test-deps
                 if (td && !testFix) {
                     // yes -> exit
-                    xml = fixOpenideUtil(xml);
-                    PrintStream ps = new PrintStream(projectXmlFile);
-                    ps.print(xml);
-                    ps.close();                  
+                    xml = fixOpenideUtil(fixCoreStartup(xml));
+                    try (PrintStream ps = new PrintStream(projectXmlFile)) {
+                        ps.print(xml);
+                    }                  
                     return ;
                 }
                 Set<ModuleListParser.Entry> entries = getModuleList(projectType);
@@ -152,10 +124,10 @@ public class FixTestDependencies extends Task {
 
                 // unittest
                 //
-                Set<String> compileCNB = new TreeSet<String>();
-                Set<String> compileTestCNB = new TreeSet<String>();
-                Set<String> runtimeCNB = new TreeSet<String>();
-                Set<String> runtimeTestCNB = new TreeSet<String>();
+                Set<String> compileCNB = new TreeSet<>();
+                Set<String> compileTestCNB = new TreeSet<>();
+                Set<String> runtimeCNB = new TreeSet<>();
+                Set<String> runtimeTestCNB = new TreeSet<>();
 
                 Properties projectProperties = getTestProperties();
                 boolean found;
@@ -230,9 +202,12 @@ public class FixTestDependencies extends Task {
                 }
                 resultXml.append(xml.substring(moduleDepEnd + 1, xml.length()));
                 if (!testFix) {
-                   PrintStream ps = new PrintStream(projectXmlFile);
-                   ps.print(fixOpenideUtil(resultXml.toString()));
-                   ps.close();
+                    try (PrintStream ps = new PrintStream(projectXmlFile)) {
+                        ps.print(fixOpenideUtil(
+                                fixCoreStartup(
+                                        resultXml.toString()
+                                )));
+                    }
                 } else {
                     System.out.println(resultXml);
                 }
@@ -257,7 +232,7 @@ public class FixTestDependencies extends Task {
     }
     
     private Set<String> getCNBsFromEntries(Set<ModuleListParser.Entry> entries) {
-        Set<String> cnbs = new HashSet<String>();
+        Set<String> cnbs = new HashSet<>();
         for (ModuleListParser.Entry e : entries) {
             cnbs.add(e.getCnb());
         }
@@ -289,6 +264,7 @@ public class FixTestDependencies extends Task {
                          found = true;
                     } else if (token.endsWith("core/core.jar")) {
                         compileCNB.add("org.netbeans.core.startup");
+                        compileCNB.add("org.netbeans.core.startup.base");
                         found = true;
                     } else if (lastSlash != -1 && lastDot != -1 && lastSlash + 1< lastDot ) {
                         String codeBaseName = token.substring(lastSlash + 1, lastDot);
@@ -385,11 +361,8 @@ public class FixTestDependencies extends Task {
             return new Properties();
         }
         Properties props = new Properties();
-        FileInputStream fis = new FileInputStream(propertiesFile);
-        try {
+        try (FileInputStream fis = new FileInputStream(propertiesFile)) {
             props.load(fis);
-        } finally {
-            fis.close();
         }
         return props;
     }
@@ -409,15 +382,15 @@ public class FixTestDependencies extends Task {
             return;
         }
         try {
-            List<String> lines = new ArrayList<String>();
+            List<String> lines = new ArrayList<>();
             if (propertiesFile.isFile()) {
                 // read properties
-                BufferedReader reader = new BufferedReader (new FileReader(propertiesFile));
-                String line = null;
-                while ((line = reader.readLine()) != null) {
-                    lines.add(line);
+                try (BufferedReader reader = new BufferedReader (new FileReader(propertiesFile))) {
+                    String line;
+                    while ((line = reader.readLine()) != null) {
+                        lines.add(line);
+                    }
                 }
-                reader.close();
             }
             
             // merge properties
@@ -430,11 +403,11 @@ public class FixTestDependencies extends Task {
             }
 
             // store properties
-            PrintStream ps = new PrintStream(propertiesFile);
-            for (String l : lines) {
-                ps.println(l);
+            try (PrintStream ps = new PrintStream(propertiesFile)) {
+                for (String l : lines) {
+                    ps.println(l);
+                }
             }
-            ps.close();
             
         } catch (IOException ioe) {
             throw new BuildException(ioe);
@@ -443,7 +416,7 @@ public class FixTestDependencies extends Task {
     }
 
     private List<String> replaceProperty(String name, String value, List<String> lines) {
-        List<String> retLines = new ArrayList<String>();
+        List<String> retLines = new ArrayList<>();
         for (int i = 0 ; i < lines.size() ; i++) {
             String line = lines.get(i);
             String trimmedLine = line.trim();
@@ -465,6 +438,33 @@ public class FixTestDependencies extends Task {
         }
         return retLines;
     }
+    
+    private String fixCoreStartup(String xml) {
+        Pattern l = Pattern.compile(
+                "^ *<test-dependency>[^/]*" +
+                "<code-name-base>org.netbeans.core.startup.base</code-name-base>", Pattern.MULTILINE);
+        if (l.matcher(xml).find()) {
+            return xml;
+        }
+
+        Pattern p = Pattern.compile(
+                "^ *<test-dependency>[^/]*" +
+                "<code-name-base>org.netbeans.core.startup</code-name-base>", Pattern.MULTILINE);
+
+        Matcher m = p.matcher(xml);
+        if (m.find()) {
+            final String txt = "</test-dependency>";
+            final int s = m.start();
+            int end = xml.indexOf(txt, s);
+            if (end == -1) {
+                throw new BuildException("No end of dependency " + xml);
+            }
+            final int e = end + txt.length();
+            String dep = xml.substring(s, e);
+            return xml.substring(0, s) + dep + '\n' + dep.replace("org.netbeans.core.startup", "org.netbeans.core.startup.base") + xml.substring(e);
+        }
+        return xml;
+    }
 
     private String fixOpenideUtil(String xml) {
         Pattern l = Pattern.compile(
