@@ -1,45 +1,20 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
- * Other names may be trademarks of their respective owners.
- *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common
- * Development and Distribution License("CDDL") (collectively, the
- * "License"). You may not use this file except in compliance with the
- * License. You can obtain a copy of the License at
- * http://www.netbeans.org/cddl-gplv2.html
- * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
- * specific language governing permissions and limitations under the
- * License.  When distributing the software, include this License Header
- * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
- * particular file as subject to the "Classpath" exception as provided
- * by Oracle in the GPL Version 2 section of the License file that
- * accompanied this code. If applicable, add the following below the
- * License Header, with the fields enclosed by brackets [] replaced by
- * your own identifying information:
- * "Portions Copyrighted [year] [name of copyright owner]"
- *
- * Contributor(s):
- *
- * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
- * Microsystems, Inc. All Rights Reserved.
- *
- * If you wish your version of this file to be governed by only the CDDL
- * or only the GPL Version 2, indicate your decision by adding
- * "[Contributor] elects to include this software in this distribution
- * under the [CDDL or GPL Version 2] license." If you do not indicate a
- * single choice of license, a recipient has the option to distribute
- * your version of this file under either the CDDL, the GPL Version 2 or
- * to extend the choice of license to its licensees as provided above.
- * However, if you add GPL Version 2 code and therefore, elected the GPL
- * Version 2 license, then the option applies only if the new code is
- * made subject to such option by the copyright holder.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 package org.netbeans.nbbuild;
@@ -85,7 +60,7 @@ public class JHIndexer extends MatchingTask {
     private File db;
     private File basedir;
     private String locale;
-    private List<BrandedFileSet> brandings = new LinkedList<BrandedFileSet>();
+    private List<BrandedFileSet> brandings = new LinkedList<>();
 
     /** Set the location of <samp>jhall.jar</samp> or <samp>jsearch.jar</samp> (JavaHelp tools library). */
     public Path createClasspath() {
@@ -354,8 +329,7 @@ public class JHIndexer extends MatchingTask {
         try {
             File config = File.createTempFile ("jhindexer-config", ".txt");
             try {
-                OutputStream os = new FileOutputStream (config);
-                try {
+                try (OutputStream os = new FileOutputStream (config)) {
                     PrintWriter pw = new PrintWriter (os);
                     pw.println ("IndexRemove " + basedir + File.separator);
                     String message = "Files to be indexed:";
@@ -368,8 +342,6 @@ public class JHIndexer extends MatchingTask {
                     }
                     log (message, Project.MSG_VERBOSE);
                     pw.flush ();
-                } finally {
-                    os.close ();
                 }
                 AntClassLoader loader = new AntClassLoader(getProject(), classpath);
                 loader.addLoaderPackageRoot("com.sun.java.help.search");
@@ -381,7 +353,7 @@ public class JHIndexer extends MatchingTask {
                         "-db", db.getAbsolutePath()
                     );
                     if (locale != null) {
-                        args = new ArrayList<String>(args); // #35244
+                        args = new ArrayList<>(args); // #35244
                         args.add("-locale");
                         args.add(locale);
                     }
