@@ -1,44 +1,21 @@
 #!/bin/bash
 
- # DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
- #
- # Copyright 2012-2013 Oracle and/or its affiliates. All rights reserved.
- #
- # Oracle and Java are registered trademarks of Oracle and/or its affiliates.
- # Other names may be trademarks of their respective owners.
- #
- # The contents of this file are subject to the terms of either the GNU
- # General Public License Version 2 only ("GPL") or the Common
- # Development and Distribution License("CDDL") (collectively, the
- # "License"). You may not use this file except in compliance with the
- # License. You can obtain a copy of the License at
- # http://www.netbeans.org/cddl-gplv2.html
- # or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
- # specific language governing permissions and limitations under the
- # License.  When distributing the software, include this License Header
- # Notice in each file and include the License file at
- # nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
- # particular file as subject to the "Classpath" exception as provided
- # by Oracle in the GPL Version 2 section of the License file that
- # accompanied this code. If applicable, add the following below the
- # License Header, with the fields enclosed by brackets [] replaced by
- # your own identifying information:
- # "Portions Copyrighted [year] [name of copyright owner]"
- #
- # If you wish your version of this file to be governed by only the CDDL
- # or only the GPL Version 2, indicate your decision by adding
- # "[Contributor] elects to include this software in this distribution
- # under the [CDDL or GPL Version 2] license." If you do not indicate a
- # single choice of license, a recipient has the option to distribute
- # your version of this file under either the CDDL, the GPL Version 2 or
- # to extend the choice of license to its licensees as provided above.
- # However, if you add GPL Version 2 code and therefore, elected the GPL
- # Version 2 license, then the option applies only if the new code is
- # made subject to such option by the copyright holder.
- #
- # Contributor(s):
- #
- # Portions Copyrighted 2012 Sun Microsystems, Inc.
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
 
 set -x
 
@@ -120,15 +97,21 @@ if [ -z ${JDK_BUILDS_HOST} ]; then
     export JDK_BUILDS_HOST
 fi
 
-#JDK7_BUILDS_PATH=http://jre.us.oracle.com/java/re/jdk/7u80/promoted/all
+#JDK_BUILDS_HOST=http://jre.us.oracle.com
+if [ -z ${JRE_BUILDS_PATH} ]; then
+    JRE_BUILDS_PATH=java/re/jdk/8u101/promoted/
+    export JRE_BUILDS_PATH
+fi
+
+#JDK7_BUILDS_PATH=http://jre.us.oracle.com/java/re/jdk/7u75/promoted/all
 if [ -z ${JDK7_BUILDS_PATH} ]; then
-    JDK7_BUILDS_PATH=java/re/jdk/7u80/promoted/
+    JDK7_BUILDS_PATH=java/re/jdk/7u75/promoted/
     export JDK7_BUILDS_PATH
 fi
 
-#JDK8_BUILDS_PATH=http://jre.us.oracle.com/java/re/jdk/8u60/promoted/all/
+#JDK8_BUILDS_PATH=http://jre.us.oracle.com/java/re/jdk/8u121/promoted/all/
 if [ -z ${JDK8_BUILDS_PATH} ]; then
-    JDK8_BUILDS_PATH=java/re/jdk/8u60/promoted/
+    JDK8_BUILDS_PATH=java/re/jdk/8u121/promoted/
     export JDK8_BUILDS_PATH
 fi
 
@@ -142,7 +125,7 @@ if [ -z ${DONT_PACK_LOCALIZATION_JARS_ON_MAC} ]; then
     export DONT_PACK_LOCALIZATION_JARS_ON_MAC
 fi
 
-export ANT_OPTS="-Xmx2G -XX:MaxPermSize=500m"
+export ANT_OPTS=$ANT_OPTS" -Xmx2G -XX:MaxPermSize=500m"
 
 if [ -n ${JDK_HOME} ] && [ -z ${JAVA_HOME} ] ; then
     export JAVA_HOME=$JDK_HOME
