@@ -45,7 +45,6 @@ package org.netbeans.modules.profiler.heapwalk;
 
 import java.text.NumberFormat;
 import org.netbeans.api.progress.ProgressHandle;
-import org.netbeans.api.progress.ProgressHandleFactory;
 import org.netbeans.lib.profiler.global.CommonConstants;
 import org.netbeans.lib.profiler.heap.*;
 import org.netbeans.modules.profiler.heapwalk.ui.ClassesListControllerUI;
@@ -240,18 +239,13 @@ public class ClassesListController extends AbstractController {
             name = jc.getName();
             
             if (realClass) {
-                instancesCount = jc.getInstancesCount();
-                instanceSize = jc.getInstanceSize();
-                allInstancesSize = jc.getAllInstancesSize();
-                retainedSizeByClass = compareRetained ? jc.getRetainedSizeByClass() : -1;
+                retainedSizeByClass = compareRetained ? 0 : -1;
                 real = jc;
             } else {
-                instancesCount = -jc.getInstancesCount();
-                instanceSize = -jc.getInstanceSize();
-                allInstancesSize = -jc.getAllInstancesSize();
-                retainedSizeByClass = compareRetained ? -jc.getRetainedSizeByClass() : -1;
+                retainedSizeByClass = compareRetained ? 0 : -1;
                 real = null;
             }
+            add(jc);
         }
         
         static String createID(JavaClass jc) {
@@ -287,6 +281,20 @@ public class ClassesListController extends AbstractController {
             retainedSizeByClass += djc.retainedSizeByClass;
             real = djc.real;
         }
+     
+        private void add(JavaClass jc) {
+            if (real != null) {
+                instancesCount += jc.getInstancesCount();
+                instanceSize += jc.getInstanceSize();
+                allInstancesSize += jc.getAllInstancesSize();
+                retainedSizeByClass += retainedSizeByClass != -1 ? jc.getRetainedSizeByClass() : 0;
+            } else {
+                instancesCount -= jc.getInstancesCount();
+                instanceSize -= jc.getInstanceSize();
+                allInstancesSize -= jc.getAllInstancesSize();
+                retainedSizeByClass -= retainedSizeByClass != -1 ? jc.getRetainedSizeByClass() : 0;
+            }
+        }
         
         public boolean equals(Object o) {
             if (o instanceof DiffJavaClass)
@@ -331,6 +339,11 @@ public class ClassesListController extends AbstractController {
             return null;
         }
 
+        public Iterator getInstancesIterator() {
+            // Not implemented
+            return null;
+        }
+
         public int getInstancesCount() {
             return instancesCount;
         }
@@ -484,8 +497,13 @@ public class ClassesListController extends AbstractController {
         List<JavaClass> classes1 = h1.getAllClasses();
         for (JavaClass jc1 : classes1) {
             String id1 = DiffJavaClass.createID(jc1);
-            DiffJavaClass djc1 = DiffJavaClass.createExternal(jc1, compareRetained);
-            classes.put(id1, djc1);
+            DiffJavaClass djc1 = classes.get(id1);
+            if (djc1 == null) {
+                djc1 = DiffJavaClass.createExternal(jc1, compareRetained);
+                classes.put(id1, djc1);
+            } else {
+                djc1.add(jc1);
+            }
         }
         
         List<JavaClass> classes2 = h2.getAllClasses();
@@ -523,7 +541,7 @@ public class ClassesListController extends AbstractController {
         ProgressHandle pHandle = null;
 
         try {
-            pHandle = ProgressHandleFactory.createHandle(Bundle.ClassesListController_AnalyzingClassesMsg());
+            pHandle = ProgressHandle.createHandle(Bundle.ClassesListController_AnalyzingClassesMsg());
             pHandle.setInitialDelay(0);
             pHandle.start();
 
