@@ -55,12 +55,14 @@ import org.openide.util.lookup.ServiceProvider;
 /**
  *
  * @author Tomas Hurka
+ * @author Jiri Sedlacek
  */
 @ServiceProvider(service=DetailsProvider.class)
 public class PlatformDetailsProvider extends DetailsProvider.Basic {
 
-    private static final String STANDARD_MODULE = "org.netbeans.StandardModule+"; // NOI18N
-    private static final String STANDARD_MODULE_DATA = "org.netbeans.StandardModuleData"; // NOI18N
+    private static final String STANDARD_MODULE = "org.netbeans.Module+"; // NOI18N
+    private static final String MODULE_DATA = "org.netbeans.ModuleData+"; // NOI18N
+    private static final String DEPENDENCY = "org.openide.modules.Dependency"; // NOI18N
     private static final String SPECIFICATION_VERSION = "org.openide.modules.SpecificationVersion"; // NOI18N
     private static final String ABSTRACT_NODE = "org.openide.nodes.AbstractNode+"; // NOI18N
     private static final String MULTI_FILE_ENTRY = "org.openide.loaders.MultiDataObject$Entry+"; // NOI18N
@@ -80,6 +82,7 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
     private static final String FIXED_21_30 = "org.openide.util.CharSequences$Fixed6Bit_21_30"; // NOI18N
     private static final String BYTE_BASED_SEQUENCE = "org.openide.util.CharSequences$ByteBasedSequence"; // NOI18N
     private static final String CHAR_BASED_SEQUENCE = "org.openide.util.CharSequences$CharBasedSequence"; // NOI18N
+    private static final String REQUEST_PROCESSOR = "org.openide.util.RequestProcessor";     // NOI18N
     
     LinkedHashMap<Long, String> cache = new LinkedHashMap<Long, String>(10000) {
 
@@ -90,12 +93,13 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
     };
     
     public PlatformDetailsProvider() {
-        super(STANDARD_MODULE,STANDARD_MODULE_DATA,SPECIFICATION_VERSION,
+        super(STANDARD_MODULE,MODULE_DATA,DEPENDENCY,SPECIFICATION_VERSION,
               ABSTRACT_NODE,MULTI_FILE_ENTRY,DATA_OBJECT,JAR_FILESYSTEM,
               FILE_OBJ,FOLDER_OBJ, FILE_NAME,FOLDER_NAME,ABSTRACT_FOLDER,
               BFS_BASE,
               FIXED_0_7,FIXED_8_15,FIXED_16_23,FIXED_1_10,FIXED_11_20,
-              FIXED_21_30,BYTE_BASED_SEQUENCE,CHAR_BASED_SEQUENCE);
+              FIXED_21_30,BYTE_BASED_SEQUENCE,CHAR_BASED_SEQUENCE,
+              REQUEST_PROCESSOR);
     }
 
     @Override
@@ -117,6 +121,12 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
                 return codeName;
             }
             return DetailsUtils.getInstanceFieldString(instance, "data", heap);     // NOI18N
+        } else if (DEPENDENCY.equals(className)) {
+            String name = DetailsUtils.getInstanceFieldString(instance, "name", heap);     // NOI18N
+            String version = DetailsUtils.getInstanceFieldString(instance, "version", heap);     // NOI18N
+            int type = DetailsUtils.getIntFieldValue(instance, "type", -1); // NOI18N
+            int comparison = DetailsUtils.getIntFieldValue(instance, "comparison", -1); // NOI18N
+            return DependencyResolver.toString(name, version, type, comparison);
         } else if (SPECIFICATION_VERSION.equals(className)) {
             PrimitiveArrayInstance digits = (PrimitiveArrayInstance) instance.getValueOfField("digits"); // NOI18N
             if (digits != null) {
@@ -128,8 +138,11 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
                 }
                 return specVersion.substring(0, specVersion.length()-1);
             }
-        } else if (STANDARD_MODULE_DATA.equals(className)) {
-            return DetailsUtils.getInstanceFieldString(instance, "codeName", heap);     // NOI18N
+        } else if (MODULE_DATA.equals(className)) {
+            String name = DetailsUtils.getInstanceFieldString(instance, "codeName", heap);     // NOI18N
+            String version = DetailsUtils.getInstanceFieldString(instance, "specVers", heap);     // NOI18N
+            String implVer = DetailsUtils.getInstanceFieldString(instance, "implVersion", heap);       // NOI18N
+            return String.format("%s [%s %s]", name, version, implVer);
         } else if (ABSTRACT_NODE.equals(className)) {
             String name = DetailsUtils.getInstanceFieldString(instance, "displayName", heap); // NOI18N
 
@@ -226,6 +239,8 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
             }
         } else if (CHAR_BASED_SEQUENCE.equals(className)) {
             return DetailsUtils.getInstanceFieldString(instance, "value", heap);    // NOI18N
+        } else if (REQUEST_PROCESSOR.equals(className)) {
+            return DetailsUtils.getInstanceFieldString(instance, "name", heap);     // NOI18N
         }
         return null;
     }
@@ -1601,6 +1616,50 @@ public class PlatformDetailsProvider extends DetailsProvider.Basic {
      */
     private interface CompactCharSequence extends CharSequence {
     }
+    
+    
+    private static class DependencyResolver {
+        
+        private final static int TYPE_MODULE = 1;
+        private final static int TYPE_PACKAGE = 2;
+        private final static int TYPE_JAVA = 3;
+        private final static int TYPE_IDE = 4;
+        private final static int TYPE_REQUIRES = 5;
+        private final static int TYPE_NEEDS = 6;
+        private final static int TYPE_RECOMMENDS = 7;
+        private final static int COMPARE_SPEC = 1;
+        private final static int COMPARE_IMPL = 2;
+        private final static int COMPARE_ANY = 3;
+        
+        static String toString(String name, String version, int type, int comparison) {
+            StringBuilder buf = new StringBuilder(100);
+
+            if (type == TYPE_MODULE) {
+                buf.append("module "); // NOI18N
+            } else if (type == TYPE_PACKAGE) {
+                buf.append("package "); // NOI18N
+            } else if (type == TYPE_REQUIRES) {
+                buf.append("requires "); // NOI18N
+            } else if (type == TYPE_NEEDS) {
+                buf.append("needs "); // NOI18N
+            } else if (type == TYPE_RECOMMENDS) {
+                buf.append("recommends "); // NOI18N
+            }
+
+            buf.append(name);
+
+            if (comparison == COMPARE_IMPL) {
+                buf.append(" = "); // NOI18N
+                buf.append(version);
+            } else if (comparison == COMPARE_SPEC) {
+                buf.append(" > "); // NOI18N
+                buf.append(version);
+            }
+
+            return buf.toString();
+        }
+        
+    }
 
     //</editor-fold>
 
