@@ -100,6 +100,7 @@ import javax.swing.table.TableColumnModel;
 import javax.swing.tree.TreePath;
 import org.netbeans.lib.profiler.heap.Instance;
 import org.netbeans.lib.profiler.results.CCTNode;
+import org.netbeans.lib.profiler.utils.VMUtils;
 import org.netbeans.modules.profiler.api.icons.GeneralIcons;
 import org.netbeans.modules.profiler.api.GoToSource;
 import org.netbeans.modules.profiler.api.icons.Icons;
@@ -931,7 +932,7 @@ public class ReferencesBrowserControllerUI extends JTitledPanel {
                     className = instanceNode.getInstance().getJavaClass().getName();
                 }
             }
-            showSourceItem.setEnabled(className != null && !BrowserUtils.isPrimitiveType(BrowserUtils.getArrayBaseType(className)));
+            showSourceItem.setEnabled(className != null && !VMUtils.isPrimitiveType(BrowserUtils.getArrayBaseType(className)));
         }
         showInThreadsItem.setEnabled(false);
         if (node instanceof HeapWalkerInstanceNode) {
