@@ -46,12 +46,13 @@ package org.netbeans.modules.profiler;
 import java.awt.Cursor;
 import java.awt.Window;
 import java.io.*;
+import java.nio.file.Files;
+import java.nio.file.StandardCopyOption;
 import java.util.*;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.swing.JFileChooser;
 import org.netbeans.api.progress.ProgressHandle;
-import org.netbeans.api.progress.ProgressHandleFactory;
 import org.netbeans.lib.profiler.ProfilerClient;
 import org.netbeans.lib.profiler.ProfilerEngineSettings;
 import org.netbeans.lib.profiler.ProfilerLogger;
@@ -67,6 +68,8 @@ import org.netbeans.lib.profiler.results.RuntimeCCTNode;
 import org.netbeans.lib.profiler.results.cpu.CPUCCTProvider;
 import org.netbeans.lib.profiler.results.cpu.CPUResultsDiff;
 import org.netbeans.lib.profiler.results.cpu.CPUResultsSnapshot;
+import org.netbeans.lib.profiler.results.jdbc.JdbcResultsDiff;
+import org.netbeans.lib.profiler.results.jdbc.JdbcResultsSnapshot;
 import org.netbeans.lib.profiler.results.memory.AllocMemoryResultsDiff;
 import org.netbeans.lib.profiler.results.memory.AllocMemoryResultsSnapshot;
 import org.netbeans.lib.profiler.results.memory.LivenessMemoryResultsDiff;
@@ -74,9 +77,10 @@ import org.netbeans.lib.profiler.results.memory.LivenessMemoryResultsSnapshot;
 import org.netbeans.lib.profiler.results.memory.MemoryCCTProvider;
 import org.netbeans.lib.profiler.results.memory.SampledMemoryResultsDiff;
 import org.netbeans.lib.profiler.results.memory.SampledMemoryResultsSnapshot;
+import org.netbeans.lib.profiler.ui.swing.ExportUtils;
 import org.netbeans.lib.profiler.utils.StringUtils;
 import org.netbeans.modules.profiler.api.ProfilerDialogs;
-import org.netbeans.modules.profiler.api.project.ProjectStorage;
+import org.netbeans.modules.profiler.api.ProfilerStorage;
 import org.netbeans.modules.profiler.utilities.ProfilerUtils;
 import org.openide.cookies.OpenCookie;
 import org.openide.filesystems.FileLock;
@@ -106,7 +110,7 @@ import org.openide.windows.WindowManager;
     "ResultsManager_SnapshotDeleteFailedMsg=Failed to delete the snapshot file: {0}",
     "ResultsManager_CantFindSnapshotLocationMsg=Cannot find default location for snapshot in project: {0}",
     "ResultsManager_SnapshotCreateInProjectFailedMsg=Failed to create snapshot file in project: {0}",
-    "ResultsManager_SnapshotLoadFailedMsg=Error while loading snapshot: {0}",
+    "ResultsManager_SnapshotLoadFailed=Error while loading snapshot: {0}",
     "ResultsManager_SnapshotsLoadFailedMsg=Loading snapshots failed.",
     "ResultsManager_ObtainSavedSnapshotsFailedMsg=Failed to obtain list of saved snaphshots for project: {0}",
     "ResultsManager_SelectDirDialogCaption=Select Target Directory",
@@ -114,7 +118,8 @@ import org.openide.windows.WindowManager;
     "ResultsManager_OverwriteFileDialogCaption=Overwrite Existing File?",
     "ResultsManager_OverwriteFileDialogMsg=The target folder already contains file {0}\n Do you want to overwrite this file?",
     "ResultsManager_FileDeleteFailedMsg=Cannot delete the existing file: {0}",
-    "ResultsManager_SnapshotExportFailedMsg=Failed to save snapshot: {0}",
+    "ResultsManager_SnapshotExportFailedMsg=<html><b>Failed to export snapshot:</b><br><br>{0}</html>",
+    "ResultsManager_ExportSnapshotData=Export Snapshot Data",
     "ResultsManager_SelectFileOrDirDialogCaption=Select File or Directory",
     "ResultsManager_ProfilerSnapshotFileFilter=Profiler Snapshot File (*.{0})",
     "ResultsManager_ProfilerHeapdumpFileFilter=Heap Dump File (*.{0})",
@@ -122,7 +127,7 @@ import org.openide.windows.WindowManager;
     "ResultsManager_CannotCompareSnapshotsMsg=<html><b>Cannot compare snapshots:</b><br><br>  {0}<br>  {1}<br><br>Make sure that both snaphots are the same type.</html>",
     "ResultsManager_DirectoryDoesntExistCaption=Selected Directory Does Not Exist",
     "ResultsManager_DirectoryDoesntExistMsg=The directory you have selected does not exist.\nDo you want to create the directory?",
-    "ResultsManager_SnapshotLoadFailed=<html>Snapshot <b>{0}</b> failed to load</html>",
+    "ResultsManager_SnapshotLoadFailedMsg=<html><b>Snapshot {0} failed to load.</b><br><br>{1}</html>",
     "ResultsManager_CannotOpenSnapshotMsg=<html><b>Cannot open profiler snapshot.</b><br><br>Attempting to open null snapshot.<br>Check the logfile for details.</html>",
     "ResultsManager_CpuSnapshotDisplayName=cpu: {0}",
     "ResultsManager_MemorySnapshotDisplayName=mem: {0}",
@@ -243,16 +248,17 @@ public final class ResultsManager {
         } else {
             displayName = fileName;
         }
-        switch (snapshotType) {
-            case LoadedSnapshot.SNAPSHOT_TYPE_CPU:
-                return Bundle.ResultsManager_CpuSnapshotDisplayName(displayName);
-            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_ALLOCATIONS:
-            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_LIVENESS:
-            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_SAMPLED:
-                return Bundle.ResultsManager_MemorySnapshotDisplayName(displayName);
-            default:
-                return displayName;
-        }
+//        switch (snapshotType) {
+//            case LoadedSnapshot.SNAPSHOT_TYPE_CPU:
+//                return Bundle.ResultsManager_CpuSnapshotDisplayName(displayName);
+//            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_ALLOCATIONS:
+//            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_LIVENESS:
+//            case LoadedSnapshot.SNAPSHOT_TYPE_MEMORY_SAMPLED:
+//                return Bundle.ResultsManager_MemorySnapshotDisplayName(displayName);
+//            default:
+//                return displayName;
+//        }
+        return displayName;
     }
     
     public String getDefaultHeapDumpFileName(long time) {
@@ -278,7 +284,9 @@ public final class ResultsManager {
         } else {
             displayName = fileName;
         }
-        return Bundle.ResultsManager_HeapSnapshotDisplayName(displayName);
+        
+        return displayName;
+//        return Bundle.ResultsManager_HeapSnapshotDisplayName(displayName);
     }
 
     public LoadedSnapshot[] getLoadedSnapshots() {
@@ -344,11 +352,12 @@ public final class ResultsManager {
         private boolean isSomeResultsAvailable() {
             // check that we have data for snapshot, CPU profiling can have non-empty
             // batch, but this batch can contain only marker methods
-            ProfilerClient client = Profiler.getDefault().getTargetAppRunner().getProfilerClient();
-            int instrType = client.getCurrentInstrType();
+            Profiler profiler = Profiler.getDefault();
+            ProfilerClient client = profiler.getTargetAppRunner().getProfilerClient();
+            int profilingType = profiler.getLastProfilingSettings().getProfilingType();
 
-            if (instrType == ProfilerEngineSettings.INSTR_RECURSIVE_FULL ||
-                instrType == ProfilerEngineSettings.INSTR_RECURSIVE_SAMPLED) {
+            if (profilingType == ProfilingSettings.PROFILE_CPU_ENTIRE ||
+                profilingType == ProfilingSettings.PROFILE_CPU_PART) {
                 try {
                     // construct snapshot and check that it has some data
                     client.getCPUProfilingResultsSnapshot(false);
@@ -374,13 +383,17 @@ public final class ResultsManager {
     public void compareSnapshots(FileObject snapshot1FO, FileObject snapshot2FO) {
         LoadedSnapshot s1 = null;
         LoadedSnapshot s2 = null;
+        
+        FileObject snapshotFO = snapshot1FO;
 
         try {
             s1 = findAlreadyLoadedSnapshot(snapshot1FO);
 
             if (s1 == null) {
-                s1 = loadSnapshotFromFileObject(snapshot1FO);
+                s1 = loadSnapshotImpl(snapshot1FO);
             }
+            
+            snapshotFO = snapshot2FO;
 
             s2 = findAlreadyLoadedSnapshot(snapshot2FO);
 
@@ -388,7 +401,8 @@ public final class ResultsManager {
                 s2 = loadSnapshotFromFileObject(snapshot2FO);
             }
         } catch (Exception e) {
-            LOGGER.log(Level.SEVERE, Bundle.ResultsManager_SnapshotLoadFailed(e.getMessage()), e);
+            LOGGER.log(Level.INFO, Bundle.ResultsManager_SnapshotLoadFailed(snapshotFO.getPath()), e);
+            ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailedMsg(snapshotFO.getNameExt(), e.getMessage()));
 
             return;
         }
@@ -399,8 +413,8 @@ public final class ResultsManager {
             ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotsLoadFailedMsg());
         }
     }
-
-    public void compareSnapshots(LoadedSnapshot s1, LoadedSnapshot s2) {
+    
+    public ResultsSnapshot createDiffSnapshot(LoadedSnapshot s1, LoadedSnapshot s2) {
         ResultsSnapshot snap1 = s1.getSnapshot();
         ResultsSnapshot snap2 = s2.getSnapshot();
         ResultsSnapshot diff = null;
@@ -424,8 +438,7 @@ public final class ResultsManager {
                              sn2.getClassNames(), sn2.getObjectsCounts(), sn2.getObjectsSizePerClass(), Integer.MAX_VALUE);
             diff = new AllocMemoryResultsDiff((AllocMemoryResultsSnapshot)snap1,
                                               (AllocMemoryResultsSnapshot)snap2);
-        }
-        else if (snap1 instanceof LivenessMemoryResultsSnapshot && snap2 instanceof LivenessMemoryResultsSnapshot) {
+        } else if (snap1 instanceof LivenessMemoryResultsSnapshot && snap2 instanceof LivenessMemoryResultsSnapshot) {
             LivenessMemoryResultsSnapshot sn1 = (LivenessMemoryResultsSnapshot)snap1;
             LivenessMemoryResultsSnapshot sn2 = (LivenessMemoryResultsSnapshot)snap2;
 // Note: using track each 1 object to prevent unnecessary division, the data are always correct for liveness results
@@ -437,20 +450,28 @@ public final class ResultsManager {
                              sn2.getClassNames(), sn2.getNTrackedLiveObjects(), sn2.getTrackedLiveObjectsSize(), 1);
             diff = new LivenessMemoryResultsDiff((LivenessMemoryResultsSnapshot)snap1,
                                                  (LivenessMemoryResultsSnapshot)snap2);
-        }
-        else if (snap1 instanceof CPUResultsSnapshot && snap2 instanceof CPUResultsSnapshot) {
+        } else if (snap1 instanceof CPUResultsSnapshot && snap2 instanceof CPUResultsSnapshot) {
             diff = new CPUResultsDiff((CPUResultsSnapshot)snap1, (CPUResultsSnapshot)snap2);
+        } else if (snap1 instanceof JdbcResultsSnapshot && snap2 instanceof JdbcResultsSnapshot) {
+            diff = new JdbcResultsDiff((JdbcResultsSnapshot)snap1, (JdbcResultsSnapshot)snap2);
         }
+        
+        return diff;
+    }
 
-        if (diff != null) {
-            SnapshotsDiffWindow sdw = SnapshotsDiffWindow.get(diff, s1, s2);
-            sdw.open();
-            sdw.requestActive();
-        } else {
-            ProfilerDialogs.displayError(Bundle.ResultsManager_CannotCompareSnapshotsMsg(
-                                            s1.getFile().getName(), 
-                                            s2.getFile().getName()));
-        }
+    public void compareSnapshots(final LoadedSnapshot s1, final LoadedSnapshot s2) {
+        CommonUtils.runInEventDispatchThread(new Runnable() {
+            public void run() {
+                SnapshotResultsWindow srw = SnapshotResultsWindow.get(s1, 0, false);
+                if (!srw.setRefSnapshot(s2)) {
+                    ProfilerDialogs.displayError(Bundle.ResultsManager_CannotCompareSnapshotsMsg(
+                                                 s1.getFile().getName(), s2.getFile().getName()));
+                } else {
+                    srw.open();
+                    srw.requestActive();
+                }
+            }
+        });
     }
     
     static void checkObjectSizes(String[] names1, int[] counts1, long[] sizes1, int n1,
@@ -464,7 +485,7 @@ public final class ResultsManager {
                 idx1 = i;
                 break;
             }
-        if (idx1 == -1) return; // Should not happen
+        if (idx1 == -1 || counts1[idx1] == 0) return; // Should not happen
         
         int idx2 = -1;
         for (int i = 0; i < names2.length; i++)
@@ -472,7 +493,7 @@ public final class ResultsManager {
                 idx2 = i;
                 break;
             }
-        if (idx2 == -1) return; // Should not happen
+        if (idx2 == -1 || counts2[idx2] == 0) return; // Should not happen
         
         // Note: instrumented allocations not compared because of #236363
         long objsize1 = n1 == 1 ? sizes1[idx1] / counts1[idx1] : 0;
@@ -550,7 +571,7 @@ public final class ResultsManager {
                 return;
             }
         }
-        final ProgressHandle ph = ProgressHandleFactory.createHandle(Bundle.MSG_SavingSnapshots());
+        final ProgressHandle ph = ProgressHandle.createHandle(Bundle.MSG_SavingSnapshots());
         ph.setInitialDelay(500);
         ph.start();
         ProfilerUtils.runInProfilerRequestProcessor(new Runnable() {
@@ -623,7 +644,7 @@ public final class ResultsManager {
             if (project == null && directory != null) {
                 snapshotsFolder = FileUtil.toFileObject(directory);
             } else {
-                snapshotsFolder = ProjectStorage.getSettingsFolder(project, false);
+                snapshotsFolder = ProfilerStorage.getProjectFolder(project, false);
             }
 
             if (snapshotsFolder == null) {
@@ -652,10 +673,7 @@ public final class ResultsManager {
                     }
                 });
 
-            FileObject[] ret = new FileObject[files.size()];
-            files.toArray(ret);
-
-            return ret;
+            return (FileObject[])files.toArray(new FileObject[0]);
         } catch (IOException e) {
             LOGGER.log(Level.SEVERE, Bundle.ResultsManager_ObtainSavedSnapshotsFailedMsg(e.getMessage()), e);
 
@@ -670,7 +688,7 @@ public final class ResultsManager {
             if (project == null && directory != null) {
                 snapshotsFolder = FileUtil.toFileObject(directory);
             } else {
-                snapshotsFolder = ProjectStorage.getSettingsFolder(project, false);
+                snapshotsFolder = ProfilerStorage.getProjectFolder(project, false);
             }
 
             if (snapshotsFolder == null) {
@@ -701,10 +719,7 @@ public final class ResultsManager {
                     }
                 });
 
-            FileObject[] ret = new FileObject[files.size()];
-            files.toArray(ret);
-
-            return ret;
+            return (FileObject[])files.toArray(new FileObject[0]);
         } catch (IOException e) {
             LOGGER.log(Level.SEVERE, Bundle.ResultsManager_ObtainSavedSnapshotsFailedMsg(e.getMessage()), e);
 
@@ -712,11 +727,53 @@ public final class ResultsManager {
         }
     }
 
+    public boolean hasSnapshotsFor(Lookup.Provider project) {
+        try {
+            FileObject snapshotsFolder = ProfilerStorage.getProjectFolder(project, false);
+            FileObject[] children;
+            
+            if (snapshotsFolder == null) {
+                return false;
+            }
+            snapshotsFolder.refresh();
+            children = snapshotsFolder.getChildren();
+            for (FileObject child : children) {
+                if (child.getExt().equalsIgnoreCase(SNAPSHOT_EXTENSION)) return true;
+                if (checkHprofFile(FileUtil.toFile(child))) return true;
+            }
+        } catch (IOException e) {
+            LOGGER.log(Level.SEVERE, Bundle.ResultsManager_ObtainSavedSnapshotsFailedMsg(e.getMessage()), e);            
+        }
+        return false;
+    }
+    
+    public int getSnapshotsCountFor(Lookup.Provider project) {
+        int count = 0;
+        try {
+            FileObject snapshotsFolder = ProfilerStorage.getProjectFolder(project, false);
+            FileObject[] children;
+            
+            if (snapshotsFolder == null) {
+                return count;
+            }
+            snapshotsFolder.refresh();
+            children = snapshotsFolder.getChildren();
+            for (FileObject child : children) {
+                if (child.getExt().equalsIgnoreCase(SNAPSHOT_EXTENSION) ||
+                    checkHprofFile(FileUtil.toFile(child))) count++;
+            }
+        } catch (IOException e) {
+            LOGGER.log(Level.SEVERE, Bundle.ResultsManager_ObtainSavedSnapshotsFailedMsg(e.getMessage()), e);            
+        }
+        return count;
+    }
+    
     public LoadedSnapshot loadSnapshot(FileObject selectedFile) {
         try {
             return loadSnapshotImpl(selectedFile);
         } catch (IOException e) {
-            LOGGER.log(Level.SEVERE, Bundle.ResultsManager_SnapshotLoadFailed(e.getMessage()), e);
+            LOGGER.log(Level.INFO, Bundle.ResultsManager_SnapshotLoadFailed(selectedFile.getPath()), e);
+            ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailedMsg(selectedFile.getNameExt(), e.getMessage()));
 
             return null;
         }
@@ -731,7 +788,8 @@ public final class ResultsManager {
                     ret[i] = loadSnapshotImpl(selectedFiles[i]);
                 }
             } catch (IOException e) {
-                ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailedMsg(selectedFiles[i].getNameExt()));
+                LOGGER.log(Level.INFO, Bundle.ResultsManager_SnapshotLoadFailed(selectedFiles[i].getPath()), e);
+                ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailedMsg(selectedFiles[i].getNameExt(), e.getMessage()));
             }
         }
 
@@ -790,10 +848,8 @@ public final class ResultsManager {
                 srw.requestActive(); // activate the last one
             }
         } catch (Exception e) {
-            ProfilerLogger.log(e);
-
-            ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailed(
-                    loadedSnapshot.getFile().getAbsolutePath()));
+            LOGGER.log(Level.INFO, Bundle.ResultsManager_SnapshotLoadFailed(loadedSnapshot.getFile().getAbsoluteFile()), e);
+            ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotLoadFailedMsg(loadedSnapshot.getFile().getName(), e.getMessage()));
         }
     }
 
@@ -809,27 +865,32 @@ public final class ResultsManager {
         }
 
         try {
-            final TargetAppRunner runner = Profiler.getDefault().getTargetAppRunner();
+            final Profiler profiler = Profiler.getDefault();
+            final TargetAppRunner runner = profiler.getTargetAppRunner();
             final ProfilerClient client = runner.getProfilerClient();
-            final int currentInstrType = client.getCurrentInstrType();
+            final int currentProfilingType = profiler.getLastProfilingSettings().getProfilingType();
 
             try {
-                switch (currentInstrType) {
-                    case ProfilerEngineSettings.INSTR_OBJECT_ALLOCATIONS:
-                    case ProfilerEngineSettings.INSTR_OBJECT_LIVENESS:
-                    case ProfilerEngineSettings.INSTR_NONE_MEMORY_SAMPLING:
+                switch (currentProfilingType) {
+                    case ProfilingSettings.PROFILE_MEMORY_ALLOCATIONS:
+                    case ProfilingSettings.PROFILE_MEMORY_LIVENESS:
+                    case ProfilingSettings.PROFILE_MEMORY_SAMPLING:
                         snapshot = client.getMemoryProfilingResultsSnapshot(reqeustData);
 
                         break;
-                    case ProfilerEngineSettings.INSTR_RECURSIVE_FULL:
-                    case ProfilerEngineSettings.INSTR_RECURSIVE_SAMPLED:
-                    case ProfilerEngineSettings.INSTR_NONE_SAMPLING:
+                    case ProfilingSettings.PROFILE_CPU_ENTIRE:
+                    case ProfilingSettings.PROFILE_CPU_PART:
+                    case ProfilingSettings.PROFILE_CPU_SAMPLING:
                         snapshot = client.getCPUProfilingResultsSnapshot(reqeustData);
 
                         break;
-                    case ProfilerEngineSettings.INSTR_CODE_REGION:
+                    case ProfilingSettings.PROFILE_CPU_STOPWATCH:
                         snapshot = client.getCodeRegionProfilingResultsSnapshot();
 
+                        break;
+                    case ProfilingSettings.PROFILE_CPU_JDBC:
+                        snapshot = client.getJdbcProfilingResultsSnapshot(reqeustData);
+
                         break;
                 }
             } catch (ClientUtils.TargetAppOrVMTerminated e1) {
@@ -843,7 +904,7 @@ public final class ResultsManager {
                     runner.resetTimers(); // reset the server data
                 } catch (ClientUtils.TargetAppOrVMTerminated targetAppOrVMTerminated) {
                     // the target app has died; clean up all client data
-                    runner.getProfilerClient().resetClientData();
+                    client.resetClientData();
                 }
 
                 LOGGER.log(Level.SEVERE, Bundle.ResultsManager_OutOfMemoryMsg(), e);
@@ -876,6 +937,97 @@ public final class ResultsManager {
     public boolean resultsAvailable() {
         return resultsAvailable;
     }
+    
+    
+    public static interface SnapshotHandle {
+        public LoadedSnapshot getSnapshot();
+    }
+    
+    public ExportUtils.Exportable createSnapshotExporter(final LoadedSnapshot snapshot) {
+        return createSnapshotExporter(new SnapshotHandle() {
+            public LoadedSnapshot getSnapshot() { return snapshot; }
+        });
+    }
+    
+    public ExportUtils.Exportable createSnapshotExporter(final SnapshotHandle handle) {
+        return new ExportUtils.Exportable() {
+            public String getName() {
+                return Bundle.ResultsManager_ExportSnapshotData();
+            }
+
+            public boolean isEnabled() {
+                return true;
+            }
+
+            public ExportUtils.ExportProvider[] getProviders() {
+                final LoadedSnapshot snapshot = handle.getSnapshot();
+                return new ExportUtils.ExportProvider[] {
+                    new ExportUtils.AbstractNPSExportProvider(snapshot.getFile()) {
+                        protected void doExport(File targetFile) {
+                            exportSnapshot(snapshot, targetFile);
+                        }
+                    }
+                };
+            }
+        };
+    }
+    
+    private boolean exportSnapshot(LoadedSnapshot snapshot, File targetFile) {
+        File sourceFile = snapshot.getFile();
+        if (sourceFile != null) {
+            try {
+                Files.copy(sourceFile.toPath(), targetFile.toPath(), StandardCopyOption.REPLACE_EXISTING,
+                                                                     StandardCopyOption.COPY_ATTRIBUTES);
+                return true;
+            } catch (Throwable t) {
+                LOGGER.log(Level.INFO, t.getMessage(), t);
+                String msg = t.getLocalizedMessage().replace("<", "&lt;").replace(">", "&gt;"); // NOI18N
+                ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotExportFailedMsg(msg));
+                return false;
+            }
+        } else {
+            FileLock lock = null;
+            FileObject target = null;
+            DataOutputStream dos = null;
+            
+            try {
+                targetFile = FileUtil.normalizeFile(targetFile);
+                target = targetFile.isFile() ? FileUtil.toFileObject(targetFile) :
+                                               FileUtil.createData(targetFile);
+            
+                lock = target.lock();
+
+                OutputStream os = target.getOutputStream(lock);
+                BufferedOutputStream bos = new BufferedOutputStream(os);
+                dos = new DataOutputStream(bos);
+                //      System.out.println("Saving snapshot [" + snapshot.getSnapshot().getTimeTaken() + "]");
+                snapshot.save(dos);
+                dos.close();
+            } catch (IOException e) {
+                try {
+                    if (dos != null) dos.close();
+                    if (lock != null) target.delete(lock);
+                } catch (Exception e2) {}
+
+                ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotSaveFailedMsg(e.getMessage()));
+
+                return false; // failure => we wont continue with firing the event
+            } catch (OutOfMemoryError e) {
+                try {
+                    if (dos != null) dos.close();
+                    if (lock != null) target.delete(lock);
+                } catch (Exception e2) {}
+
+                ProfilerDialogs.displayError(Bundle.ResultsManager_OutOfMemorySavingMsg());
+
+                return false; // failure => we wont continue with firing the event
+            } finally {
+                if (lock != null) lock.releaseLock();
+            }
+
+            return true;
+        }
+    }
 
     public boolean saveSnapshot(LoadedSnapshot snapshot, FileObject profFile) {
         FileLock lock = null;
@@ -943,7 +1095,7 @@ public final class ResultsManager {
         FileObject saveDir = null;
 
         try {
-            saveDir = ProjectStorage.getSettingsFolder(p, true);
+            saveDir = ProfilerStorage.getProjectFolder(p, true);
         } catch (IOException e) {
             LOGGER.log(Level.SEVERE, Bundle.ResultsManager_CantFindSnapshotLocationMsg(e.getMessage()), e);
 
@@ -1105,6 +1257,7 @@ public final class ResultsManager {
         chooser.setMultiSelectionEnabled(false);
         chooser.setDialogTitle(Bundle.ResultsManager_SelectFileOrDirDialogCaption());
         chooser.setApproveButtonText(Bundle.ResultsManager_SaveButtonName());
+        chooser.setAcceptAllFileFilterUsed(false);
         chooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                 public boolean accept(File f) {
                     return f.isDirectory() || f.getName().endsWith("." + (heapdump ? HEAPDUMP_EXTENSION : SNAPSHOT_EXTENSION)); //NOI18N
@@ -1145,8 +1298,12 @@ public final class ResultsManager {
 
                 // extension will be used from source file
             } else { // extension exists
-                targetName = fName.substring(0, idx);
-                targetExt = fName.substring(idx + 1);
+                if (heapdump || fName.endsWith("." + targetExt)) { // NOI18N
+                    targetName = fName.substring(0, idx);
+                    targetExt = fName.substring(idx + 1);
+                } else {
+                    targetName = fName;
+                }
             }
         }
 
@@ -1182,8 +1339,10 @@ public final class ResultsManager {
         if (checkFileExists(new SelectedFile(targetFolder, fileName, fileExt))) {
             try {
                 FileUtil.copyFile(selectedSnapshot, targetFolder, fileName, fileExt);
-            } catch (IOException e) {
-                LOGGER.log(Level.SEVERE, Bundle.ResultsManager_SnapshotExportFailedMsg(e.getMessage()), e);
+            } catch (Throwable t) {
+                LOGGER.log(Level.INFO, t.getMessage(), t);
+                String msg = t.getLocalizedMessage().replace("<", "&lt;").replace(">", "&gt;"); // NOI18N
+                ProfilerDialogs.displayError(Bundle.ResultsManager_SnapshotExportFailedMsg(msg));
             }
         }
     }
@@ -1208,7 +1367,7 @@ public final class ResultsManager {
     }
 
     private Lookup.Provider findProjectForSnapshot(FileObject selectedFile) {
-        return ProjectStorage.getProjectFromSettingsFolder(selectedFile.getParent());
+        return ProfilerStorage.getProjectFromFolder(selectedFile.getParent());
     }
 
     private LoadedSnapshot loadSnapshotFromFileObject(FileObject selectedFile)
