@@ -36,6 +36,7 @@ import com.sun.tools.visualvm.core.datasupport.Stateful;
 import com.sun.tools.visualvm.core.ui.DataSourceView;
 import com.sun.tools.visualvm.core.ui.DataSourceWindowManager;
 import com.sun.tools.visualvm.host.Host;
+import com.sun.tools.visualvm.profiling.presets.ProfilerPreset;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
@@ -46,7 +47,6 @@ import org.netbeans.lib.profiler.common.ProfilingSettings;
 import org.netbeans.lib.profiler.common.SessionSettings;
 import org.netbeans.lib.profiler.global.Platform;
 import org.netbeans.modules.profiler.NetBeansProfiler;
-import org.netbeans.modules.profiler.ProfilerControlPanel2;
 import org.netbeans.modules.profiler.api.ProfilerIDESettings;
 import org.openide.DialogDisplayer;
 import org.openide.NotifyDescriptor;
@@ -65,17 +65,23 @@ public final class ProfilerSupport {
             Boolean.getBoolean("com.sun.tools.visualvm.profiler.SupportAllVMs");   // NOI18N
     private static final String HOTSPOT_VM_NAME_PREFIX = "Java HotSpot";    // NOI18N
     private static final String OPENJDK_VM_NAME_PREFIX = "OpenJDK ";    // NOI18N
+    private static final String SAPJDK_VM_NAME_PREFIX = "SAP Java ";    // NOI18N
     private static final String SUN_VM_VENDOR_PREFIX = "Sun ";  // NOI18N
     private static final String ORACLE_VM_VENDOR_PREFIX = "Oracle ";  // NOI18N
     private static final String APPLE_VM_VENDOR_PREFIX = "Apple ";  // NOI18N
     private static final String HP_VM_VENDOR_PREFIX = "\"Hewlett-Packard "; // NOI18N
     private static final String AZUL_VM_VENDOR_PREFIX = "Azul ";  // NOI18N
+    private static final String SAP_VM_VENDOR_PREFIX = "SAP AG";  // NOI18N
+    private static final String GRAAL_VM_VENDOR_PREFIX = "GraalVM ";  // NOI18N
+    private static final String ORACLE1_VM_VENDOR_PREFIX = "\"Oracle ";  // NOI18N
     
     private static final String JAVA_RT_16_PREFIX = "1.6.0";  // NOI18N
     private static final String JAVA_RT_17_PREFIX = "1.7.0";  // NOI18N
     private static final String JAVA_RT_18_PREFIX = "1.8.0";  // NOI18N
     private static final String JAVA_RT_19_PREFIX = "1.9.0";  // NOI18N
     private static final String JAVA_RT_9_PREFIX = "9";  // NOI18N
+    private static final String JAVA_RT_100_PREFIX = "10";  // NOI18N
+    private static final String JAVA_RT_110_PREFIX = "11";  // NOI18N
     
     private static ProfilerSupport instance;
     
@@ -125,6 +131,10 @@ public final class ProfilerSupport {
             String code = "jdk1" + (5 + i); // NOI18N
             if (supportsProfiling(code, 32) || supportsProfiling(code, 64)) codesl.add(code);
         }
+        String code = "jdk100"; // NOI18N
+        if (supportsProfiling(code, 32) || supportsProfiling(code, 64)) codesl.add(code);
+        code = "jdk110"; // NOI18N
+        if (supportsProfiling(code, 32) || supportsProfiling(code, 64)) codesl.add(code);
         
         String[] names = new String[codesl.size()];
         String[] codes = new String[codesl.size()];
@@ -167,6 +177,10 @@ public final class ProfilerSupport {
             return NbBundle.getMessage(ProfilerSupport.class, "STR_Java_platform_name", 8); // NOI18N
         if (Platform.JDK_19_STRING.equals(code))
             return NbBundle.getMessage(ProfilerSupport.class, "STR_Java_platform_name", 9); // NOI18N
+        if (Platform.JDK_100_STRING.equals(code))
+            return NbBundle.getMessage(ProfilerSupport.class, "STR_Java_platform_name", 10); // NOI18N
+        if (Platform.JDK_110_STRING.equals(code))
+            return NbBundle.getMessage(ProfilerSupport.class, "STR_Java_platform_name", 11); // NOI18N
         throw new IllegalArgumentException("Unknown java code " + code); // NOI18N
     }
     
@@ -193,53 +207,50 @@ public final class ProfilerSupport {
     }
     
     public void profileProcessStartup(final String java, final int architecture, final int port,
-                                      final CPUSettingsSupport cpuSettings,
-                                      final MemorySettingsSupport memorySettings,
-                                      final boolean isCPUProfiling) {
+                                      ProfilerSettingsSupport settings, final ProfilerPreset preset) {
         
         if (!CalibrationSupport.checkCalibration(java, architecture, null, null)) return;
         
+        final ProfilingSettings pSettings = settings.getSettings();
+        
         SwingUtilities.invokeLater(new Runnable() {
+            public void run() {                
+        // Perform the actual attach
+        RequestProcessor.getDefault().post(new Runnable() {
             public void run() {
-                // prevent deadlock, similar to issue #570
-                ProfilerControlPanel2.getDefault();
-                
-                // Perform the actual attach
-                RequestProcessor.getDefault().post(new Runnable() {
-                    public void run() {
-                        if (!checkStartedApp(port)) return;
-                        
-                        final RequestProcessor processor = new RequestProcessor("Startup Profiler @ " + port); // NOI18N
-                        Host.LOCALHOST.getRepository().addDataChangeListener(
-                            new DataChangeListener<Application>() {
-                                public void dataChanged(final DataChangeEvent<Application> event) {
-                                    final DataChangeListener listener = this;
-                                    processor.post(new Runnable() {
-                                        public void run() {
-                                            if (!event.getAdded().equals(event.getCurrent())) // filter-out initial sync event
-                                                for (Application a : event.getAdded()) {
-                                                    if (isProfiledApplication(a, port)) {
-                                                        Host.LOCALHOST.getRepository().removeDataChangeListener(listener);
+                if (!checkStartedApp(port)) return;
 
-                                                        setProfiledApplication(a);
-                                                        selectProfilerView(a, cpuSettings, memorySettings);
+                final RequestProcessor processor = new RequestProcessor("Startup Profiler @ " + port); // NOI18N
+                Host.LOCALHOST.getRepository().addDataChangeListener(
+                    new DataChangeListener<Application>() {
+                        public void dataChanged(final DataChangeEvent<Application> event) {
+                            final DataChangeListener listener = this;
+                            processor.post(new Runnable() {
+                                public void run() {
+                                    if (!event.getAdded().equals(event.getCurrent())) // filter-out initial sync event
+                                        for (Application a : event.getAdded()) {
+                                            if (isProfiledApplication(a, port)) {
+                                                Host.LOCALHOST.getRepository().removeDataChangeListener(listener);
 
-                                                        break;
-                                                    }
-                                                }
+                                                setProfiledApplication(a);
+                                                selectProfilerView(a, preset, pSettings);
+
+                                                break;
+                                            }
                                         }
-                                    });
                                 }
-                            }, Application.class);
+                            });
+                        }
+                    }, Application.class);
 
-                        ProfilingSettings ps = createProfilingSettings(cpuSettings, memorySettings, isCPUProfiling);
-                        SessionSettings ss = createSessionSettings(java, architecture, port);
-                        NetBeansProfiler.getDefaultNB().connectToStartedApp(ps, ss);
-                        resetTerminateDialogs();
-                    }
-                });
+                ProfilingSettings ps = pSettings;
+                SessionSettings ss = createSessionSettings(java, architecture, port);
+                NetBeansProfiler.getDefaultNB().connectToStartedApp(ps, ss);
+                resetTerminateDialogs();
             }
         });
+        }
+        });
     }
     
     private static boolean checkStartedApp(int port) {
@@ -263,12 +274,6 @@ public final class ProfilerSupport {
         if ("NO_OPTION".equals(dnsa)) ProfilerIDESettings.getInstance().setDoNotShowAgain(dnsaKey, null); // NOI18N        }
     }
     
-    private static ProfilingSettings createProfilingSettings(CPUSettingsSupport cpuSettings,
-            MemorySettingsSupport memorySettings, boolean isCPUProfiling) {
-        
-        return isCPUProfiling ? cpuSettings.getSettings() : memorySettings.getSettings();
-    }
-    
     private static SessionSettings createSessionSettings(String java, int architecture, int port) {
         SessionSettings ss = new SessionSettings();
         ss.setJavaVersionString(java);
@@ -308,17 +313,26 @@ public final class ProfilerSupport {
         // User explicitly requests to profile any VM
         if (FORCE_PROFILING_SUPPORTED) return true;
         
+//////        //TODO
+//////        // temporary disabled until profiling is rewritten to NB 90
+//////        return false;
+
         // Profiled application needs to be running JDK 6.0 or 7.0 or 8.0 or 9.0
-        if (!jvm.is16() && !jvm.is17() && !jvm.is18() && !jvm.is19()) return false;
+        // or 10 or 11
+        if (jvm.is14() || jvm.is15()) return false;
         
         String vmName = jvm.getVmName();
         String vmVendor = jvm.getVmVendor();
         
-        // VM has to be a HotSpot VM or OpenJDK by Sun Microsystems Inc. or Oracle Co. or Apple Inc. or Hewlett-Packard Co. or Azul Systems, Inc.
-        return vmName != null && (vmName.startsWith(HOTSPOT_VM_NAME_PREFIX) || vmName.startsWith(OPENJDK_VM_NAME_PREFIX)) && 
-               vmVendor != null && (vmVendor.startsWith(ORACLE_VM_VENDOR_PREFIX) || vmVendor.startsWith(SUN_VM_VENDOR_PREFIX) 
+        // VM has to be a HotSpot VM or OpenJDK by Sun Microsystems Inc. or 
+        // Oracle Co. or Apple Inc. or Hewlett-Packard Co. or 
+        // Azul Systems, Inc. or SAP AG or Graal VM
+        return vmName != null && (vmName.startsWith(HOTSPOT_VM_NAME_PREFIX) || vmName.startsWith(OPENJDK_VM_NAME_PREFIX)
+                               || vmName.startsWith(SAPJDK_VM_NAME_PREFIX) || vmName.startsWith(GRAAL_VM_VENDOR_PREFIX)) &&
+               vmVendor != null && (vmVendor.startsWith(ORACLE_VM_VENDOR_PREFIX) || vmVendor.startsWith(SUN_VM_VENDOR_PREFIX)
                                  || vmVendor.startsWith(APPLE_VM_VENDOR_PREFIX) || vmVendor.startsWith(HP_VM_VENDOR_PREFIX)
-                                 || vmVendor.startsWith(AZUL_VM_VENDOR_PREFIX));
+                                 || vmVendor.startsWith(AZUL_VM_VENDOR_PREFIX) || vmVendor.startsWith(SAP_VM_VENDOR_PREFIX)
+                                 || vmVendor.startsWith(ORACLE1_VM_VENDOR_PREFIX));
     }
     
     static boolean classSharingBreaksProfiling(Application application) {
@@ -359,6 +373,10 @@ public final class ProfilerSupport {
                 return false;
             } else if (javaRTVersion.startsWith(JAVA_RT_9_PREFIX)) {
                 return false;
+            } else if (javaRTVersion.startsWith(JAVA_RT_100_PREFIX)) {
+                return false;
+            } else if (javaRTVersion.startsWith(JAVA_RT_110_PREFIX)) {
+                return false;
             }
         // OpenJDK -------------------------------------------------------------
         } else if(vmName.startsWith(OPENJDK_VM_NAME_PREFIX)) {
@@ -384,6 +402,10 @@ public final class ProfilerSupport {
                 return false;
             } else if (javaRTVersion.startsWith(JAVA_RT_9_PREFIX)) {
                 return false;
+            } else if (javaRTVersion.startsWith(JAVA_RT_100_PREFIX)) {
+                return false;
+            } else if (javaRTVersion.startsWith(JAVA_RT_110_PREFIX)) {
+                return false;
             }
         }
         
@@ -427,13 +449,16 @@ public final class ProfilerSupport {
         selectProfilerView(application, null, null);
     }
     
-    private void selectProfilerView(Application application, final CPUSettingsSupport cpu, final MemorySettingsSupport memory) {
+    private void selectProfilerView(Application application, final ProfilerPreset preset, final ProfilingSettings settings) {
         if (application == null) return;
+        
         final DataSourceView activeView = profilerViewProvider.view(application);
         if (activeView == null) return;
+        
         SwingUtilities.invokeLater(new Runnable() {
             public void run() {
-                if (cpu != null || memory != null) ((ApplicationProfilerView)activeView).copySettings(cpu, memory);
+                if (preset != null && settings != null)
+                    ((ApplicationProfilerView)activeView).selectPreset(preset, settings);
                 DataSourceWindowManager.sharedInstance().selectView(activeView);
             }
         });
@@ -441,11 +466,17 @@ public final class ProfilerSupport {
     
     
     private ProfilerSupport() {
-        profilerViewProvider = new ApplicationProfilerViewProvider();
-        profilerViewProvider.initialize();
+        isInitialized = NetBeansProfiler.isInitialized();
+        
+        if (isInitialized) {
+            profilerViewProvider = new ApplicationProfilerViewProvider();
+            profilerViewProvider.initialize();
 
-        ProfilerIDESettings.getInstance().setAutoOpenSnapshot(false);
-        ProfilerIDESettings.getInstance().setAutoSaveSnapshot(true);
+            ProfilerIDESettings.getInstance().setAutoOpenSnapshot(false);
+            ProfilerIDESettings.getInstance().setAutoSaveSnapshot(true);
+        } else {
+            profilerViewProvider = null;
+        }
     }
 
 }
